@@ -8,6 +8,43 @@ managing them.
 
 from __future__ import absolute_import
 
+import sys
+import os
+
+
+def _is_astropy_source(path=None):
+    """
+    Returns whether the source for this module is directly in an astropy
+    source distribution or checkout.
+    """
+
+    # If this __init__.py file is in ./astropy/ then import is within a source
+    # dir .astropy-root is a file distributed with the source, but that should
+    # not installed
+    if path is None:
+        path = os.path.join(os.path.dirname(__file__), os.pardir)
+    elif os.path.isfile(path):
+        path = os.path.dirname(path)
+
+    source_dir = os.path.abspath(path)
+    return os.path.exists(os.path.join(source_dir, '.astropy-root'))
+
+
+def _is_astropy_setup():
+    """
+    Returns whether we are currently being imported in the context of running
+    Astropy's setup.py.
+    """
+
+    main_mod = sys.modules.get('__main__')
+    if not main_mod:
+        return False
+
+    return (getattr(main_mod, '__file__', False) and
+            os.path.basename(main_mod.__file__).rstrip('co') == 'setup.py' and
+            _is_astropy_source(main_mod.__file__))
+
+
 # this indicates whether or not we are in astropy's setup.py
 try:
     _ASTROPY_SETUP_
@@ -17,9 +54,11 @@ except NameError:
         import builtins
     else:
         import __builtin__ as builtins
-    builtins._ASTROPY_SETUP_ = False
-    del version_info
-    del builtins
+
+    # This will set the _ASTROPY_SETUP_ to True by default if
+    # we are running Astropy's setup.py
+    builtins._ASTROPY_SETUP_ = _is_astropy_setup()
+
 
 try:
     from .version import version as __version__
@@ -74,7 +113,6 @@ if not _ASTROPY_SETUP_:
 
 
 from . import config as _config
-import sys
 
 
 class Conf(_config.ConfigNamespace):
@@ -113,96 +151,9 @@ UNICODE_OUTPUT = _config.ConfigAlias(
     '0.4', 'UNICODE_OUTPUT', 'unicode_output')
 
 
-del sys
-
-
-# set up the test command
-def _get_test_runner():
-    from .tests.helper import TestRunner
-    return TestRunner(__path__[0])
-
-
-def test(package=None, test_path=None, args=None, plugins=None,
-         verbose=False, pastebin=None, remote_data=False, pep8=False,
-         pdb=False, open_files=False, parallel=0, docs_path=None,
-         skip_docs=False, repeat=None):
-    """
-    Run Astropy tests using py.test. A proper set of arguments is
-    constructed and passed to `pytest.main`.
-
-    Parameters
-    ----------
-    package : str, optional
-        The name of a specific package to test, e.g. 'io.fits' or 'utils'.
-        If nothing is specified all default Astropy tests are run.
-
-    test_path : str, optional
-        Specify location to test by path. May be a single file or
-        directory. Must be specified absolutely or relative to the
-        calling directory.
-
-    args : str, optional
-        Additional arguments to be passed to `pytest.main` in the `args`
-        keyword argument.
-
-    plugins : list, optional
-        Plugins to be passed to `pytest.main` in the `plugins` keyword
-        argument.
-
-    verbose : bool, optional
-        Convenience option to turn on verbose output from py.test. Passing
-        True is the same as specifying `-v` in `args`.
-
-    pastebin : {'failed','all',None}, optional
-        Convenience option for turning on py.test pastebin output. Set to
-        'failed' to upload info for failed tests, or 'all' to upload info
-        for all tests.
-
-    remote_data : bool, optional
-        Controls whether to run tests marked with @remote_data. These
-        tests use online data and are not run by default. Set to True to
-        run these tests.
-
-    pep8 : bool, optional
-        Turn on PEP8 checking via the pytest-pep8 plugin and disable normal
-        tests. Same as specifying `--pep8 -k pep8` in `args`.
-
-    pdb : bool, optional
-        Turn on PDB post-mortem analysis for failing tests. Same as
-        specifying `--pdb` in `args`.
-
-    open_files : bool, optional
-        Fail when any tests leave files open.  Off by default, because
-        this adds extra run time to the test suite.  Requires the
-        ``psutil`` package.
-
-    parallel : int, optional
-        When provided, run the tests in parallel on the specified
-        number of CPUs.  If parallel is negative, it will use the all
-        the cores on the machine.  Requires the `pytest-xdist` plugin.
-
-    docs_path : str, optional
-        The path to the documentation .rst files.
-
-    skip_docs : bool, optional
-        When `True`, skips running the doctests in the .rst files.
-
-    repeat : int, optional
-        If set, specifies how many times each test should be run. This is
-        useful for diagnosing sporadic failures.
-
-    See Also
-    --------
-    pytest.main : py.test function wrapped by `run_tests`.
-
-    """
-    test_runner = _get_test_runner()
-    return test_runner.run_tests(
-        package=package, test_path=test_path, args=args,
-        plugins=plugins, verbose=verbose, pastebin=pastebin,
-        remote_data=remote_data, pep8=pep8, pdb=pdb,
-        open_files=open_files, parallel=parallel, docs_path=docs_path,
-        skip_docs=skip_docs, repeat=repeat)
+# Create the test() function
+from .tests.runner import TestRunner
+test = TestRunner.make_test_runner_in(__path__[0])
 
 
 # if we are *not* in setup mode, import the logger and possibly populate the
@@ -210,15 +161,12 @@ def test(package=None, test_path=None, args=None, plugins=None,
 def _initialize_astropy():
     from . import config
 
-    import os
-    import sys
     from warnings import warn
+    from .utils.exceptions import AstropyDeprecationWarning
 
-    # If this __init__.py file is in ./astropy/ then import is within a source dir
-    source_dir = os.path.abspath(os.path.dirname(__file__))
-    is_astropy_source_dir = (
-            os.path.exists(os.path.join(source_dir, os.pardir, '.git')) and
-            os.path.isfile(os.path.join(source_dir, os.pardir, 'setup.py')))
+    if sys.version_info[:2] < (2, 7):
+        warn("Python 2.6 will no longer be supported from Astropy v1.2.0 and "
+             "above", AstropyDeprecationWarning)
 
     def _rollback_import(message):
         log.error(message)
@@ -237,7 +185,7 @@ def _initialize_astropy():
     try:
         from .utils import _compiler
     except ImportError:
-        if is_astropy_source_dir:
+        if _is_astropy_source():
             log.warn('You appear to be trying to import astropy from '
                      'within a source checkout without building the '
                      'extension modules first.  Attempting to (re)build '
@@ -269,9 +217,10 @@ def _initialize_astropy():
 
 
 def _rebuild_extensions():
-    import os
+    global __version__
+    global __githash__
+
     import subprocess
-    import sys
     import time
 
     from .utils.console import Spinner
@@ -290,15 +239,29 @@ def _rebuild_extensions():
                 time.sleep(0.05)
     finally:
         os.chdir(old_cwd)
+        devnull.close()
 
     if sp.returncode != 0:
         raise OSError('Running setup.py build_ext --inplace failed '
                       'with error code {0}: try rerunning this command '
                       'manually to check what the error was.'.format(
                           sp.returncode))
+
+    # Try re-loading module-level globals from the astropy.version module,
+    # which may not have existed before this function ran
+    try:
+        from .version import version as __version__
+    except ImportError:
+        pass
+
+    try:
+        from .version import githash as __githash__
+    except ImportError:
+        pass
+
+
 # Set the bibtex entry to the article referenced in CITATION
 def _get_bibtex():
-    import os
     import re
     if os.path.exists('CITATION'):
         with open('CITATION', 'r') as citation:
@@ -325,3 +288,54 @@ if not _ASTROPY_SETUP_:
     _initialize_astropy()
 
     from .utils.misc import find_api_page
+
+
+def online_help(query):
+    """
+    Search the online Astropy documentation for the given query.
+    Opens the results in the default web browser.  Requires an active
+    Internet connection.
+
+    Parameters
+    ----------
+    query : str
+        The search query.
+    """
+    from .extern.six.moves.urllib.parse import urlencode
+    import webbrowser
+
+    version = __version__
+    if 'dev' in version:
+        version = 'latest'
+    else:
+        version = 'v' + version
+
+    url = 'http://docs.astropy.org/en/{0}/search.html?{1}'.format(
+        version, urlencode({'q': query}))
+
+    webbrowser.open(url)
+
+
+__dir__ = ['__version__', '__githash__', '__minimum_numpy_version__',
+           '__bibtex__', 'test', 'log', 'find_api_page', 'online_help',
+           'online_docs_root', 'conf', 'UNICODE_OUTPUT']
+
+
+from types import ModuleType as __module_type__
+# Clean up top-level namespace--delete everything that isn't in __dir__
+# or is a magic attribute, and that isn't a submodule of this package
+for varname in dir():
+    if not ((varname.startswith('__') and varname.endswith('__')) or
+            varname in __dir__ or
+            (varname[0] != '_' and
+                isinstance(locals()[varname], __module_type__) and
+                locals()[varname].__name__.startswith(__name__ + '.'))):
+        # The last clause in the the above disjunction deserves explanation:
+        # When using relative imports like ``from .. import config``, the
+        # ``config`` variable is automatically created in the namespace of
+        # whatever module ``..`` resolves to (in this case astropy).  This
+        # happens a few times just in the module setup above.  This allows
+        # the cleanup to keep any public submodules of the astropy package
+        del locals()[varname]
+
+del varname, __module_type__
