@@ -26,6 +26,7 @@ from warnings import warn
 from ..extern.configobj import configobj, validate
 from ..utils.exceptions import AstropyWarning, AstropyDeprecationWarning
 from ..utils import find_current_module
+from ..utils.introspection import resolve_name
 from ..utils.misc import InheritDocstrings
 from .paths import get_config_dir
 
@@ -219,6 +220,11 @@ class ConfigItem(object):
     # this is used to make validation faster so a Validator object doesn't
     # have to be created every time
     _validator = validate.Validator()
+    cfgtype = None
+    """
+    A type specifier like those used as the *values* of a particular key in a
+    ``configspec`` file of ``configobj``.
+    """
 
     def __init__(self, defaultvalue='', description=None, cfgtype=None,
                  module=None, aliases=None):
@@ -276,7 +282,8 @@ class ConfigItem(object):
         return self()
 
     def set(self, value):
-        """ Sets the current value of this `ConfigItem`.
+        """
+        Sets the current value of this ``ConfigItem``.
 
         This also updates the comments that give the description and type
         information.
@@ -289,7 +296,7 @@ class ConfigItem(object):
         Raises
         ------
         TypeError
-            If the provided ``value`` is not valid for this `ConfigItem`.
+            If the provided ``value`` is not valid for this ``ConfigItem``.
         """
         try:
             value = self._validate_val(value)
@@ -307,12 +314,14 @@ class ConfigItem(object):
         Sets this item to a specified value only inside a with block.
 
         Use as::
+
             ITEM = ConfigItem('ITEM', 'default', 'description')
 
             with ITEM.set_temp('newval'):
-                ... do something that wants ITEM's value to be 'newval' ...
+                #... do something that wants ITEM's value to be 'newval' ...
+                print(ITEM)
 
-             # ITEM is now 'default' after the with block
+            # ITEM is now 'default' after the with block
 
         Parameters
         ----------
@@ -328,7 +337,7 @@ class ConfigItem(object):
             self.set(initval)
 
     def reload(self):
-        """ Reloads the value of this `ConfigItem` from the relevant
+        """ Reloads the value of this ``ConfigItem`` from the relevant
         configuration file.
 
         Returns
@@ -373,12 +382,12 @@ class ConfigItem(object):
         return out
 
     def __call__(self):
-        """ Returns the value of this `ConfigItem`
+        """ Returns the value of this ``ConfigItem``
 
         Returns
         -------
         val
-            This item's value, with a type determined by the `cfgtype`
+            This item's value, with a type determined by the ``cfgtype``
             attribute.
 
         Raises
@@ -455,7 +464,7 @@ class ConfigurationItem(ConfigItem):
     """
     A backward-compatibility layer to support the old
     `ConfigurationItem` API.  The only difference between this and
-    `ConfigItem` is that this requires an explicit name to be set as
+    ``ConfigItem`` is that this requires an explicit name to be set as
     the first argument.
     """
     # REMOVE in astropy 0.5
@@ -564,11 +573,7 @@ class ConfigAlias(ConfigItem):
             AstropyDeprecationWarning)
 
     def _get_target(self):
-        if self._new_module not in sys.modules:
-            __import__(self._new_module)
-        mod = sys.modules[self._new_module]
-        cfg = getattr(mod, 'conf')
-        return cfg
+        return resolve_name(self._new_module, 'conf')
 
     def set(self, value):
         self._deprecation_warning()
@@ -840,8 +845,7 @@ def update_default_config(pkg, default_cfg_dir_or_fn, version=None):
             identical = False
 
     if version is None:
-        mod = __import__(pkg)
-        version = mod.__version__
+        version = resolve_name(pkg, '__version__')
 
     # Don't install template files for dev versions, or we'll end up
     # spamming `~/.astropy/config`.
