@@ -5,7 +5,7 @@ from ...extern.six.moves import cPickle
 from ... import convolution as conv
 from ...tests.helper import pytest, pickle_protocol, check_pickling_recovery
 
-@pytest.mark.parametrize("name,args,kwargs,xfail",
+@pytest.mark.parametrize(("name","args","kwargs","xfail"),
                          [(conv.CustomKernel, [],
                            {'array':np.random.rand(15)},
                            False),
