@@ -22,13 +22,12 @@ from ..utils.compat.misc import override__dir__
 from ..extern import six
 from ..utils.exceptions import AstropyDeprecationWarning, AstropyWarning
 from .. import units as u
-from ..utils import OrderedDict
+from ..utils import OrderedDict, OrderedDescriptor, OrderedDescriptorContainer
 from .transformations import TransformGraph
 from .representation import (BaseRepresentation, CartesianRepresentation,
                              SphericalRepresentation,
                              UnitSphericalRepresentation,
                              REPRESENTATION_CLASSES)
-from .earth import EarthLocation
 
 
 __all__ = ['BaseCoordinateFrame', 'frame_transform_graph', 'GenericFrame',
@@ -58,7 +57,7 @@ def _get_repr_cls(value):
     return value
 
 
-class FrameMeta(type):
+class FrameMeta(OrderedDescriptorContainer):
     def __new__(mcls, name, bases, members):
         if 'default_representation' in members:
             default_repr = members.pop('default_representation')
@@ -123,7 +122,7 @@ class FrameMeta(type):
         members[attr] = property(getter)
 
 
-class FrameAttribute(object):
+class FrameAttribute(OrderedDescriptor):
     """A non-mutable data descriptor to hold a frame attribute.
 
     This class must be used to define frame attributes (e.g. ``equinox`` or
@@ -158,20 +157,14 @@ class FrameAttribute(object):
         ``default is None`` and no value was supplied during initialization.
     """
 
-    _nextid = 1
-    """
-    Used to ascribe some ordering to FrameAttribute instances so that the
-    order they were assigned in a class body can be determined.
-    """
+    _class_attribute_ = 'frame_attributes'
+    _name_attribute_ = 'name'
+    name = '<unbound>'
 
     def __init__(self, default=None, secondary_attribute=''):
         self.default = default
         self.secondary_attribute = secondary_attribute
-
-        # Use FrameAttribute._nextid explicitly so that subclasses of
-        # FrameAttribute use the same counter
-        self._order = FrameAttribute._nextid
-        FrameAttribute._nextid += 1
+        super(FrameAttribute, self).__init__()
 
     def convert_input(self, value):
         """
@@ -207,25 +200,6 @@ class FrameAttribute(object):
         return value, False
 
     def __get__(self, instance, frame_cls=None):
-        if not hasattr(self, 'name'):
-            # Find attribute name of self by finding this object in the frame
-            # class which is requesting this attribute or any of its
-            # superclasses.
-            for mro_cls in frame_cls.__mro__:
-                for name, val in mro_cls.__dict__.items():
-                    if val is self:
-                        self.name = name
-                        break
-                if hasattr(self, 'name'):  # Can't nicely break out of two loops
-                    break
-            else:
-                # Cannot think of a way to actually raise this exception.  This
-                # instance containing this code must be in the class dict in
-                # order to get excecuted by attribute access.  But leave this
-                # here just in case...
-                raise AttributeError(
-                        'Unexpected inability to locate descriptor')
-
         out = None
 
         if instance is not None:
@@ -470,6 +444,11 @@ class BaseCoordinateFrame(object):
     # specifies special names/units for representation attributes
     frame_specific_representation_info = {}
 
+    _inherit_descriptors_ = (FrameAttribute,)
+
+    frame_attributes = OrderedDict()
+    # Default empty frame_attributes dict
+
     # This __new__ provides for backward-compatibility with pre-0.4 API.
     # TODO: remove in 1.0
     def __new__(cls, *args, **kwargs):
@@ -482,24 +461,21 @@ class BaseCoordinateFrame(object):
 
         use_skycoord = False
 
-        if (len(args) > 1 or (len(args) == 1 and
-                not isinstance(args[0], BaseRepresentation))):
-            for arg in args:
-                if (not isinstance(arg, u.Quantity)
-                    and not isinstance(arg, BaseRepresentation)):
-                    msg = ('Initializing frame classes like "{0}" using string '
-                           'or other non-Quantity arguments is deprecated, and '
-                           'will be removed in the next version of Astropy.  '
-                           'Instead, you probably want to use the SkyCoord '
-                           'class with the "system={1}" keyword, or if you '
-                           'really want to use the low-level frame classes, '
-                           'create it with an Angle or Quantity.')
+        for arg in args:
+            if not isinstance(arg, (u.Quantity, BaseRepresentation)):
+                msg = ('Initializing frame classes like "{0}" using string '
+                       'or other non-Quantity arguments is deprecated, and '
+                       'will be removed in the next version of Astropy.  '
+                       'Instead, you probably want to use the SkyCoord '
+                       'class with the "frame={1}" keyword, or if you '
+                       'really want to use the low-level frame classes, '
+                       'create it with an Angle or Quantity.')
 
-                    warnings.warn(msg.format(cls.__name__,
-                                             cls.__name__.lower()),
-                                  AstropyDeprecationWarning)
-                    use_skycoord = True
-                    break
+                warnings.warn(msg.format(cls.__name__,
+                                         cls.__name__.lower()),
+                              AstropyDeprecationWarning)
+                use_skycoord = True
+                break
 
         if 'unit' in kwargs and not use_skycoord:
             warnings.warn(
@@ -584,7 +560,7 @@ class BaseCoordinateFrame(object):
                     del repr_kwargs['distance']
                 if (issubclass(self.representation, SphericalRepresentation) and
                         'distance' not in repr_kwargs):
-                    representation_data = UnitSphericalRepresentation(**repr_kwargs)
+                    representation_data = self.representation._unit_representation(**repr_kwargs)
                 else:
                     representation_data = self.representation(**repr_kwargs)
 
@@ -643,20 +619,8 @@ class BaseCoordinateFrame(object):
 
     @classmethod
     def get_frame_attr_names(cls):
-        seen = set()
-        attributes = []
-        for mro_cls in cls.__mro__:
-            for name, val in mro_cls.__dict__.items():
-                if isinstance(val, FrameAttribute) and name not in seen:
-                    seen.add(name)
-                    # Add the sort order, name, and actual value of the frame
-                    # attribute in question
-                    attributes.append((val._order, name,
-                                       getattr(mro_cls, name)))
-
-        # Sort by the frame attribute order
-        attributes.sort(key=lambda a: a[0])
-        return OrderedDict((a[1], a[2]) for a in attributes)
+        return OrderedDict((name, getattr(cls, name))
+                           for name in cls.frame_attributes)
 
     @property
     def representation(self):
@@ -745,7 +709,6 @@ class BaseCoordinateFrame(object):
                 out[repr_name] = repr_unit
         return out
 
-
     def realize_frame(self, representation):
         """
         Generates a new frame *with new data* from another frame (which may or
@@ -975,52 +938,63 @@ class BaseCoordinateFrame(object):
             return False
 
     def __repr__(self):
-        frameattrs = ', '.join([attrnm + '=' + str(getattr(self, attrnm))
-                                for attrnm in self.get_frame_attr_names()])
+        frameattrs = self._frame_attrs_repr()
+        data_repr = self._data_repr()
 
-        if self.has_data:
-            if self.representation:
-                if (issubclass(self.representation, SphericalRepresentation) and
-                        isinstance(self.data, UnitSphericalRepresentation)):
-                    data = self.represent_as(self.data.__class__,
-                                             in_frame_units=True)
-                else:
-                    data = self.represent_as(self.representation,
-                                             in_frame_units=True)
+        if frameattrs:
+            frameattrs = ' ({0})'.format(frameattrs)
 
-                data_repr = repr(data)
-                for nmpref, nmrepr in self.representation_component_names.items():
-                    data_repr = data_repr.replace(nmrepr, nmpref)
-
-            else:
-                data = self.data
-                data_repr = repr(self.data)
-
-            if data_repr.startswith('<' + data.__class__.__name__):
-                # standard form from BaseRepresentation
-                if frameattrs:
-                    frameattrs = ' (' + frameattrs + ')'
-
-                #remove both the leading "<" and the space after the name
-                data_repr = data_repr[(len(data.__class__.__name__) + 2):]
-
-                return '<{0} Coordinate{1}: {2}'.format(self.__class__.__name__,
-                                                        frameattrs, data_repr)
-            else:
-                # should only happen if a representation has a non-standard
-                # __repr__ method, and we just punt to that
-                if frameattrs:
-                    frameattrs = ' (' + frameattrs + '), '
-                s = '<{0} Coordinate{1}Data:\n{2}>'
-                return s.format(self.__class__.__name__, frameattrs, data_repr)
+        if data_repr:
+            return '<{0} Coordinate{1}: {2}>'.format(self.__class__.__name__,
+                                                     frameattrs, data_repr)
         else:
-            if frameattrs:
-                frameattrs = ' (' + frameattrs + ')'
-            return '<{0} Frame{1}>'.format(self.__class__.__name__, frameattrs)
+            return '<{0} Frame{1}>'.format(self.__class__.__name__,
+                                            frameattrs)
+
+    def _data_repr(self):
+        """Returns a string representation of the coordinate data."""
+
+        if not self.has_data:
+            return ''
+
+        if self.representation:
+            if (issubclass(self.representation, SphericalRepresentation) and
+                    isinstance(self.data, UnitSphericalRepresentation)):
+                data = self.represent_as(self.data.__class__,
+                                         in_frame_units=True)
+            else:
+                data = self.represent_as(self.representation,
+                                         in_frame_units=True)
+
+            data_repr = repr(data)
+            for nmpref, nmrepr in self.representation_component_names.items():
+                data_repr = data_repr.replace(nmrepr, nmpref)
+
+        else:
+            data = self.data
+            data_repr = repr(self.data)
+
+        if data_repr.startswith('<' + data.__class__.__name__):
+            # remove both the leading "<" and the space after the name, as well
+            # as the trailing ">"
+            data_repr = data_repr[(len(data.__class__.__name__) + 2):-1]
+        else:
+            data_repr = 'Data:\n' + data_repr
+
+        return data_repr
+
+    def _frame_attrs_repr(self):
+        """
+        Returns a string representation of the frame's attributes, if any.
+        """
+        return ', '.join([attrnm + '=' + str(getattr(self, attrnm))
+                          for attrnm in self.get_frame_attr_names()])
 
     def __getitem__(self, view):
         if self.has_data:
-            return self.realize_frame(self.data[view])
+            out = self.realize_frame(self.data[view])
+            out.representation = self.representation
+            return out
         else:
             raise ValueError('Cannot index a frame with no data')
 
@@ -1060,10 +1034,10 @@ class BaseCoordinateFrame(object):
         return val
 
     def __setattr__(self, attr, value):
-        repr_attr_names = []
+        repr_attr_names = set()
         if hasattr(self, 'representation_info'):
             for representation_attr in self.representation_info.values():
-                repr_attr_names.extend(representation_attr['names'])
+                repr_attr_names.update(representation_attr['names'])
         if attr in repr_attr_names:
             raise AttributeError(
                 'Cannot set any frame attribute {0}'.format(attr))
@@ -1096,7 +1070,7 @@ class BaseCoordinateFrame(object):
         from .angles import Angle
 
         self_unit_sph = self.represent_as(UnitSphericalRepresentation)
-        other_transformed = other.transform_to(self.__class__)
+        other_transformed = other.transform_to(self)
         other_unit_sph = other_transformed.represent_as(UnitSphericalRepresentation)
 
         # Get the separation as a Quantity, convert to Angle in degrees
@@ -1127,14 +1101,14 @@ class BaseCoordinateFrame(object):
 
         from .distances import Distance
 
-        if self.data.__class__ == UnitSphericalRepresentation:
+        if issubclass(self.data.__class__, UnitSphericalRepresentation):
             raise ValueError('This object does not have a distance; cannot '
                              'compute 3d separation.')
 
         # do this first just in case the conversion somehow creates a distance
-        other_in_self_system = other.transform_to(self.__class__)
+        other_in_self_system = other.transform_to(self)
 
-        if other_in_self_system.__class__ == UnitSphericalRepresentation:
+        if issubclass(other_in_self_system.__class__, UnitSphericalRepresentation):
             raise ValueError('The other object does not have a distance; '
                              'cannot compute 3d separation.')
 
@@ -1179,18 +1153,17 @@ class GenericFrame(BaseCoordinateFrame):
         A dictionary of attributes to be used as the frame attributes for this
         frame.
     """
+
     name = None  # it's not a "real" frame so it doesn't have a name
 
     def __init__(self, frame_attrs):
-        super(GenericFrame, self).__setattr__('_frame_attr_names', frame_attrs)
+        self.frame_attributes = OrderedDict()
+        for name, default in frame_attrs.items():
+            self.frame_attributes[name] = FrameAttribute(default)
+            setattr(self, '_' + name, default)
+
         super(GenericFrame, self).__init__(None)
 
-        for attrnm, attrval in frame_attrs.items():
-            setattr(self, '_' + attrnm, attrval)
-
-    def get_frame_attr_names(self):
-        return self._frame_attr_names
-
     def __getattr__(self, name):
         if '_' + name in self.__dict__:
             return getattr(self, '_' + name)
@@ -1198,7 +1171,11 @@ class GenericFrame(BaseCoordinateFrame):
             raise AttributeError('no {0}'.format(name))
 
     def __setattr__(self, name, value):
-        if name in self._frame_attr_names:
+        if name in self.get_frame_attr_names():
             raise AttributeError("can't set frame attribute '{0}'".format(name))
         else:
             super(GenericFrame, self).__setattr__(name, value)
+
+# doing this import at the bottom prevents a circular import issue that is
+# otherwise present due to EarthLocation needing to import ITRS
+from .earth import EarthLocation
