@@ -16,9 +16,6 @@ class ITRS(BaseCoordinateFrame):
     defined by a series of reference locations near the surface of the Earth.
     For more background on the ITRS, see the references provided in the
     :ref:`astropy-coordinates-seealso` section of the documentation.
-
-    Note that this frame does *not* have an ``obstime``, because it is
-    rotating with the Earth.
     """
 
     default_representation = CartesianRepresentation
