@@ -13,6 +13,7 @@ from ..units import Unit, IrreducibleUnit
 from .. import units as u
 from ..wcs.utils import skycoord_to_pixel, pixel_to_skycoord
 from ..utils.exceptions import AstropyDeprecationWarning
+from ..utils.data_info import MixinInfo
 
 from .distances import Distance
 from .baseframe import BaseCoordinateFrame, frame_transform_graph, GenericFrame, _get_repr_cls
@@ -41,6 +42,43 @@ def FRAME_ATTR_NAMES_SET():
     return out
 
 
+class SkyCoordInfo(MixinInfo):
+    """
+    Container for meta information like name, description, format.  This is
+    required when the object is used as a mixin column within a table, but can
+    be used as a general way to store meta information.
+    """
+    attrs_from_parent = set(['unit'])  # Unit is read-only
+    _supports_indexing = False
+
+    @staticmethod
+    def default_format(val):
+        repr_data = val.info._repr_data
+        formats = ['{0.' + compname + '.value:}' for compname
+                   in repr_data.components]
+        return ','.join(formats).format(repr_data)
+
+    @property
+    def unit(self):
+        repr_data = self._repr_data
+        unit = ','.join(str(getattr(repr_data, comp).unit) or 'None'
+                        for comp in repr_data.components)
+        return unit
+
+    @property
+    def _repr_data(self):
+        if self._parent is None:
+            return None
+
+        sc = self._parent
+        if (issubclass(sc.representation, SphericalRepresentation) and
+                isinstance(sc.data, UnitSphericalRepresentation)):
+            repr_data = sc.represent_as(sc.data.__class__, in_frame_units=True)
+        else:
+            repr_data = sc.represent_as(sc.representation, in_frame_units=True)
+        return repr_data
+
+
 class SkyCoord(object):
     """High-level object providing a flexible interface for celestial coordinate
     representation, manipulation, and transformation between systems.
@@ -139,10 +177,10 @@ class SkyCoord(object):
             Cartesian coordinates values for the Galactic frame.
     """
 
-
     # Declare that SkyCoord can be used as a Table column by defining the
-    # attribute where column attributes will be stored.
-    _astropy_column_attrs = None
+    # info property.
+    info = SkyCoordInfo()
+
 
     def __init__(self, *args, **kwargs):
 
@@ -198,7 +236,15 @@ class SkyCoord(object):
             # First turn `self` into a mockup of the thing we want - we can copy
             # this to get all the right attributes
             self._sky_coord_frame = self_frame[item]
-            return SkyCoord(self)
+            out = SkyCoord(self, representation=self.representation)
+
+            # Copy other 'info' attr only if it has actually been defined.
+            # See PR #3898 for further explanation and justification, along
+            # with Quantity.__array_finalize__
+            if 'info' in self.__dict__:
+                out.info = self.info
+
+            return out
         finally:
             # now put back the right frame in self
             self._sky_coord_frame = self_frame
@@ -244,6 +290,10 @@ class SkyCoord(object):
                 # were explicitly specified in the coordinate object (i.e. non-default).
                 coord_kwargs = _parse_coordinate_arg(args[0], frame, units, kwargs)
 
+                # Copy other 'info' attr only if it has actually been defined.
+                if 'info' in getattr(args[0], '__dict__', ()):
+                    self.info = args[0].info
+
             elif len(args) <= 3:
                 frame_attr_names = frame.representation_component_names.keys()
                 repr_attr_names = frame.representation_component_names.values()
@@ -429,14 +479,15 @@ class SkyCoord(object):
     def __repr__(self):
         clsnm = self.__class__.__name__
         coonm = self.frame.__class__.__name__
+        frameattrs = self.frame._frame_attrs_repr()
+        if frameattrs:
+            frameattrs = ': ' + frameattrs
 
-        crepr = repr(self.frame)
-        frameattrs = ''
-        if crepr.find('):') != -1:
-            frameattrs = ': '+crepr[crepr.index('(')+1:crepr.index('):')]
+        data = self.frame._data_repr()
+        if data:
+            data = ': ' + data
 
-        s = '<{clsnm} ({coonm}{frameattrs})'.format(**locals())
-        return s + crepr[crepr.index(':'):]
+        return '<{clsnm} ({coonm}{frameattrs}){data}>'.format(**locals())
 
     def to_string(self, style='decimal', **kwargs):
         """
@@ -623,10 +674,10 @@ class SkyCoord(object):
             raise TypeError('Can only get separation to another SkyCoord or a '
                             'coordinate frame with data')
 
-        if self.data.__class__ == UnitSphericalRepresentation:
+        if issubclass(self.data.__class__, UnitSphericalRepresentation):
             raise ValueError('This object does not have a distance; cannot '
                              'compute 3d separation.')
-        if other.data.__class__ == UnitSphericalRepresentation:
+        if issubclass(other.data.__class__, UnitSphericalRepresentation):
             raise ValueError('The other object does not have a distance; '
                              'cannot compute 3d separation.')
 
@@ -769,7 +820,7 @@ class SkyCoord(object):
         Searches for all coordinates in this object around a supplied set of
         points within a given on-sky separation.
 
-        This is inteded for use on `~astropy.coordinates.SkyCoord` objects
+        This is intended for use on `~astropy.coordinates.SkyCoord` objects
         with coordinate arrays, rather than a scalar coordinate.  For a scalar
         coordinate, it is better to use
         `~astropy.coordinates.SkyCoord.separation`.
@@ -827,7 +878,7 @@ class SkyCoord(object):
         Searches for all coordinates in this object around a supplied set of
         points within a given 3D radius.
 
-        This is inteded for use on `~astropy.coordinates.SkyCoord` objects
+        This is intended for use on `~astropy.coordinates.SkyCoord` objects
         with coordinate arrays, rather than a scalar coordinate.  For a scalar
         coordinate, it is better to use
         `~astropy.coordinates.SkyCoord.separation_3d`.
@@ -924,6 +975,42 @@ class SkyCoord(object):
 
         return angle_utilities.position_angle(slon, slat, olon, olat)
 
+    def get_constellation(self, short_name=False, constellation_list='iau'):
+        """
+        Determines the constellation(s) of the coordinates this `SkyCoord`
+        contains.
+
+        Parameters
+        ----------
+        short_name : bool
+            If True, the returned names are the IAU-sanctioned abbreviated
+            names.  Otherwise, full names for the constellations are used.
+        constellation_list : str
+            The set of constellations to use.  Currently only ``'iau'`` is
+            supported, meaning the 88 "modern" constellations endorsed by the IAU.
+
+        Returns
+        -------
+        constellation : str or string array
+            If this is a scalar coordinate, returns the name of the
+            constellation.  If it is an array `SkyCoord`, it returns an array of
+            names.
+
+        Notes
+        -----
+        To determine which constellation a point on the sky is in, this first
+        precesses to B1875, and then uses the Delporte boundaries of the 88
+        modern constellations, as tabulated by
+        `Roman 1987 <http://cdsarc.u-strasbg.fr/viz-bin/Cat?VI/42>`_.
+
+        See Also
+        --------
+        astropy.coordinates.get_constellation
+        """
+        from .funcs import get_constellation
+
+        return get_constellation(self, short_name, constellation_list)
+
     # WCS pixel to/from sky conversions
     def to_pixel(self, wcs, origin=0, mode='all'):
         """
@@ -947,7 +1034,6 @@ class SkyCoord(object):
 
         See Also
         --------
-        from_pixel : to do the inverse operation
         astropy.wcs.utils.skycoord_to_pixel : the implementation of this method
         """
         return skycoord_to_pixel(self, wcs=wcs, origin=origin, mode=mode)
@@ -1315,11 +1401,15 @@ def _parse_coordinate_arg(coords, frame, units, init_kwargs):
             # SkyCoords from the list elements and then combine them.
             scs = [SkyCoord(coord, **init_kwargs) for coord in coords]
 
-            # now check that they're all self-consistent in their frames
+            # now check that they're all self-consistent in their frames and
+            # check if they are all UnitSphericalRepresentation internally
+            allunitsphrepr = True
             for sc in scs[1:]:
                 if not sc.is_equivalent_frame(scs[0]):
                         raise ValueError("List of inputs don't have equivalent "
                                          "frames: {0} != {1}".format(sc, scs[0]))
+                if allunitsphrepr and not isinstance(sc.data, UnitSphericalRepresentation):
+                    allunitsphrepr = False
 
             # get the frame attributes from the first one, because from above we
             # know it matches all the others
@@ -1328,7 +1418,10 @@ def _parse_coordinate_arg(coords, frame, units, init_kwargs):
 
             # Now combine the values, to be used below
             values = []
-            for data_attr_name in frame_attr_names:
+            for data_attr_name, repr_attr_name in zip(frame_attr_names, repr_attr_names):
+                if allunitsphrepr and repr_attr_name == 'distance':
+                    #if they are *all* UnitSpherical, don't give a distance
+                    continue
                 data_vals = []
                 for sc in scs:
                     data_val = getattr(sc, data_attr_name)
