@@ -5,7 +5,7 @@ from __future__ import (absolute_import, division, print_function,
 
 import numpy as np
 from numpy import testing as npt
-from ...tests.helper import pytest
+from ...tests.helper import pytest, assert_quantity_allclose as assert_allclose
 
 from ... import units as u
 from ...utils import minversion
@@ -61,15 +61,15 @@ def test_matching_function_3d_and_sky():
     npt.assert_array_equal(idx, [2, 3])
 
 
-    npt.assert_allclose(d2d.degree, [1, 1.9])
+    assert_allclose(d2d, [1, 1.9] * u.deg)
     assert np.abs(d3d[0].to(u.kpc).value - np.radians(1)) < 1e-6
     assert np.abs(d3d[1].to(u.kpc).value - 5*np.radians(1.9)) < 1e-5
 
     idx, d2d, d3d = match_coordinates_sky(cmatch, ccatalog)
     npt.assert_array_equal(idx, [3, 1])
 
-    npt.assert_allclose(d2d.degree, [0, 0.1])
-    npt.assert_allclose(d3d.to(u.kpc).value, [4, 4.0000019])
+    assert_allclose(d2d, [0, 0.1] * u.deg)
+    assert_allclose(d3d, [4, 4.0000019] * u.kpc)
 
 
 @pytest.mark.skipif(str('not HAS_SCIPY'))
@@ -108,16 +108,16 @@ def test_matching_method():
     idx2, d2d2, d3d2 = match_coordinates_3d(cmatch, ccatalog)
 
     npt.assert_array_equal(idx1, idx2)
-    npt.assert_allclose(d2d1, d2d2)
-    npt.assert_allclose(d3d1, d3d2)
+    assert_allclose(d2d1, d2d2)
+    assert_allclose(d3d1, d3d2)
 
     #should be the same as above because there's no distance, but just make sure this method works
     idx1, d2d1, d3d1 = SkyCoord(cmatch).match_to_catalog_sky(ccatalog)
     idx2, d2d2, d3d2 = match_coordinates_sky(cmatch, ccatalog)
 
     npt.assert_array_equal(idx1, idx2)
-    npt.assert_allclose(d2d1, d2d2)
-    npt.assert_allclose(d3d1, d3d2)
+    assert_allclose(d2d1, d2d2)
+    assert_allclose(d3d1, d3d2)
 
     assert len(idx1) == len(d2d1) == len(d3d1) == 20
 
@@ -136,7 +136,7 @@ def test_search_around():
 
     assert list(zip(idx1_1deg, idx2_1deg)) == [(0, 2), (0, 3), (1, 1), (1, 2)]
     assert d2d_1deg[0] == 1.0*u.deg
-    npt.assert_allclose(d2d_1deg, [1, 0, .1, .9]*u.deg)
+    assert_allclose(d2d_1deg, [1, 0, .1, .9]*u.deg)
 
     assert list(zip(idx1_0p05deg, idx2_0p05deg)) == [(0, 3)]
 
@@ -145,7 +145,7 @@ def test_search_around():
 
     assert list(zip(idx1_1kpc, idx2_1kpc)) == [(0, 0), (0, 1), (0, 2), (1, 3)]
     assert list(zip(idx1_sm, idx2_sm)) == [(0, 1), (0, 2)]
-    npt.assert_allclose(d2d_sm, [2, 1]*u.deg)
+    assert_allclose(d2d_sm, [2, 1]*u.deg)
 
 
 @pytest.mark.skipif(str('not HAS_SCIPY'))
