@@ -15,16 +15,18 @@ import numpy as np
 import numpy.testing as npt
 
 from ... import units as u
-from ...tests.helper import (pytest, catch_warnings, quantity_allclose,
+from ...tests.helper import (pytest, remote_data, catch_warnings,
+                             quantity_allclose,
                              assert_quantity_allclose as assert_allclose)
 from ..representation import REPRESENTATION_CLASSES
 from ...coordinates import (ICRS, FK4, FK5, Galactic, SkyCoord, Angle,
                             SphericalRepresentation, CartesianRepresentation,
-                            UnitSphericalRepresentation)
-from ...coordinates import Latitude, Longitude
+                            UnitSphericalRepresentation, AltAz)
+from ...coordinates import Latitude, Longitude, EarthLocation
 from ...time import Time
 from ...utils import minversion
 from ...utils.exceptions import AstropyDeprecationWarning
+from ...utils.compat import NUMPY_LT_1_7
 
 RA = 1.0 * u.deg
 DEC = 2.0 * u.deg
@@ -71,10 +73,10 @@ for rt_frame0 in rt_frames:
                 for obstime0 in (None, 'J1980.0'):
                     for equinox1 in (None, 'J1975.0'):
                         for obstime1 in (None, 'J1980.0'):
-                            rt_sets.append([rt_frame0, rt_frame1,
+                            rt_sets.append((rt_frame0, rt_frame1,
                                             equinox0, equinox1,
-                                            obstime0, obstime1])
-rt_args = 'frame0,frame1,equinox0,equinox1,obstime0,obstime1'
+                                            obstime0, obstime1))
+rt_args = ('frame0','frame1','equinox0','equinox1','obstime0','obstime1')
 
 
 @pytest.mark.parametrize(rt_args, rt_sets)
@@ -479,7 +481,6 @@ def test_seps():
 
     assert sep3d == 1 * u.kpc
 
-
 def test_repr():
     # Repr tests must use exact floating point vals because Python 2.6
     # outputs values like 0.1 as 0.1000000000001.  No workaround found.
@@ -499,6 +500,18 @@ def test_repr():
     assert repr(sc_default) == ('<SkyCoord (ICRS): (ra, dec) in deg\n'
                                 '    (0.0, 1.0)>')
 
+@pytest.mark.skipif('NUMPY_LT_1_7')
+def test_repr_altaz():
+    sc2 = SkyCoord(1 * u.deg, 1 * u.deg, frame='icrs', distance=1 * u.kpc)
+    loc = EarthLocation(-2309223 * u.m, -3695529 * u.m, -4641767 * u.m)
+    time = Time('2005-03-21 00:00:00')
+    sc4 = sc2.transform_to(AltAz(location=loc, obstime=time))
+    assert repr(sc4).startswith("<SkyCoord (AltAz: obstime=2005-03-21 00:00:00.000, "
+                         "location=(-2309223.0, -3695529.0, "
+                                "-4641767.0) m, pressure=0.0 hPa, "
+                         "temperature=0.0 deg_C, relative_humidity=0, "
+                         "obswl=1.0 micron): (az, alt, distance) in "
+                         "(deg, deg, m)\n")
 
 def test_ops():
     """
@@ -627,11 +640,11 @@ for base_unit_attr_set in base_unit_attr_sets:
                     c2 = np.array(c2)
                     c3 = np.array(c3)
                 units_attr_sets.append(base_unit_attr_set + (representation, c1, c2, c3))
-units_attr_args = 'repr_name,unit1,unit2,unit3,cls2,attr1,attr2,attr3,representation,c1,c2,c3'
+units_attr_args = ('repr_name','unit1','unit2','unit3','cls2','attr1','attr2','attr3','representation','c1','c2','c3')
 
 
 @pytest.mark.parametrize(units_attr_args,
-                         (x for x in units_attr_sets if x[0] != 'unitspherical'))
+                         [x for x in units_attr_sets if x[0] != 'unitspherical'])
 def test_skycoord_three_components(repr_name, unit1, unit2, unit3, cls2, attr1, attr2, attr3,
                                    representation, c1, c2, c3):
     """
@@ -663,8 +676,8 @@ def test_skycoord_three_components(repr_name, unit1, unit2, unit3, cls2, attr1,
 
 
 @pytest.mark.parametrize(units_attr_args,
-                         (x for x in units_attr_sets
-                          if x[0] in ('spherical', 'unitspherical')))
+                         [x for x in units_attr_sets
+                          if x[0] in ('spherical', 'unitspherical')])
 def test_skycoord_spherical_two_components(repr_name, unit1, unit2, unit3, cls2,
                                            attr1, attr2, attr3, representation, c1, c2, c3):
     """
@@ -690,7 +703,7 @@ def test_skycoord_spherical_two_components(repr_name, unit1, unit2, unit3, cls2,
 
 
 @pytest.mark.parametrize(units_attr_args,
-                         (x for x in units_attr_sets if x[0] != 'unitspherical'))
+                         [x for x in units_attr_sets if x[0] != 'unitspherical'])
 def test_galactic_three_components(repr_name, unit1, unit2, unit3, cls2, attr1, attr2, attr3,
                                    representation, c1, c2, c3):
     """
@@ -715,8 +728,8 @@ def test_galactic_three_components(repr_name, unit1, unit2, unit3, cls2, attr1,
 
 
 @pytest.mark.parametrize(units_attr_args,
-                         (x for x in units_attr_sets
-                          if x[0] in ('spherical', 'unitspherical')))
+                         [x for x in units_attr_sets
+                          if x[0] in ('spherical', 'unitspherical')])
 def test_galactic_spherical_two_components(repr_name, unit1, unit2, unit3, cls2,
                                            attr1, attr2, attr3, representation, c1, c2, c3):
     """
@@ -735,8 +748,8 @@ def test_galactic_spherical_two_components(repr_name, unit1, unit2, unit3, cls2,
     assert_quantities_allclose(sc, (c1*unit1, c2*unit2), (attr1, attr2))
 
 
-@pytest.mark.parametrize('repr_name,unit1,unit2,unit3,cls2,attr1,attr2,attr3',
-                         (x for x in base_unit_attr_sets if x[0] != 'unitspherical'))
+@pytest.mark.parametrize(('repr_name','unit1','unit2','unit3','cls2','attr1','attr2','attr3'),
+                         [x for x in base_unit_attr_sets if x[0] != 'unitspherical'])
 def test_skycoord_coordinate_input(repr_name, unit1, unit2, unit3, cls2, attr1, attr2, attr3):
     c1, c2, c3 = 1, 2, 3
     sc = SkyCoord([(c1, c2, c3)], unit=(unit1, unit2, unit3), representation=repr_name,
@@ -798,7 +811,7 @@ def test_nodata_failure():
         SkyCoord()
 
 
-@pytest.mark.parametrize('mode, origin', [('wcs', 0),
+@pytest.mark.parametrize(('mode', 'origin'), [('wcs', 0),
                                           ('all', 0),
                                           ('all', 1)])
 def test_wcs_methods(mode, origin):
@@ -1113,3 +1126,43 @@ def test_equiv_skycoord():
     assert scf2.is_equivalent_frame(FK5(equinox='J2005'))
     assert not scf3.is_equivalent_frame(scf1)
     assert not scf3.is_equivalent_frame(FK5(equinox='J2005'))
+
+
+def test_constellations():
+    # the actual test for accuracy is in test_funcs - this is just meant to make
+    # sure we get sensible answers
+    sc = SkyCoord(135*u.deg, 65*u.deg)
+    assert sc.get_constellation() == 'Ursa Major'
+    assert sc.get_constellation(short_name=True) == 'UMa'
+
+    scs = SkyCoord([135]*2*u.deg, [65]*2*u.deg)
+    npt.assert_equal(scs.get_constellation(), ['Ursa Major']*2)
+    npt.assert_equal(scs.get_constellation(short_name=True), ['UMa']*2)
+
+
+@remote_data
+def test_constellations_with_nameresolve():
+    assert SkyCoord.from_name('And I').get_constellation(short_name=True) == 'And'
+
+    #you'd think "And ..." should be in Andromeda.  But you'd be wrong.
+    assert SkyCoord.from_name('And VI').get_constellation() == 'Pegasus'
+
+    #maybe it's because And VI isn't really a galaxy?
+    assert SkyCoord.from_name('And XXII').get_constellation() == 'Pisces'
+    assert SkyCoord.from_name('And XXX').get_constellation() == 'Cassiopeia'
+    #ok maybe not
+
+    #ok, but at least some of the others do make sense...
+    assert SkyCoord.from_name('Coma Cluster').get_constellation(short_name=True) == 'Com'
+    assert SkyCoord.from_name('UMa II').get_constellation() == 'Ursa Major'
+    assert SkyCoord.from_name('Triangulum Galaxy').get_constellation() == 'Triangulum'
+
+
+def test_getitem_representation():
+    """
+    Make sure current representation survives __getitem__ even if different
+    from data representation.
+    """
+    sc = SkyCoord([1, 1] * u.deg, [2, 2] * u.deg)
+    sc.representation = 'cartesian'
+    assert sc[0].representation is CartesianRepresentation
