@@ -6,5 +6,8 @@ See the `Astropy documentation
 <http://docs.astropy.org/en/latest/cosmology/index.html>`_ for more
 detailed usage examples and references.
 """
-from core import *
-from funcs import *
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from .core import *
+from .funcs import *
