@@ -30,6 +30,9 @@ class CdsHeader(core.BaseHeader):
                     'i': core.IntType,
                     'a': core.StrType}
 
+    'The ReadMe file to construct header from.'
+    readme = None
+
     def get_type_map_key(self, col):
         match = re.match(r'\d*(\S)', col.raw_type.lower())
         if not match:
@@ -37,28 +40,19 @@ class CdsHeader(core.BaseHeader):
                 col.raw_type, col.name))
         return match.group(1)
 
-    def __init__(self, readme=None):
-        """Initialize ReadMe filename.
-
-        :param readme: The ReadMe file to construct header from.
-        :type readme: String
-
-        CDS tables have their header information in a separate file
-        named "ReadMe". The ``get_cols`` method will read the contents
-        of the ReadMe file given by ``self.readme`` and set the various
-        properties needed to read the data file. The data file name
-        will be the ``table`` passed to the ``read`` method.
-        """
-        core.BaseHeader.__init__(self)
-        self.readme = readme
 
     def get_cols(self, lines):
-        """Initialize the header Column objects from the table ``lines`` for a CDS
+        """
+        Initialize the header Column objects from the table ``lines`` for a CDS
         header.
 
-        :param lines: list of table lines
-        :returns: list of table Columns
+        Parameters
+        ----------
+        lines : list
+            List of table lines
+
         """
+
         # Read header block for the table ``self.data.table_name`` from the read
         # me file ``self.readme``.
         if self.readme and self.data.table_name:
@@ -234,12 +228,12 @@ class Cds(core.BaseReader):
       >>> table = ascii.read("t/vizier/table1.dat", readme="t/vizier/ReadMe")
       >>> table = ascii.read("t/cds/multi/lhs2065.dat", readme="t/cds/multi/ReadMe")
       >>> table = ascii.read("t/cds/glob/lmxbrefs.dat", readme="t/cds/glob/ReadMe")
-      
-    The table name and the CDS ReadMe file can be entered as URLs.  This can be used 
-    to directly load tables from the Internet.  For example, Vizier tables from the 
+
+    The table name and the CDS ReadMe file can be entered as URLs.  This can be used
+    to directly load tables from the Internet.  For example, Vizier tables from the
     CDS::
 
-      >>> table = ascii.read("ftp://cdsarc.u-strasbg.fr/pub/cats/VII/253/snrs.dat", 
+      >>> table = ascii.read("ftp://cdsarc.u-strasbg.fr/pub/cats/VII/253/snrs.dat",
       ...             readme="ftp://cdsarc.u-strasbg.fr/pub/cats/VII/253/ReadMe")
 
     If the header (ReadMe) and data are stored in a single file and there
@@ -287,10 +281,12 @@ class Cds(core.BaseReader):
     _io_registry_can_write = False
     _description = 'CDS format table'
 
+    data_class = CdsData
+    header_class = CdsHeader
+
     def __init__(self, readme=None):
-        core.BaseReader.__init__(self)
-        self.header = CdsHeader(readme)
-        self.data = CdsData()
+        super(Cds, self).__init__()
+        self.header.readme = readme
 
     def write(self, table=None):
         """Not available for the Cds class (raises NotImplementedError)"""
