@@ -25,7 +25,6 @@ from ...extern.six.moves import cStringIO as StringIO
 from ...utils.exceptions import AstropyWarning
 
 from ...table import Table
-from ...table.column import col_getattr, col_setattr, col_iter_str_vals
 from ...utils.compat import ignored
 from ...utils.data import get_readable_fileobj
 from ...utils import OrderedDict
@@ -485,12 +484,12 @@ class BaseHeader(object):
         self._set_cols_from_names()
 
     def process_lines(self, lines):
-        """Generator to yield non-comment lines"""
+        """Generator to yield non-blank and non-comment lines"""
         if self.comment:
             re_comment = re.compile(self.comment)
         # Yield non-comment lines
         for line in lines:
-            if line and (not self.comment or not re_comment.match(line)):
+            if line.strip() and (not self.comment or not re_comment.match(line)):
                 yield line
 
     def write_comments(self, lines, meta):
@@ -503,12 +502,12 @@ class BaseHeader(object):
             for i, spacer_line in zip(range(self.start_line),
                                       itertools.cycle(self.write_spacer_lines)):
                 lines.append(spacer_line)
-            lines.append(self.splitter.join([col_getattr(x, 'name') for x in self.cols]))
+            lines.append(self.splitter.join([x.info.name for x in self.cols]))
 
     @property
     def colnames(self):
         """Return the column names of the table"""
-        return tuple(col.name if isinstance(col, Column) else col_getattr(col, 'name')
+        return tuple(col.name if isinstance(col, Column) else col.info.name
                      for col in self.cols)
 
     def get_type_map_key(self, col):
@@ -552,7 +551,7 @@ class BaseHeader(object):
                                      .format(name))
         # When guessing require at least two columns
         if guessing and len(self.colnames) <= 1:
-            raise ValueError
+            raise ValueError('Strict name guessing requires at least two columns')
 
         if names is not None and len(names) != len(self.colnames):
             raise ValueError('Length of names argument ({0}) does not match number'
@@ -708,7 +707,7 @@ class BaseData(object):
         self._set_fill_values(self.cols)
         self._set_col_formats()
         for col in self.cols:
-            col.str_vals = list(col_iter_str_vals(col))
+            col.str_vals = list(col.info.iter_str_vals())
         self._replace_vals(self.cols)
         return [col.str_vals for col in self.cols]
 
@@ -729,8 +728,8 @@ class BaseData(object):
         """
         """
         for col in self.cols:
-            if col_getattr(col, 'name') in self.formats:
-                col_setattr(col, 'format', self.formats[col.name])
+            if col.info.name in self.formats:
+                col.info.format = self.formats[col.name]
 
 
 def convert_numpy(numpy_type):
