@@ -1,5 +1,19 @@
 # Licensed under a 3-clause BSD style license
+from __future__ import absolute_import
 
+import os
+from distutils.extension import Extension
+
+ROOT = os.path.relpath(os.path.dirname(__file__))
+
+def get_extensions():
+    sources = [os.path.join(ROOT, 'cparser.pyx'),
+               os.path.join(ROOT, 'src', 'tokenizer.c')]
+    ascii_ext = Extension(
+        name="astropy.io.ascii.cparser",
+        include_dirs=["numpy"],
+        sources=sources)
+    return [ascii_ext]
 
 def get_package_data():
     # Installs the testing data files.  Unable to get package_data
@@ -13,21 +27,29 @@ def get_package_data():
                                    't/bad.txt',
                                    't/bars_at_ends.txt',
                                    't/cds.dat',
+                                   't/cds_malformed.dat',
                                    't/cds/glob/ReadMe',
                                    't/cds/glob/lmxbrefs.dat',
                                    't/cds/multi/ReadMe',
                                    't/cds/multi/lhs2065.dat',
+                                   't/cds/multi/lp944-20.dat',
                                    't/cds2.dat',
                                    't/commented_header.dat',
                                    't/commented_header2.dat',
                                    't/continuation.dat',
                                    't/daophot.dat',
                                    't/daophot2.dat',
+                                   't/daophot3.dat',
+                                   't/daophot4.dat',
                                    't/sextractor.dat',
+                                   't/sextractor2.dat',
                                    't/daophot.dat.gz',
                                    't/fill_values.txt',
+                                   't/html.html',
+                                   't/html2.html',
                                    't/ipac.dat',
                                    't/ipac.dat.bz2',
+                                   't/ipac.dat.xz',
                                    't/latex1.tex',
                                    't/latex1.tex.gz',
                                    't/latex2.tex',
@@ -41,6 +63,7 @@ def get_package_data():
                                    't/short.rdb',
                                    't/short.rdb.bz2',
                                    't/short.rdb.gz',
+                                   't/short.rdb.xz',
                                    't/short.tab',
                                    't/simple.txt',
                                    't/simple2.txt',
@@ -54,5 +77,12 @@ def get_package_data():
                                    't/test5.dat',
                                    't/vots_spec.dat',
                                    't/whitespace.dat',
+                                   't/simple_csv.csv',
+                                   't/simple_csv_missing.csv',
+                                   't/fixed_width_2_line.txt',
                                    ]
     }
+
+
+def requires_2to3():
+    return False
