@@ -16,7 +16,7 @@ try:
     HAS_BEAUTIFUL_SOUP = True
 except ImportError:
     HAS_BEAUTIFUL_SOUP = False
-    
+
 if HAS_BEAUTIFUL_SOUP:
     files.append('t/html.html')
 
@@ -120,13 +120,20 @@ def test_write_rdb_noformat(tmpdir):
 
 
 def test_read_csv():
-    Table.read(os.path.join(ROOT, 't/simple_csv.csv'), format='ascii.csv')
+    '''If properly registered, filename should be sufficient to specify format
+
+    #3189
+    '''
+    Table.read(os.path.join(ROOT, 't/simple_csv.csv'))
 
 
 def test_write_csv(tmpdir):
+    '''If properly registered, filename should be sufficient to specify format
+
+    #3189
+    '''
     t = Table()
     t.add_column(Column(name='a', data=[1, 2, 3]))
     t.add_column(Column(name='b', data=['a', 'b', 'c']))
     path = str(tmpdir.join("data.csv"))
-    t.write(path, format='ascii.csv')
-
+    t.write(path)
