@@ -11,7 +11,6 @@ from .verify import _Verify, _ErrList, VerifyError, VerifyWarning
 
 from . import conf
 from ...extern.six import string_types, integer_types, text_type, binary_type
-from ...extern.six.moves import xrange
 from ...utils import deprecated
 from ...utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
 
@@ -501,7 +500,7 @@ class Card(_Verify):
             elif self._keywd_hierarch_RE.match(keyword):
                 # In prior versions of PyFITS HIERARCH cards would only be
                 # created if the user-supplied keyword explicitly started with
-                # 'HIERARCH '.  Now we will create them automtically for long
+                # 'HIERARCH '.  Now we will create them automatically for long
                 # keywords, but we still want to support the old behavior too;
                 # the old behavior makes it possible to create HEIRARCH cards
                 # that would otherwise be recognized as RVKCs
@@ -855,7 +854,7 @@ class Card(_Verify):
         If one argument is given, that argument is treated as a full card image
         and parsed as such.  If two arguments are given, the first is treated
         as the card keyword (including the field-specifier if the card is
-        intened as a RVKC), and the second as the card value OR the first value
+        intended as a RVKC), and the second as the card value OR the first value
         can be the base keyword, and the second value the 'field-specifier:
         value' string.
 
@@ -901,7 +900,7 @@ class Card(_Verify):
     def _check_if_rvkc_image(self, *args):
         """
         Implements `Card._check_if_rvkc` for the case of an unparsed card
-        image.  If given one argment this is the full intact image.  If given
+        image.  If given one argument this is the full intact image.  If given
         two arguments the card has already been split between keyword and
         value+comment at the standard value indicator '= '.
         """
@@ -1268,7 +1267,6 @@ class Card(_Verify):
         output = []
 
         # do the value string
-        value_format = "'%-s&'"
         value = self._value.replace("'", "''")
         words = _words_group(value, value_length)
         for idx, word in enumerate(words):
@@ -1276,15 +1274,29 @@ class Card(_Verify):
                 headstr = '%-*s= ' % (KEYWORD_LENGTH, self.keyword)
             else:
                 headstr = 'CONTINUE  '
+
+            # If this is the final CONTINUE remove the '&'
+            if not self.comment and idx == len(words) - 1:
+                value_format = "'%-s'"
+            else:
+                value_format = "'%-s&'"
+
             value = value_format % word
+
             output.append('%-80s' % (headstr + value))
 
         # do the comment string
         comment_format = "%-s"
         if self.comment:
             words = _words_group(self.comment, comment_length)
-            for word in words:
-                comment = "CONTINUE  '&' / " + comment_format % word
+            for idx, word in enumerate(words):
+                # If this is the final CONTINUE remove the '&'
+                if idx == len(words) - 1:
+                    headstr = "CONTINUE  '' / "
+                else:
+                    headstr = "CONTINUE  '&' / "
+
+                comment = headstr + comment_format % word
                 output.append('%-80s' % comment)
 
         return ''.join(output)
@@ -1339,7 +1351,7 @@ class Card(_Verify):
                 # keywords can only occur if they came from the wild
                 keyword = self._split()[0]
                 if keyword != keyword.upper():
-                # Keyword should be uppercase unless it's a HIERARCH card
+                    # Keyword should be uppercase unless it's a HIERARCH card
                     errs.append(self.run_option(
                         option,
                         err_text='Card keyword %r is not upper case.' %
