@@ -2,13 +2,13 @@
 
 from __future__ import division, with_statement
 
-from ...utils.compat import gzip as _astropy_gzip
-import gzip as _system_gzip
 import mmap
 import os
+import sys
 import tempfile
 import warnings
 import zipfile
+import bz2
 
 from functools import reduce
 
@@ -17,9 +17,12 @@ from numpy import memmap as Memmap
 
 from .util import (isreadable, iswritable, isfile, fileobj_open, fileobj_name,
                    fileobj_closed, fileobj_mode, _array_from_file,
-                   _array_to_file, _write_string, encode_ascii)
+                   _array_to_file, _write_string, _GZIP_FILE_TYPES)
 from ...extern.six import b, string_types
 from ...extern.six.moves import urllib
+from ...utils.compat import gzip
+from ...utils.data import download_file, _is_url
+from ...utils.decorators import classproperty
 from ...utils.exceptions import AstropyUserWarning
 
 
@@ -70,20 +73,20 @@ MEMMAP_MODES = {'readonly': 'c', 'copyonwrite': 'c', 'update': 'r+',
 
 GZIP_MAGIC = b('\x1f\x8b\x08')
 PKZIP_MAGIC = b('\x50\x4b\x03\x04')
-
-_GZIP_FILE_TYPES = (_astropy_gzip, _system_gzip)
+BZIP2_MAGIC = b('\x42\x5a')
 
 class _File(object):
     """
     Represents a FITS file on disk (or in some other file-like object).
     """
 
-    # See self._test_mmap
-    _mmap_available = None
+    def __init__(self, fileobj=None, mode=None, memmap=None, clobber=False,
+                 cache=True):
+        self.strict_memmap = bool(memmap)
+        memmap = True if memmap is None else memmap
 
-    def __init__(self, fileobj=None, mode=None, memmap=False, clobber=False):
         if fileobj is None:
-            self.__file = None
+            self._file = None
             self.closed = False
             self.binary = True
             self.mode = mode
@@ -96,6 +99,9 @@ class _File(object):
         else:
             self.simulateonly = False
 
+        # Holds mmap instance for files that use mmap
+        self._mmap = None
+
         if mode is None:
             if _is_random_access_file_backed(fileobj):
                 fmode = fileobj_mode(fileobj)
@@ -109,25 +115,9 @@ class _File(object):
             raise ValueError("Mode '%s' not recognized" % mode)
 
         if (isinstance(fileobj, string_types) and
-                mode not in ('ostream', 'append') and
-                not os.path.exists(fileobj)):
-
-            # Not writing file and file does not exist on local machine and
-            # name does not begin with a drive letter (Windows), try to get it
-            # over the web.
-            try:
-                if not os.path.splitdrive(fileobj)[0]:
-                    # Basically if the filename (on Windows anyways) doesn't
-                    # have a drive letter try to open it as a URL
-                    self.name, _ = urllib.request.urlretrieve(fileobj)
-                else:
-                    # Otherwise the file was already not found so just raise
-                    # a ValueError
-                    raise ValueError("File not found")
-            except (TypeError, ValueError, IOError):
-                # A couple different exceptions can occur here when passing a
-                # filename into urlretrieve in Python 3
-                raise IOError('File does not exist: {0!r}'.format(fileobj))
+            mode not in ('ostream', 'append') and
+            _is_url(fileobj)): # This is an URL.
+                self.name = download_file(fileobj, cache=cache)
         else:
             self.name = fileobj_name(fileobj)
 
@@ -144,7 +134,7 @@ class _File(object):
         self.readonly = False
         self.writeonly = False
 
-        # Initialize the internal self.__file object
+        # Initialize the internal self._file object
         if _is_random_access_file_backed(fileobj):
             self._open_fileobj(fileobj, mode, clobber)
         elif isinstance(fileobj, string_types):
@@ -152,13 +142,15 @@ class _File(object):
         else:
             self._open_filelike(fileobj, mode, clobber)
 
-        self.fileobj_mode = fileobj_mode(self.__file)
+        self.fileobj_mode = fileobj_mode(self._file)
 
-        if isinstance(fileobj, (_astropy_gzip.GzipFile, _system_gzip.GzipFile)):
+        if isinstance(fileobj, _GZIP_FILE_TYPES):
             self.compression = 'gzip'
         elif isinstance(fileobj, zipfile.ZipFile):
             # Reading from zip files is supported but not writing (yet)
             self.compression = 'zip'
+        elif isinstance(fileobj, bz2.BZ2File):
+            self.compression = 'bzip2'
 
         if (mode in ('readonly', 'copyonwrite', 'denywrite') or
                 (self.compression and mode == 'update')):
@@ -170,27 +162,27 @@ class _File(object):
         # For 'ab+' mode, the pointer is at the end after the open in
         # Linux, but is at the beginning in Solaris.
         if (mode == 'ostream' or self.compression or
-            not hasattr(self.__file, 'seek')):
+            not hasattr(self._file, 'seek')):
             # For output stream start with a truncated file.
             # For compressed files we can't really guess at the size
             self.size = 0
         else:
-            pos = self.__file.tell()
-            self.__file.seek(0, 2)
-            self.size = self.__file.tell()
-            self.__file.seek(pos)
+            pos = self._file.tell()
+            self._file.seek(0, 2)
+            self.size = self._file.tell()
+            self._file.seek(pos)
 
         if self.memmap:
-            if not isfile(self.__file):
+            if not isfile(self._file):
                 self.memmap = False
-            elif not self.readonly and not self._test_mmap():
+            elif not self.readonly and not self._mmap_available:
                 # Test mmap.flush--see
                 # https://github.com/astropy/astropy/issues/968
                 self.memmap = False
 
     def __repr__(self):
         return '<%s.%s %s>' % (self.__module__, self.__class__.__name__,
-                               self.__file)
+                               self._file)
 
     # Support the 'with' statement
     def __enter__(self):
@@ -202,13 +194,13 @@ class _File(object):
     def readable(self):
         if self.writeonly:
             return False
-        return isreadable(self.__file)
+        return isreadable(self._file)
 
     def read(self, size=None):
-        if not hasattr(self.__file, 'read'):
+        if not hasattr(self._file, 'read'):
             raise EOFError
         try:
-            return self.__file.read(size)
+            return self._file.read(size)
         except IOError:
             # On some versions of Python, it appears, GzipFile will raise an
             # IOError if you try to read past its end (as opposed to just
@@ -227,7 +219,7 @@ class _File(object):
         it's provided for compatibility.
         """
 
-        if not hasattr(self.__file, 'read'):
+        if not hasattr(self._file, 'read'):
             raise EOFError
 
         if not isinstance(dtype, np.dtype):
@@ -239,8 +231,17 @@ class _File(object):
         if isinstance(shape, int):
             shape = (shape,)
 
+        if not (size or shape):
+            warnings.warn('No size or shape given to readarray(); assuming a '
+                          'shape of (1,)', AstropyUserWarning)
+            shape = (1,)
+
+        if size and not shape:
+            shape = (size // dtype.itemsize,)
+
         if size and shape:
-            actualsize = sum(dim * dtype.itemsize for dim in shape)
+            actualsize = np.prod(shape) * dtype.itemsize
+
             if actualsize < size:
                 raise ValueError('size %d is too few bytes for a %s array of '
                                  '%s' % (size, shape, dtype))
@@ -248,35 +249,46 @@ class _File(object):
                 raise ValueError('size %d is too many bytes for a %s array of '
                                  '%s' % (size, shape, dtype))
 
-        if size and not shape:
-            shape = (size // dtype.itemsize,)
-
-        if not (size or shape):
-            warnings.warn('No size or shape given to readarray(); assuming a '
-                          'shape of (1,)', AstropyUserWarning)
-            shape = (1,)
-
         if self.memmap:
-            return Memmap(self.__file, offset=offset,
-                          mode=MEMMAP_MODES[self.mode], dtype=dtype,
-                          shape=shape).view(np.ndarray)
+            if self._mmap is None:
+                # Instantiate Memmap array of the file offset at 0
+                # (so we can return slices of it to offset anywhere else into
+                # the file)
+                memmap = Memmap(self._file,
+                                mode=MEMMAP_MODES[self.mode],
+                                dtype=np.uint8)
+
+                # Now we immediately discard the memmap array; we are really
+                # just using it as a factory function to instantiate the mmap
+                # object in a convenient way (may later do away with this
+                # usage)
+                self._mmap = memmap.base
+
+                # Prevent dorking with self._memmap._mmap by memmap.__del__ in
+                # Numpy 1.6 (see
+                # https://github.com/numpy/numpy/commit/dcc355a0b179387eeba10c95baf2e1eb21d417c7)
+                memmap._mmap = None
+                del memmap
+
+            return np.ndarray(shape=shape, dtype=dtype, offset=offset,
+                              buffer=self._mmap)
         else:
             count = reduce(lambda x, y: x * y, shape)
-            pos = self.__file.tell()
-            self.__file.seek(offset)
-            data = _array_from_file(self.__file, dtype, count, '')
+            pos = self._file.tell()
+            self._file.seek(offset)
+            data = _array_from_file(self._file, dtype, count, '')
             data.shape = shape
-            self.__file.seek(pos)
+            self._file.seek(pos)
             return data
 
     def writable(self):
         if self.readonly:
             return False
-        return iswritable(self.__file)
+        return iswritable(self._file)
 
     def write(self, string):
-        if hasattr(self.__file, 'write'):
-            _write_string(self.__file, string)
+        if hasattr(self._file, 'write'):
+            _write_string(self._file, string)
 
     def writearray(self, array):
         """
@@ -286,54 +298,73 @@ class _File(object):
         the file on disk reflects the data written.
         """
 
-        if hasattr(self.__file, 'write'):
-            _array_to_file(array, self.__file)
+        if hasattr(self._file, 'write'):
+            _array_to_file(array, self._file)
 
     def flush(self):
-        if hasattr(self.__file, 'flush'):
-            self.__file.flush()
+        if hasattr(self._file, 'flush'):
+            self._file.flush()
 
     def seek(self, offset, whence=0):
         # In newer Python versions, GzipFiles support the whence argument, but
         # I don't think it was added until 2.6; instead of assuming it's
         # present, we implement our own support for it here
-        if not hasattr(self.__file, 'seek'):
+        if not hasattr(self._file, 'seek'):
             return
-        if isinstance(self.__file, (_astropy_gzip.GzipFile, _system_gzip.GzipFile)):
+        if isinstance(self._file, _GZIP_FILE_TYPES):
             if whence:
                 if whence == 1:
-                    offset = self.__file.offset + offset
+                    offset = self._file.offset + offset
                 else:
                     raise ValueError('Seek from end not supported')
-            self.__file.seek(offset)
+            self._file.seek(offset)
         else:
-            self.__file.seek(offset, whence)
+            self._file.seek(offset, whence)
 
-        pos = self.__file.tell()
+        pos = self._file.tell()
         if self.size and pos > self.size:
             warnings.warn('File may have been truncated: actual file length '
                           '(%i) is smaller than the expected size (%i)' %
                           (self.size, pos), AstropyUserWarning)
 
     def tell(self):
-        if not hasattr(self.__file, 'tell'):
+        if not hasattr(self._file, 'tell'):
             raise EOFError
-        return self.__file.tell()
+        return self._file.tell()
 
     def truncate(self, size=None):
-        if hasattr(self.__file, 'truncate'):
-            self.__file.truncate(size)
+        if hasattr(self._file, 'truncate'):
+            self._file.truncate(size)
 
     def close(self):
         """
         Close the 'physical' FITS file.
         """
 
-        if hasattr(self.__file, 'close'):
-            self.__file.close()
+        if hasattr(self._file, 'close'):
+            self._file.close()
+
+        self._maybe_close_mmap()
+        # Set self._memmap to None anyways since no new .data attributes can be
+        # loaded after the file is closed
+        self._mmap = None
 
         self.closed = True
 
+    def _maybe_close_mmap(self, refcount_delta=0):
+        """
+        When mmap is in use these objects hold a reference to the mmap of the
+        file (so there is only one, shared by all HDUs that reference this
+        file).
+
+        This will close the mmap if there are no arrays referencing it.
+        """
+
+        if (self._mmap is not None and
+                sys.getrefcount(self._mmap) == 2 + refcount_delta):
+            self._mmap.close()
+            self._mmap = None
+
     def _overwrite_existing(self, clobber, fileobj, closed):
         """Overwrite an existing file if ``clobber`` is ``True``, otherwise
         raise an IOError.  The exact behavior of this method depends on the
@@ -342,13 +373,9 @@ class _File(object):
         """
 
         # The file will be overwritten...
-        if ((self.file_like and
-                (hasattr(fileobj, 'len') and fileobj.len > 0)) or
-                (os.path.exists(self.name) and
-                 os.path.getsize(self.name) != 0)):
+        if ((self.file_like and hasattr(fileobj, 'len') and fileobj.len > 0) or
+            (os.path.exists(self.name) and os.path.getsize(self.name) != 0)):
             if clobber:
-                warnings.warn("Overwriting existing file %r." % self.name,
-                              AstropyUserWarning)
                 if self.file_like and hasattr(fileobj, 'truncate'):
                     fileobj.truncate(0)
                 else:
@@ -381,16 +408,16 @@ class _File(object):
                 raise ValueError(
                     "Mode argument '%s' does not match mode of the input "
                     "file (%s)." % (mode, fmode))
-            self.__file = fileobj
+            self._file = fileobj
         elif isfile(fileobj):
-            self.__file = fileobj_open(self.name, PYFITS_MODES[mode])
+            self._file = fileobj_open(self.name, PYFITS_MODES[mode])
         else:
-            self.__file = _astropy_gzip.open(self.name, PYFITS_MODES[mode])
+            self._file = gzip.open(self.name, PYFITS_MODES[mode])
 
         if fmode == 'ab+':
             # Return to the beginning of the file--in Python 3 when opening in
             # append mode the file pointer is at the end of the file
-            self.__file.seek(0)
+            self._file.seek(0)
 
     def _open_filelike(self, fileobj, mode, clobber):
         """Open a FITS file from a file-like object, i.e. one that has
@@ -398,7 +425,7 @@ class _File(object):
         """
 
         self.file_like = True
-        self.__file = fileobj
+        self._file = fileobj
 
         if fileobj_closed(fileobj):
             raise IOError("Cannot read from/write to a closed file-like "
@@ -406,15 +433,15 @@ class _File(object):
 
         if isinstance(fileobj, zipfile.ZipFile):
             self._open_zipfile(fileobj, mode)
-            self.__file.seek(0)
+            self._file.seek(0)
             # We can bypass any additional checks at this point since now
-            # self.__file points to the temp file extracted from the zip
+            # self._file points to the temp file extracted from the zip
             return
 
         # If there is not seek or tell methods then set the mode to
         # output streaming.
-        if (not hasattr(self.__file, 'seek') or
-            not hasattr(self.__file, 'tell')):
+        if (not hasattr(self._file, 'seek') or
+            not hasattr(self._file, 'tell')):
             self.mode = mode = 'ostream'
 
         if mode == 'ostream':
@@ -422,13 +449,13 @@ class _File(object):
 
         # Any "writeable" mode requires a write() method on the file object
         if (self.mode in ('update', 'append', 'ostream') and
-            not hasattr(self.__file, 'write')):
+            not hasattr(self._file, 'write')):
             raise IOError("File-like object does not have a 'write' "
                           "method, required for mode '%s'."
                           % self.mode)
 
         # Any mode except for 'ostream' requires readability
-        if self.mode != 'ostream' and not hasattr(self.__file, 'read'):
+        if self.mode != 'ostream' and not hasattr(self._file, 'read'):
             raise IOError("File-like object does not have a 'read' "
                           "method, required for mode %r."
                           % self.mode)
@@ -449,17 +476,32 @@ class _File(object):
 
         if ext == '.gz' or magic.startswith(GZIP_MAGIC):
             # Handle gzip files
-            self.__file = _astropy_gzip.open(self.name, PYFITS_MODES[mode])
+            self._file = gzip.open(self.name, PYFITS_MODES[mode])
             self.compression = 'gzip'
         elif ext == '.zip' or magic.startswith(PKZIP_MAGIC):
             # Handle zip files
             self._open_zipfile(self.name, mode)
+        elif ext == '.bz2' or magic.startswith(BZIP2_MAGIC):
+            # Handle bzip2 files
+            if mode in ['update', 'append']:
+                raise IOError("update and append modes are not supported "
+                              "with bzip2 files")
+            # bzip2 only supports 'w' and 'r' modes
+            bzip2_mode = 'w' if mode == 'ostream' else 'r'
+            self._file = bz2.BZ2File(self.name, bzip2_mode)
         else:
-            self.__file = fileobj_open(self.name, PYFITS_MODES[mode])
-            # Make certain we're back at the beginning of the file
-        self.__file.seek(0)
+            self._file = fileobj_open(self.name, PYFITS_MODES[mode])
 
-    def _test_mmap(self):
+        # Make certain we're back at the beginning of the file
+        # BZ2File does not support seek when the file is open for writing, but
+        # when opening a file for write, bz2.BZ2File always truncates anyway.
+        if isinstance(self._file, bz2.BZ2File) and mode == 'ostream':
+            pass
+        else:
+            self._file.seek(0)
+
+    @classproperty(lazy=True)
+    def _mmap_available(cls):
         """Tests that mmap, and specifically mmap.flush works.  This may
         be the case on some uncommon platforms (see
         https://github.com/astropy/astropy/issues/968).
@@ -468,9 +510,6 @@ class _File(object):
         set and a warning is issued.
         """
 
-        if self._mmap_available is not None:
-            return self._mmap_available
-
         tmpfd, tmpname = tempfile.mkstemp()
         try:
             # Windows does not allow mappings on empty files
@@ -481,7 +520,6 @@ class _File(object):
             except mmap.error as e:
                 warnings.warn('Failed to create mmap: %s; mmap use will be '
                               'disabled' % str(e), AstropyUserWarning)
-                _File._mmap_available = False
                 del exc
                 return False
             try:
@@ -490,7 +528,6 @@ class _File(object):
                 warnings.warn('mmap.flush is unavailable on this platform; '
                               'using mmap in writeable mode will be disabled',
                               AstropyUserWarning)
-                _File._mmap_available = False
                 return False
             finally:
                 mm.close()
@@ -498,7 +535,6 @@ class _File(object):
             os.close(tmpfd)
             os.remove(tmpname)
 
-        _File._mmap_available = True
         return True
 
     def _open_zipfile(self, fileobj, mode):
@@ -523,8 +559,8 @@ class _File(object):
         if len(namelist) != 1:
             raise IOError(
               "Zip files with multiple members are not supported.")
-        self.__file = tempfile.NamedTemporaryFile(suffix='.fits')
-        self.__file.write(zfile.read(namelist[0]))
+        self._file = tempfile.NamedTemporaryFile(suffix='.fits')
+        self._file.write(zfile.read(namelist[0]))
 
         if close:
             zfile.close()
@@ -540,4 +576,4 @@ def _is_random_access_file_backed(fileobj):
     from an already opened `zipfile.ZipFile` object.
     """
 
-    return isfile(fileobj) or isinstance(fileobj, (_astropy_gzip.GzipFile, _system_gzip.GzipFile))
+    return isfile(fileobj) or isinstance(fileobj, _GZIP_FILE_TYPES)
