@@ -9,8 +9,8 @@ import warnings
 
 import numpy as np
 
-from .base import DELAYED, ExtensionHDU
-from .image import _ImageBaseHDU, ImageHDU
+from .base import DELAYED, ExtensionHDU, BITPIX2DTYPE, DTYPE2BITPIX
+from .image import ImageHDU
 from .table import BinTableHDU
 from ..card import Card
 from ..column import Column, ColDefs, TDEF_RE
@@ -1428,7 +1428,10 @@ class CompImageHDU(BinTableHDU):
         # data) from the file, if there is any.
         compressed_data = super(BinTableHDU, self).data
         if isinstance(compressed_data, np.rec.recarray):
-            del self.data
+            # Make sure not to use 'del self.data' so we don't accidentally
+            # go through the self.data.fdel and close the mmap underlying
+            # the compressed_data array
+            del self.__dict__['data']
             return compressed_data
         else:
             # This will actually set self.compressed_data with the
@@ -1621,7 +1624,7 @@ class CompImageHDU(BinTableHDU):
             for idx in range(self.header['NAXIS']):
                 _shape += (self.header['NAXIS' + str(idx + 1)],)
 
-            _format = _ImageBaseHDU.NumCode[self.header['BITPIX']]
+            _format = BITPIX2DTYPE[self.header['BITPIX']]
 
         return (self.name, class_name, len(self.header), _shape,
                 _format)
@@ -1632,7 +1635,7 @@ class CompImageHDU(BinTableHDU):
         """
 
         # Check to see that the image_header matches the image data
-        image_bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
+        image_bitpix = DTYPE2BITPIX[self.data.dtype.name]
 
         if image_bitpix != self._orig_bitpix or self.data.shape != self.shape:
             self._update_header_data(self.header)
@@ -1734,7 +1737,7 @@ class CompImageHDU(BinTableHDU):
 
         # Determine the destination (numpy) data type
         if type is None:
-            type = _ImageBaseHDU.NumCode[self._bitpix]
+            type = BITPIX2DTYPE[self._bitpix]
         _type = getattr(np, type)
 
         # Determine how to scale the data
@@ -1785,7 +1788,7 @@ class CompImageHDU(BinTableHDU):
             self.data = np.array(np.around(self.data), dtype=_type)  # 0.7.7.1
 
         # Update the BITPIX Card to match the data
-        self._bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
+        self._bitpix = DTYPE2BITPIX[self.data.dtype.name]
         self._bzero = self.header.get('BZERO', 0)
         self._bscale = self.header.get('BSCALE', 1)
         # Update BITPIX for the image header specifically
@@ -1806,7 +1809,7 @@ class CompImageHDU(BinTableHDU):
 
     def _prewriteto(self, checksum=False, inplace=False):
         if self._scale_back:
-            self.scale(_ImageBaseHDU.NumCode[self._orig_bitpix])
+            self.scale(BITPIX2DTYPE[self._orig_bitpix])
 
         if self._has_data:
             self._update_compressed_data()
@@ -1917,7 +1920,7 @@ class CompImageHDU(BinTableHDU):
             if dtype is None:
                 dtype = self._dtype_for_bitpix()
             if dtype is not None:
-                self.header['BITPIX'] = _ImageBaseHDU.ImgCode[dtype.name]
+                self.header['BITPIX'] = DTYPE2BITPIX[dtype.name]
 
             self._bzero = 0
             self._bscale = 1
