@@ -2,6 +2,7 @@
 
 from __future__ import division  # confidence high
 
+import contextlib
 import csv
 import os
 import re
@@ -21,12 +22,13 @@ from ..column import (FITS2NUMPY, KEYWORD_NAMES, KEYWORD_TO_ATTRIBUTE,
                       _AsciiColDefs, _FormatP, _FormatQ, _makep,
                       _parse_tformat, _scalar_to_format, _convert_format,
                       _cmp_recformats, _get_index)
-from ..fitsrec import FITS_rec, _get_recarray_field
+from ..fitsrec import FITS_rec, _get_recarray_field, _has_unicode_fields
 from ..header import Header, _pad_length
 from ..util import _is_int, _str_to_num
 
 from ....extern import six
 from ....extern.six import string_types
+from ....extern.six.moves import xrange as range
 from ....utils import deprecated, lazyproperty
 from ....utils.compat import ignored
 from ....utils.exceptions import AstropyUserWarning
@@ -194,11 +196,9 @@ class _TableLikeHDU(_ValidHDU):
         data._gap = self._theap - tbsize
 
         # pass the attributes
-        fidx = 0
-        for idx in range(len(columns)):
+        for idx, col in enumerate(columns):
             # get the data for each column object from the rec.recarray
-            columns[idx].array = data.field(fidx)
-            fidx += 1
+            col.array = data.field(idx)
 
         # delete the _arrays attribute so that it is recreated to point to the
         # new data placed in the column object above
@@ -343,7 +343,11 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                         self.data._coldefs[indx]._physical_values = False
                         self.data._coldefs[indx]._pseudo_unsigned_ints = True
 
-                self._header['NAXIS1'] = self.data.itemsize
+                # TODO: Too much of the code in this class uses header keywords
+                # in making calculations related to the data size.  This is
+                # unreliable, however, in cases when users mess with the header
+                # unintentionally--code that does this should be cleaned up.
+                self._header['NAXIS1'] = self.data._raw_itemsize
                 self._header['NAXIS2'] = self.data.shape[0]
                 self._header['TFIELDS'] = len(self.data._coldefs)
 
@@ -354,8 +358,8 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                     # Make the ndarrays in the Column objects of the ColDefs
                     # object of the HDU reference the same ndarray as the HDU's
                     # FITS_rec object.
-                    for idx in range(len(self.columns)):
-                        self.columns[idx].array = self.data.field(idx)
+                    for idx, col in enumerate(self.columns):
+                        col.array = self.data.field(idx)
 
                     # Delete the _arrays attribute so that it is recreated to
                     # point to the new data placed in the column objects above
@@ -443,8 +447,8 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                 # Make the ndarrays in the Column objects of the ColDefs
                 # object of the HDU reference the same ndarray as the HDU's
                 # FITS_rec object.
-                for idx in range(len(self.columns)):
-                    self.columns[idx].array = self.data.field(idx)
+                for idx, col in enumerate(self.columns):
+                    col.array = self.data.field(idx)
 
                 # Delete the _arrays attribute so that it is recreated to
                 # point to the new data placed in the column objects above
@@ -478,7 +482,7 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
         Update header keywords to reflect recent changes of columns.
         """
 
-        self._header.set('NAXIS1', self.data.itemsize, after='NAXIS')
+        self._header.set('NAXIS1', self.data._raw_itemsize, after='NAXIS')
         self._header.set('NAXIS2', self.data.shape[0], after='NAXIS1')
         self._header.set('TFIELDS', len(self.columns), after='GCOUNT')
 
@@ -492,8 +496,8 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
 
         # touch the data, so it's defined (in the case of reading from a
         # FITS file)
-        self.data
-        return new_table(self.columns, header=self._header)
+        return self.__class__(data=self.data.copy(),
+                              header=self._header.copy())
 
     def _prewriteto(self, checksum=False, inplace=False):
         if self._has_data:
@@ -747,10 +751,11 @@ class TableHDU(_TableBaseHDU):
             # We have the data to be used.
             # We need to pad the data to a block length before calculating
             # the datasum.
+            bytes_array = self.data.view(type=np.ndarray, dtype=np.ubyte)
+            padding = np.fromstring(_pad_length(self.size) * b' ',
+                                    dtype=np.ubyte)
 
-            d = np.append(self.data.view(dtype='ubyte'),
-                          np.fromstring(_pad_length(self.size) * ' ',
-                                        dtype='ubyte'))
+            d = np.append(bytes_array, padding)
 
             cs = self._compute_checksum(d, blocking=blocking)
             return cs
@@ -798,17 +803,23 @@ class BinTableHDU(_TableBaseHDU):
         Calculate the value for the ``DATASUM`` card given the input data
         """
 
-        swapped = self._binary_table_byte_swap()
-        try:
-            dout = self.data.view(dtype='ubyte')
+        with _binary_table_byte_swap(self.data) as data:
+            dout = data.view(type=np.ndarray, dtype=np.ubyte)
             csum = self._compute_checksum(dout, blocking=blocking)
 
             # Now add in the heap data to the checksum (we can skip any gap
             # between the table and the heap since it's all zeros and doesn't
             # contribute to the checksum
-            for idx in range(self.data._nfields):
-                if isinstance(self.data.columns._recformats[idx], _FormatP):
-                    for coldata in self.data.field(idx):
+            # TODO: The following code may no longer be necessary since it is
+            # now possible to get a pointer directly to the heap data as a
+            # whole.  That said, it is possible for the heap section to contain
+            # data that is not actually pointed to by the table (i.e. garbage;
+            # this *shouldn't* happen but it is not disallowed either)--need to
+            # double check whether or not the checksum should include such
+            # garbage
+            for idx in range(data._nfields):
+                if isinstance(data.columns._recformats[idx], _FormatP):
+                    for coldata in data.field(idx):
                         # coldata should already be byteswapped from the call
                         # to _binary_table_byte_swap
                         if not len(coldata):
@@ -818,9 +829,6 @@ class BinTableHDU(_TableBaseHDU):
                                                       blocking=blocking)
 
             return csum
-        finally:
-            for arr in swapped:
-                arr.byteswap(True)
 
     def _calculate_datasum(self, blocking):
         """
@@ -842,85 +850,68 @@ class BinTableHDU(_TableBaseHDU):
     def _writedata_internal(self, fileobj):
         size = 0
 
-        if self.data is not None:
-            swapped = self._binary_table_byte_swap()
-            try:
-                fileobj.writearray(self.data)
+        if self.data is None:
+            return size
+
+        with _binary_table_byte_swap(self.data) as data:
+            if _has_unicode_fields(data):
+                # If the raw data was a user-supplied recarray, we can't write
+                # unicode columns directly to the file, so we have to switch
+                # to a slower row-by-row write
+                self._writedata_by_row(fileobj)
+            else:
+                fileobj.writearray(data)
                 # write out the heap of variable length array columns this has
                 # to be done after the "regular" data is written (above)
-                fileobj.write((self.data._gap * '\0').encode('ascii'))
+                fileobj.write((data._gap * '\0').encode('ascii'))
 
-                nbytes = self.data._gap
+            nbytes = data._gap
 
-                if not self._manages_own_heap:
-                    # Write the heap data one column at a time, in the order
-                    # that the data pointers appear in the column (regardless
-                    # if that data pointer has a different, previous heap
-                    # offset listed)
-                    for idx in range(self.data._nfields):
-                        if not isinstance(self.data.columns._recformats[idx],
-                                          _FormatP):
-                            continue
+            if not self._manages_own_heap:
+                # Write the heap data one column at a time, in the order
+                # that the data pointers appear in the column (regardless
+                # if that data pointer has a different, previous heap
+                # offset listed)
+                for idx in range(data._nfields):
+                    if not isinstance(data.columns._recformats[idx],
+                                      _FormatP):
+                        continue
 
-                        field = self.data.field(idx)
-                        for row in field:
-                            if len(row) > 0:
-                                nbytes += row.nbytes
-                                if not fileobj.simulateonly:
-                                    fileobj.writearray(row)
-                else:
-                    heap_data = self.data._get_heap_data()
-                    if len(heap_data) > 0:
-                        nbytes += len(heap_data)
-                        if not fileobj.simulateonly:
-                            fileobj.writearray(heap_data)
+                    field = self.data.field(idx)
+                    for row in field:
+                        if len(row) > 0:
+                            nbytes += row.nbytes
+                            if not fileobj.simulateonly:
+                                fileobj.writearray(row)
+            else:
+                heap_data = data._get_heap_data()
+                if len(heap_data) > 0:
+                    nbytes += len(heap_data)
+                    if not fileobj.simulateonly:
+                        fileobj.writearray(heap_data)
 
-                self.data._heapsize = nbytes - self.data._gap
-                size += nbytes
-            finally:
-                for arr in swapped:
-                    arr.byteswap(True)
-            size += self.data.size * self.data.itemsize
+            data._heapsize = nbytes - data._gap
+            size += nbytes
+
+        size += self.data.size * self.data._raw_itemsize
 
         return size
 
-    def _binary_table_byte_swap(self):
-        """Prepares data in the native FITS format and writes the raw bytes
-        out to the given file object.  This handles byte swapping from native
-        to big endian (if necessary).  In addition, however, this also handles
-        writing the binary table heap when variable length array columns are
-        present.
-        """
+    def _writedata_by_row(self, fileobj):
+        fields = [self.data.field(idx)
+                  for idx in range(len(self.data.columns))]
 
-        to_swap = []
+        # Creating Record objects is expensive (as in
+        # `for row in self.data:` so instead we just iterate over the row
+        # indicies and get one field at a time:
+        for idx in range(len(self.data)):
+            for field in fields:
+                item = field[idx]
 
-        if sys.byteorder == 'little':
-            swap_types = ('<', '=')
-        else:
-            swap_types = ('<',)
+                if field.dtype.kind == 'U':
+                    item = np.char.encode(item, 'ascii')
 
-        for idx in range(self.data._nfields):
-            field = _get_recarray_field(self.data, idx)
-            if isinstance(field, chararray.chararray):
-                continue
-
-            # only swap unswapped
-            if field.itemsize > 1 and field.dtype.str[0] in swap_types:
-                to_swap.append(field)
-
-            # deal with var length table
-            recformat = self.data.columns._recformats[idx]
-            if isinstance(recformat, _FormatP):
-                coldata = self.data.field(idx)
-                for c in coldata:
-                    if (not isinstance(c, chararray.chararray) and
-                            c.itemsize > 1 and c.dtype.str[0] in swap_types):
-                        to_swap.append(c)
-
-        for arr in reversed(to_swap):
-            arr.byteswap(True)
-
-        return to_swap
+                fileobj.writearray(item)
 
     _tdump_file_format = textwrap.dedent("""
 
@@ -1036,7 +1027,8 @@ class BinTableHDU(_TableBaseHDU):
         if hfile:
             self._header.tofile(hfile, sep='\n', endcard=False, padding=False)
 
-    dump.__doc__ += _tdump_file_format.replace('\n', '\n        ')
+    if isinstance(dump.__doc__, string_types):
+        dump.__doc__ += _tdump_file_format.replace('\n', '\n        ')
 
     @deprecated('0.1', alternative=':meth:`dump`')
     def tdump(self, datafile=None, cdfile=None, hfile=None, clobber=False):
@@ -1119,7 +1111,10 @@ class BinTableHDU(_TableBaseHDU):
         hdu = cls(data=data, header=header)
         hdu.columns = coldefs
         return hdu
-    load.__doc__ += _tdump_file_format.replace('\n', '\n        ')
+
+    if isinstance(load.__doc__, string_types):
+        load.__doc__ += _tdump_file_format.replace('\n', '\n        ')
+
     load = classmethod(load)
     # Have to create a classmethod from this here instead of as a decorator;
     # otherwise we can't update __doc__
@@ -1465,3 +1460,68 @@ def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
 
     # construct a table HDU of the requested type
     return cls.from_columns(input, header=header, nrows=nrows, fill=fill)
+
+
+@contextlib.contextmanager
+def _binary_table_byte_swap(data):
+    """
+    Ensures that all the data of a binary FITS table (represented as a FITS_rec
+    object) is in a big-endian byte order.  Columns are swapped in-place one
+    at a time, and then returned to their previous byte order when this context
+    manager exits.
+
+    Because a new dtype is needed to represent the byte-swapped columns, the
+    new dtype is temporarily applied as well.
+    """
+
+    orig_dtype = data.dtype
+
+    names = []
+    formats = []
+
+    to_swap = []
+
+    if sys.byteorder == 'little':
+        swap_types = ('<', '=')
+    else:
+        swap_types = ('<',)
+
+    for idx, name in enumerate(orig_dtype.names):
+        field = _get_recarray_field(data, idx)
+
+        field_dtype = orig_dtype.fields[name][0]
+        names.append(name)
+        formats.append(field_dtype)
+
+        if isinstance(field, chararray.chararray):
+            continue
+
+        # only swap unswapped
+        # must use field_dtype.base here since for multi-element dtypes,
+        # the .str with be '|V<N>' where <N> is the total bytes per element
+        if field.itemsize > 1 and field_dtype.base.str[0] in swap_types:
+            to_swap.append(field)
+            # Override the dtype for this field in the new record dtype with
+            # the byteswapped version
+            formats[-1] = field_dtype.newbyteorder()
+
+        # deal with var length table
+        recformat = data.columns._recformats[idx]
+        if isinstance(recformat, _FormatP):
+            coldata = data.field(idx)
+            for c in coldata:
+                if (not isinstance(c, chararray.chararray) and
+                        c.itemsize > 1 and c.dtype.str[0] in swap_types):
+                    to_swap.append(c)
+
+    for arr in reversed(to_swap):
+        arr.byteswap(True)
+
+    data.dtype = np.dtype(list(zip(names, formats)))
+
+    yield data
+
+    for arr in to_swap:
+        arr.byteswap(True)
+
+    data.dtype = orig_dtype
