@@ -579,6 +579,21 @@ class TestDiff(FitsTestCase):
         assert np.isnan(diff.diff_values[1][1][0])
         assert diff.diff_values[1][1][1] == 2.0
 
+    def test_diff_types(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/4122
+        """
+
+        f = io.StringIO()
+
+        a = 1.0
+        b = '1.0'
+
+        report_diff_values(f, a, b)
+        out = f.getvalue()
+
+        assert out.lstrip('u') == "  (float) a> 1.0\n    (str) b> '1.0'\n           ? +   +\n"
+
     def test_float_comparison(self):
         """
         Regression test for https://github.com/spacetelescope/PyFITS/issues/21
