@@ -2,6 +2,9 @@
 # This file connects any readers/writers defined in io.misc to the
 # astropy.table.Table class
 
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
 from .hdf5 import read_table_hdf5, write_table_hdf5, is_hdf5
 
 from .. import registry as io_registry
