@@ -159,7 +159,7 @@ class Converter(object):
 
     Parameters
     ----------
-    field : `~astropy.io.votable.table.Field`
+    field : `~astropy.io.votable.tree.Field`
         object describing the datatype
 
     config : dict
@@ -170,7 +170,7 @@ class Converter(object):
         found.  Used for error messages.
 
     """
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         pass
 
     @staticmethod
@@ -181,7 +181,13 @@ class Converter(object):
     def _write_length(length):
         return struct_pack(">I", int(length))
 
-    def parse(self, value, config={}, pos=None):
+    def supports_empty_values(self, config):
+        """
+        Returns True when the field can be completely empty.
+        """
+        return config.get('version_1_3_or_later')
+
+    def parse(self, value, config=None, pos=None):
         """
         Convert the string *value* from the TABLEDATA_ format into an
         object with the correct native in-memory datatype and mask flag.
@@ -200,7 +206,7 @@ class Converter(object):
         raise NotImplementedError(
             "This datatype must implement a 'parse' method.")
 
-    def parse_scalar(self, value, config={}, pos=None):
+    def parse_scalar(self, value, config=None, pos=None):
         """
         Parse a single scalar of the underlying type of the converter.
         For non-array converters, this is equivalent to parse.  For
@@ -297,7 +303,10 @@ class Char(Converter):
     """
     default = _empty_bytes
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
+        if config is None:
+            config = {}
+
         Converter.__init__(self, field, config, pos)
 
         if field.arraysize is None:
@@ -326,12 +335,15 @@ class Char(Converter):
         else:
             self.parse = self._str_parse
 
-    def _ascii_parse(self, value, config={}, pos=None):
+    def supports_empty_values(self, config):
+        return True
+
+    def _ascii_parse(self, value, config=None, pos=None):
         if self.arraysize != '*' and len(value) > self.arraysize:
             vo_warn(W46, ('char', self.arraysize), config, pos)
         return value.encode('ascii'), False
 
-    def _str_parse(self, value, config={}, pos=None):
+    def _str_parse(self, value, config=None, pos=None):
         if self.arraysize != '*' and len(value) > self.arraysize:
             vo_warn(W46, ('char', self.arraysize), config, pos)
         return value.encode('utf-8'), False
@@ -373,7 +385,7 @@ class UnicodeChar(Converter):
     """
     default = ''
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         Converter.__init__(self, field, config, pos)
 
         if field.arraysize is None:
@@ -395,7 +407,7 @@ class UnicodeChar(Converter):
             self.binoutput = self._binoutput_fixed
             self._struct_format = ">%ds" % (self.arraysize * 2)
 
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if self.arraysize != '*' and len(value) > self.arraysize:
             vo_warn(W46, ('unicodeChar', self.arraysize), config, pos)
         return value, False
@@ -433,23 +445,24 @@ class Array(Converter):
     """
     Handles both fixed and variable-lengths arrays.
     """
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
+        if config is None:
+            config = {}
         Converter.__init__(self, field, config, pos)
-
         if config.get('pedantic'):
             self._splitter = self._splitter_pedantic
         else:
             self._splitter = self._splitter_lax
 
-    def parse_scalar(self, value, config={}, pos=0):
+    def parse_scalar(self, value, config=None, pos=0):
         return self._base.parse_scalar(value, config, pos)
 
     @staticmethod
-    def _splitter_pedantic(value, config={}, pos=None):
+    def _splitter_pedantic(value, config=None, pos=None):
         return pedantic_array_splitter.split(value)
 
     @staticmethod
-    def _splitter_lax(value, config={}, pos=None):
+    def _splitter_lax(value, config=None, pos=None):
         if ',' in value:
             vo_warn(W01, (), config, pos)
         return array_splitter.split(value)
@@ -461,7 +474,7 @@ class VarArray(Array):
     """
     format = 'O'
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
+    def __init__(self, field, base, arraysize, config=None, pos=None):
         Array.__init__(self, field, config)
 
         self._base = base
@@ -469,7 +482,7 @@ class VarArray(Array):
 
     def output(self, value, mask):
         output = self._base.output
-        result = [output(x, m) for x, m in np.broadcast(value, value.mask)]
+        result = [output(x, m) for x, m in np.broadcast(value, mask)]
         return ' '.join(result)
 
     def binparse(self, read):
@@ -502,7 +515,7 @@ class ArrayVarArray(VarArray):
     Handles an array of variable-length arrays, i.e. where *arraysize*
     ends in '*'.
     """
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value.strip() == '':
             return ma.array([]), False
 
@@ -525,7 +538,7 @@ class ScalarVarArray(VarArray):
     """
     Handles a variable-length array of numeric scalars.
     """
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value.strip() == '':
             return ma.array([]), False
 
@@ -548,7 +561,7 @@ class NumericArray(Array):
     """
     vararray_type = ArrayVarArray
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
+    def __init__(self, field, base, arraysize, config=None, pos=None):
         Array.__init__(self, field, config, pos)
 
         self._base = base
@@ -565,7 +578,9 @@ class NumericArray(Array):
         self.default = np.empty(arraysize, dtype=self._base.format)
         self.default[...] = self._base.default
 
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
+        if config is None:
+            config = {}
         if config['version_1_3_or_later'] and value == '':
             return np.zeros(self._arraysize, dtype=self._base.format), True
         parts = self._splitter(value, config, pos)
@@ -583,7 +598,7 @@ class NumericArray(Array):
                          ([self._base.default] * (self._items - len(parts))))
             return self.parse_parts(parts, config, pos)
 
-    def parse_parts(self, parts, config={}, pos=None):
+    def parse_parts(self, parts, config=None, pos=None):
         base_parse = self._base.parse
         result = []
         result_mask = []
@@ -624,7 +639,7 @@ class Numeric(Converter):
     vararray_type = ScalarVarArray
     null = None
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         Converter.__init__(self, field, config, pos)
 
         self._memsize = np.dtype(self.format).itemsize
@@ -651,7 +666,10 @@ class FloatingPoint(Numeric):
     """
     default = np.nan
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
+        if config is None:
+            config = {}
+
         Numeric.__init__(self, field, config, pos)
 
         precision = field.precision
@@ -662,7 +680,7 @@ class FloatingPoint(Numeric):
             format_parts.append(six.text_type(width))
 
         if precision is None:
-            format_parts.append('s')
+            format_parts.append('r')
         elif precision.startswith("E"):
             format_parts.append('.%dg' % int(precision[1:]))
         elif precision.startswith("F"):
@@ -688,13 +706,16 @@ class FloatingPoint(Numeric):
         else:
             self.parse = self._parse_permissive
 
-    def _parse_pedantic(self, value, config={}, pos=None):
+    def supports_empty_values(self, config):
+        return True
+
+    def _parse_pedantic(self, value, config=None, pos=None):
         if value.strip() == '':
             return self.null, True
         f = float(value)
         return f, self.is_null(f)
 
-    def _parse_permissive(self, value, config={}, pos=None):
+    def _parse_permissive(self, value, config=None, pos=None):
         try:
             f = float(value)
             return f, self.is_null(f)
@@ -713,8 +734,12 @@ class FloatingPoint(Numeric):
         if mask:
             return self._null_output
         if np.isfinite(value):
+            if not np.isscalar(value):
+                value = value.dtype.type(value)
             result = self._output_format % value
-            if (self._output_format[-1] == 's' and
+            if result.startswith('array'):
+                raise RuntimeError()
+            if (self._output_format[-1] == 'r' and
                 result.endswith('.0')):
                 result = result[:-2]
             return result
@@ -762,10 +787,12 @@ class Integer(Numeric):
     """
     default = 0
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         Numeric.__init__(self, field, config, pos)
 
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
+        if config is None:
+            config = {}
         mask = False
         if isinstance(value, six.string_types):
             value = value.lower()
@@ -870,10 +897,7 @@ class ComplexArrayVarArray(VarArray):
     """
     Handles an array of variable-length arrays of complex numbers.
     """
-    def __init__(self, field, base, arraysize, config={}, pos=None):
-        VarArray.__init__(self, field, base, arraysize, config, pos)
-
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value.strip() == '':
             return ma.array([]), True
 
@@ -896,10 +920,7 @@ class ComplexVarArray(VarArray):
     """
     Handles a variable-length array of complex numbers.
     """
-    def __init__(self, field, base, arraysize, config={}, pos=None):
-        VarArray.__init__(self, field, base, arraysize, config, pos)
-
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value.strip() == '':
             return ma.array([]), True
 
@@ -923,17 +944,17 @@ class ComplexArray(NumericArray):
     """
     vararray_type = ComplexArrayVarArray
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
+    def __init__(self, field, base, arraysize, config=None, pos=None):
         NumericArray.__init__(self, field, base, arraysize, config, pos)
         self._items *= 2
 
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         parts = self._splitter(value, config, pos)
         if parts == ['']:
             parts = []
         return self.parse_parts(parts, config, pos)
 
-    def parse_parts(self, parts, config={}, pos=None):
+    def parse_parts(self, parts, config=None, pos=None):
         if len(parts) != self._items:
             vo_raise(E02, (self._items, len(parts)), config, pos)
         base_parse = self._base.parse_parts
@@ -959,12 +980,13 @@ class Complex(FloatingPoint, Array):
     vararray_type = ComplexVarArray
     default = np.nan
 
-    def __init__(self, field, config={}, pos=None):
+    def __init__(self, field, config=None, pos=None):
         FloatingPoint.__init__(self, field, config, pos)
         Array.__init__(self, field, config, pos)
 
-    def parse(self, value, config={}, pos=None):
-        if value.strip() == '':
+    def parse(self, value, config=None, pos=None):
+        stripped = value.strip()
+        if stripped == '' or stripped.lower() == 'nan':
             return np.nan, True
         splitter = self._splitter
         parts = [float(x) for x in splitter(value, config, pos)]
@@ -974,7 +996,7 @@ class Complex(FloatingPoint, Array):
     _parse_permissive = parse
     _parse_pedantic = parse
 
-    def parse_parts(self, parts, config={}, pos=None):
+    def parse_parts(self, parts, config=None, pos=None):
         value = complex(*parts)
         return value, self.is_null(value)
 
@@ -984,9 +1006,9 @@ class Complex(FloatingPoint, Array):
                 return 'NaN'
             else:
                 value = self.null
-        real = self._output_format % value.real
-        imag = self._output_format % value.imag
-        if self._output_format[-1] == 's':
+        real = self._output_format % float(value.real)
+        imag = self._output_format % float(value.imag)
+        if self._output_format[-1] == 'r':
             if real.endswith('.0'):
                 real = real[:-2]
             if imag.endswith('.0'):
@@ -1016,22 +1038,24 @@ class BitArray(NumericArray):
     """
     vararray_type = ArrayVarArray
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
+    def __init__(self, field, base, arraysize, config=None, pos=None):
         NumericArray.__init__(self, field, base, arraysize, config, pos)
 
         self._bytes = ((self._items - 1) // 8) + 1
 
     @staticmethod
-    def _splitter_pedantic(value, config={}, pos=None):
+    def _splitter_pedantic(value, config=None, pos=None):
         return list(re.sub('\s', '', value))
 
     @staticmethod
-    def _splitter_lax(value, config={}, pos=None):
+    def _splitter_lax(value, config=None, pos=None):
         if ',' in value:
             vo_warn(W01, (), config, pos)
         return list(re.sub('\s|,', '', value))
 
     def output(self, value, mask):
+        if np.any(mask):
+            vo_warn(W39)
         value = np.asarray(value)
         mapping = {False: '0', True: '1'}
         return ''.join(mapping[x] for x in value.flat)
@@ -1061,10 +1085,9 @@ class Bit(Converter):
     binary_one = b'\x08'
     binary_zero = b'\0'
 
-    def __init__(self, field, config={}, pos=None):
-        Converter.__init__(self, field, config, pos)
-
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
+        if config is None:
+            config = {}
         mapping = {'1': True, '0': False}
         if value is False or value.strip() == '':
             if not config['version_1_3_or_later']:
@@ -1104,9 +1127,6 @@ class BooleanArray(NumericArray):
     """
     vararray_type = ArrayVarArray
 
-    def __init__(self, field, base, arraysize, config={}, pos=None):
-        NumericArray.__init__(self, field, base, arraysize, config, pos)
-
     def binparse(self, read):
         data = read(self._items)
         binparse = self._base.binparse_value
@@ -1145,10 +1165,7 @@ class Boolean(Converter):
     binary_true = b'T'
     binary_false = b'F'
 
-    def __init__(self, field, config={}, pos=None):
-        Converter.__init__(self, field, config, pos)
-
-    def parse(self, value, config={}, pos=None):
+    def parse(self, value, config=None, pos=None):
         if value == '':
             return False, True
         if value is False:
@@ -1219,7 +1236,7 @@ converter_mapping = {
     'unicodeChar'   : UnicodeChar }
 
 
-def get_converter(field, config={}, pos=None):
+def get_converter(field, config=None, pos=None):
     """
     Get an appropriate converter instance for a given field.
 
@@ -1237,6 +1254,9 @@ def get_converter(field, config={}, pos=None):
     -------
     converter : astropy.io.votable.converters.Converter
     """
+    if config is None:
+        config = {}
+
     if field.datatype not in converter_mapping:
         vo_raise(E06, (field.datatype, field.ID), config)
 
