@@ -1,9 +1,9 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+from __future__ import absolute_import
+
 from distutils.core import Extension
 from os.path import join
 
-from astropy_helpers import setup_helpers
-
 
 def get_extensions(build_type='release'):
     VO_DIR = 'astropy/io/votable/src'
