@@ -13,7 +13,6 @@ from ....extern.six.moves import xrange
 
 # STDLIB
 import difflib
-from distutils import version
 import io
 import os
 import shutil
@@ -612,6 +611,21 @@ class TestParse:
         assert fields[0].name == "int"
         assert fields[0].values.min == -1000
 
+    def test_get_info_by_id(self):
+        info = self.votable.get_info_by_id('QUERY_STATUS')
+        assert info.value == 'OK'
+
+        if self.votable.version != '1.1':
+            info = self.votable.get_info_by_id("ErrorInfo")
+            assert info.value == "One might expect to find some INFO here, too..."
+
+    def test_repr(self):
+        assert '3 tables' in repr(self.votable)
+        assert repr(list(self.votable.iter_fields_and_params())[0]) == \
+            '<PARAM ID="awesome" arraysize="*" datatype="float" name="INPUT" unit="deg" value="[0.0 0.0]"/>'
+        # Smoke test
+        repr(list(self.votable.iter_groups()))
+
 
 class TestThroughTableData(TestParse):
     def setup_class(self):
@@ -724,7 +738,9 @@ def test_open_files():
     def test_file(filename):
         parse(filename, pedantic=False)
 
-    for filename in get_pkg_data_filenames('data', '*.xml'):
+    for filename in get_pkg_data_filenames('data', pattern='*.xml'):
+        if filename.endswith('custom_datatype.xml'):
+            continue
         yield test_file, filename
 
 
@@ -992,3 +1008,14 @@ def test_instantiate_vowarning():
     # This used to raise a deprecation exception on Python 2.6.
     # See https://github.com/astropy/astroquery/pull/276
     VOWarning(())
+
+
+def test_custom_datatype():
+    votable = parse(
+        get_pkg_data_filename('data/custom_datatype.xml'),
+        pedantic=False,
+        datatype_mapping={'bar': 'int'}
+    )
+
+    table = votable.get_first_table()
+    assert table.array.dtype['foo'] == np.int32
