@@ -24,7 +24,7 @@ from .. import exceptions
 from .. import xmlutil
 
 
-class Result:
+class Result(object):
     def __init__(self, url, root='results', timeout=10):
         self.url = url
         m = hashlib.md5()
