@@ -12,16 +12,21 @@ import numpy as np
 from .core import (Fittable1DModel, Fittable2DModel, Model,
                    ModelDefinitionError, custom_model)
 from .parameters import Parameter, InputParameterError
+from .utils import ellipse_extent
 from ..utils import deprecated
+from ..utils.compat.funcsigs import signature
 from ..extern import six
 
+from .utils import get_inputs_and_params
+
+
 __all__ = sorted([
     'AiryDisk2D', 'Moffat1D', 'Moffat2D', 'Box1D',
     'Box2D', 'Const1D', 'Const2D', 'Ellipse2D', 'Disk2D',
     'Gaussian1D', 'GaussianAbsorption1D', 'Gaussian2D', 'Linear1D',
-    'Lorentz1D', 'MexicanHat1D', 'MexicanHat2D', 'Scale', 'Redshift', 'Shift',
-    'Sine1D', 'Trapezoid1D', 'TrapezoidDisk2D', 'Ring2D',
-    'custom_model_1d'
+    'Lorentz1D', 'MexicanHat1D', 'MexicanHat2D', 'Redshift', 'Scale',
+    'Sersic1D', 'Sersic2D', 'Shift', 'Sine1D', 'Trapezoid1D', 'TrapezoidDisk2D',
+    'Ring2D', 'custom_model_1d', 'Voigt1D'
 ])
 
 
@@ -92,6 +97,38 @@ class Gaussian1D(Fittable1DModel):
     mean = Parameter(default=0)
     stddev = Parameter(default=1)
 
+    def bounding_box(self, factor=5.5):
+        """
+        Tuple defining the default ``bounding_box`` limits,
+        ``(x_low, x_high)``
+
+        Parameters
+        ----------
+        factor : float
+            The multiple of `stddev` used to define the limits.
+            The default is 5.5, corresponding to a relative error < 1e-7.
+
+        Examples
+        --------
+        >>> from astropy.modeling.models import Gaussian1D
+        >>> model = Gaussian1D(mean=0, stddev=2)
+        >>> model.bounding_box
+        (-11.0, 11.0)
+
+        This range can be set directly (see: `Model.bounding_box
+        <astropy.modeling.Model.bounding_box>`) or by using a different factor,
+        like:
+
+        >>> model.bounding_box = model.bounding_box(factor=2)
+        >>> model.bounding_box
+        (-4.0, 4.0)
+        """
+
+        x0 = self.mean.value
+        dx = factor * self.stddev
+
+        return (x0 - dx, x0 + dx)
+
     @staticmethod
     def evaluate(x, amplitude, mean, stddev):
         """
@@ -140,6 +177,37 @@ class GaussianAbsorption1D(Fittable1DModel):
     mean = Parameter(default=0)
     stddev = Parameter(default=1)
 
+    def bounding_box(self, factor=5.5):
+        """
+        Tuple defining the default ``bounding_box`` limits,
+        ``(x_low, x_high)``
+
+        Parameters
+        ----------
+        factor : float
+            The multiple of `stddev` used to define the limits.
+            The default is 5.5, corresponding to a relative error < 1e-7.
+
+        Examples
+        --------
+        >>> from astropy.modeling.models import Gaussian1D
+        >>> model = Gaussian1D(mean=0, stddev=2)
+        >>> model.bounding_box
+        (-11.0, 11.0)
+
+        This range can be set directly (see: ``help(model.bounding_box)``) or by
+        using a different factor, like:
+
+        >>> model.bounding_box = model.bounding_box(factor=2)
+        >>> model.bounding_box
+        (-4.0, 4.0)
+        """
+
+        x0 = self.mean.value
+        dx = factor * self.stddev
+
+        return (x0 - dx, x0 + dx)
+
     @staticmethod
     def evaluate(x, amplitude, mean, stddev):
         """
@@ -241,7 +309,6 @@ class Gaussian2D(Fittable2DModel):
     y_stddev = Parameter(default=1)
     theta = Parameter(default=0)
 
-
     def __init__(self, amplitude=amplitude.default, x_mean=x_mean.default,
                  y_mean=y_mean.default, x_stddev=None, y_stddev=None,
                  theta=0.0, cov_matrix=None, **kwargs):
@@ -282,6 +349,44 @@ class Gaussian2D(Fittable2DModel):
             amplitude=amplitude, x_mean=x_mean, y_mean=y_mean,
             x_stddev=x_stddev, y_stddev=y_stddev, theta=theta, **kwargs)
 
+    def bounding_box(self, factor=5.5):
+        """
+        Tuple defining the default ``bounding_box`` limits in each dimension,
+        ``((y_low, y_high), (x_low, x_high))``
+
+        The default offset from the mean is 5.5-sigma, corresponding
+        to a relative error < 1e-7. The limits are adjusted for rotation.
+
+        Parameters
+        ----------
+        factor : float, optional
+            The multiple of `x_stddev` and `y_stddev` used to define the limits.
+            The default is 5.5.
+
+        Examples
+        --------
+        >>> from astropy.modeling.models import Gaussian2D
+        >>> model = Gaussian2D(x_mean=0, y_mean=0, x_stddev=1, y_stddev=2)
+        >>> model.bounding_box
+        ((-11.0, 11.0), (-5.5, 5.5))
+
+        This range can be set directly (see: `Model.bounding_box
+        <astropy.modeling.Model.bounding_box>`) or by using a different factor
+        like:
+
+        >>> model.bounding_box = model.bounding_box(factor=2)
+        >>> model.bounding_box
+        ((-4.0, 4.0), (-2.0, 2.0))
+        """
+
+        a = factor * self.x_stddev
+        b = factor * self.y_stddev
+        theta = self.theta.value
+        dx, dy = ellipse_extent(a, b, theta)
+
+        return ((self.y_mean - dy, self.y_mean + dy),
+                (self.x_mean - dx, self.x_mean + dx))
+
     @staticmethod
     def evaluate(x, y, amplitude, x_mean, y_mean, x_stddev, y_stddev, theta):
         """Two dimensional Gaussian function"""
@@ -440,6 +545,92 @@ class Redshift(Fittable1DModel):
         return inv
 
 
+class Sersic1D(Fittable1DModel):
+    r"""
+    One dimensional Sersic surface brightness profile.
+
+    Parameters
+    ----------
+    amplitude : float
+        Central surface brightness, within r_eff.
+    r_eff : float
+        Effective (half-light) radius
+    n : float
+        Sersic Index.
+
+    See Also
+    --------
+    Gaussian1D, Moffat1D, Lorentz1D
+
+    Notes
+    -----
+    Model formula:
+
+    .. math::
+
+        I(r)=I_e exp\left[-b_n\left(\frac{r}{r_{e}}\right)^{(1/n)}-1\right]
+
+    The constant :math:`b_n` is defined such that :math:`r_e` contains half the total
+    luminosity, and can be solved for numerically.
+
+    .. math::
+
+        \Gamma(2n) = 2\gamma (b_n,2n)
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        from astropy.modeling.models import Sersic1D
+        import matplotlib.pyplot as plt
+
+        plt.figure()
+        plt.subplot(111, xscale='log', yscale='log')
+        s1 = Sersic1D(amplitude=1, r_eff=5)
+        r=np.arange(0, 100, .01)
+
+        for n in range(1, 10):
+             s1.n = n
+             plt.plot(r, s1(r), color=str(float(n) / 15))
+
+        plt.axis([1e-1, 30, 1e-2, 1e3])
+        plt.xlabel('log Radius')
+        plt.ylabel('log Surface Brightness')
+        plt.text(.25, 1.5, 'n=1')
+        plt.text(.25, 300, 'n=10')
+        plt.xticks([])
+        plt.yticks([])
+        plt.show()
+
+    References
+    ----------
+    .. [1] http://ned.ipac.caltech.edu/level5/March05/Graham/Graham2.html
+    """
+
+    amplitude = Parameter(default=1)
+    r_eff = Parameter(default=1)
+    n = Parameter(default=4)
+    _gammaincinv = None
+
+    def __init__(self, amplitude=amplitude.default, r_eff=r_eff.default,
+                 n=n.default, **kwargs):
+        try:
+            from scipy.special import gammaincinv
+            self.__class__._gammaincinv = gammaincinv
+        except ValueError:
+            raise ImportError("Sersic1D model requires scipy > 0.11.")
+
+        super(Sersic1D, self).__init__(
+            amplitude=amplitude, r_eff=r_eff, n=n, **kwargs)
+
+    @classmethod
+    def evaluate(cls, r, amplitude, r_eff, n):
+        """One dimensional Sersic profile function."""
+        return amplitude * np.exp(-cls._gammaincinv(2 * n, 0.5) * ((r / r_eff) ** (1 / n) - 1))
+
+
 class Sine1D(Fittable1DModel):
     """
     One dimensional Sine model.
@@ -450,6 +641,8 @@ class Sine1D(Fittable1DModel):
         Oscillation amplitude
     frequency : float
         Oscillation frequency
+    phase : float
+        Oscillation phase
 
     See Also
     --------
@@ -460,26 +653,29 @@ class Sine1D(Fittable1DModel):
     -----
     Model formula:
 
-        .. math:: f(x) = A \\sin(2 \\pi f x)
+        .. math:: f(x) = A \\sin(2 \\pi f x + 2 \\pi p)
     """
 
     amplitude = Parameter(default=1)
     frequency = Parameter(default=1)
+    phase = Parameter(default=0)
 
     @staticmethod
-    def evaluate(x, amplitude, frequency):
+    def evaluate(x, amplitude, frequency, phase):
         """One dimensional Sine model function"""
 
-        return amplitude * np.sin(2 * np.pi * frequency * x)
+        return amplitude * np.sin(2 * np.pi * frequency * x + 2 * np.pi * phase)
 
     @staticmethod
-    def fit_deriv(x, amplitude, frequency):
+    def fit_deriv(x, amplitude, frequency, phase):
         """One dimensional Sine model derivative"""
 
-        d_amplitude = np.sin(2 * np.pi * frequency * x)
+        d_amplitude = np.sin(2 * np.pi * frequency * x + 2 * np.pi * phase)
         d_frequency = (2 * np.pi * x * amplitude *
-                       np.cos(2 * np.pi * frequency * x))
-        return [d_amplitude, d_frequency]
+                       np.cos(2 * np.pi * frequency * x + 2 * np.pi * phase))
+        d_phase = (2 * np.pi * amplitude *
+                   np.cos(2 * np.pi * frequency * x + 2 * np.pi * phase))
+        return [d_amplitude, d_frequency, d_phase]
 
 
 class Linear1D(Fittable1DModel):
@@ -523,6 +719,12 @@ class Linear1D(Fittable1DModel):
         d_intercept = np.ones_like(x)
         return [d_slope, d_intercept]
 
+    @property
+    def inverse(self):
+        new_slope = self.slope ** -1
+        new_intercept = -self.intercept / self.slope
+        return self.__class__(slope=new_slope, intercept=new_intercept)
+
 
 class Lorentz1D(Fittable1DModel):
     """
@@ -559,7 +761,7 @@ class Lorentz1D(Fittable1DModel):
         """One dimensional Lorentzian model function"""
 
         return (amplitude * ((fwhm / 2.) ** 2) / ((x - x_0) ** 2 +
-                (fwhm / 2.) ** 2))
+                                                  (fwhm / 2.) ** 2))
 
     @staticmethod
     def fit_deriv(x, amplitude, x_0, fwhm):
@@ -572,6 +774,98 @@ class Lorentz1D(Fittable1DModel):
         return [d_amplitude, d_x_0, d_fwhm]
 
 
+class Voigt1D(Fittable1DModel):
+    """
+    One dimensional model for the Voigt profile.
+
+    Parameters
+    ----------
+    x_0 : float
+        Position of the peak
+    amplitude_L : float
+        The Lorentzian amplitude
+    fwhm_L : float
+        The Lorentzian full width at half maximum
+    fwhm_G : float
+        The Gaussian full width at half maximum
+
+    See Also
+    --------
+    Gaussian1D, Lorentz1D
+
+    Notes
+    -----
+    Algorithm for the computation taken from
+    McLean, A. B., Mitchell, C. E. J. & Swanston, D. M. Implementation of an
+    efficient analytical approximation to the Voigt function for photoemission
+    lineshape analysis. Journal of Electron Spectroscopy and Related Phenomena
+    69, 125-132 (1994)
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        from astropy.modeling.models import Voigt1D
+        import matplotlib.pyplot as plt
+
+        plt.figure()
+        x = np.arange(0, 10, 0.01)
+        v1 = Voigt1D(x_0=5, amplitude_L=10, fwhm_L=0.5, fwhm_G=0.9)
+        plt.plot(x, v1(x))
+        plt.show()
+    """
+
+    x_0 = Parameter(default=0)
+    amplitude_L = Parameter(default=1)
+    fwhm_L = Parameter(default=2/np.pi)
+    fwhm_G = Parameter(default=np.log(2))
+
+    _abcd = np.array([
+        [-1.2150, -1.3509, -1.2150, -1.3509],  # A
+        [1.2359, 0.3786, -1.2359, -0.3786],    # B
+        [-0.3085, 0.5906, -0.3085, 0.5906],    # C
+        [0.0210, -1.1858, -0.0210, 1.1858]])   # D
+
+    @classmethod
+    def evaluate(cls, x, x_0, amplitude_L, fwhm_L, fwhm_G):
+
+        A, B, C, D = cls._abcd
+        sqrt_ln2 = np.sqrt(np.log(2))
+        X = (x - x_0) * 2 * sqrt_ln2 / fwhm_G
+        X = np.atleast_1d(X)[..., np.newaxis]
+        Y = fwhm_L * sqrt_ln2 / fwhm_G
+        Y = np.atleast_1d(Y)[..., np.newaxis]
+
+        V = np.sum((C * (Y - A) + D * (X - B))/(((Y - A) ** 2 + (X - B) ** 2)), axis=-1)
+
+        return (fwhm_L * amplitude_L * np.sqrt(np.pi) * sqrt_ln2 / fwhm_G) * V
+
+    @classmethod
+    def fit_deriv(cls, x, x_0, amplitude_L, fwhm_L, fwhm_G):
+
+        A, B, C, D = cls._abcd
+        sqrt_ln2 = np.sqrt(np.log(2))
+        X = (x - x_0) * 2 * sqrt_ln2 / fwhm_G
+        X = np.atleast_1d(X)[:, np.newaxis]
+        Y = fwhm_L * sqrt_ln2 / fwhm_G
+        Y = np.atleast_1d(Y)[:, np.newaxis]
+        constant = fwhm_L * amplitude_L * np.sqrt(np.pi) * sqrt_ln2 / fwhm_G
+
+        alpha = C * (Y - A) + D * (X - B)
+        beta = (Y - A) ** 2 + (X - B) ** 2
+        V = np.sum((alpha / beta), axis=-1)
+        dVdx = np.sum((D/beta - 2 * (X - B) * alpha / np.square(beta)), axis=-1)
+        dVdy = np.sum((C/beta - 2 * (Y - A) * alpha / np.square(beta)), axis=-1)
+
+        dyda = [-constant * dVdx * 2 * sqrt_ln2 / fwhm_G,
+                constant * V / amplitude_L,
+                constant * (V / fwhm_L + dVdy * sqrt_ln2 / fwhm_G),
+                -constant * (V + (sqrt_ln2 / fwhm_G) * (2 * (x - x_0) * dVdx + fwhm_L * dVdy)) / fwhm_G]
+        return dyda
+
+
 class Const1D(Fittable1DModel):
     """
     One dimensional Constant model.
@@ -678,7 +972,7 @@ class Ellipse2D(Fittable2DModel):
     b : float
         The length of the semiminor axis.
 
-    theta : float, optional
+    theta : float
         The rotation angle in radians of the semimajor axis.  The
         rotation angle increases counterclockwise from the positive x
         axis.
@@ -721,18 +1015,18 @@ class Ellipse2D(Fittable2DModel):
         y, x = np.mgrid[0:50, 0:50]
         fig, ax = plt.subplots(1, 1)
         ax.imshow(e(x, y), origin='lower', interpolation='none', cmap='Greys_r')
-        e2 = mpatches.Ellipse((x0, y0), 2*a, 2*b, theta.degree,
-                              edgecolor='red', facecolor='none')
+        e2 = mpatches.Ellipse((x0, y0), 2*a, 2*b, theta.degree, edgecolor='red',
+                              facecolor='none')
         ax.add_patch(e2)
         plt.show()
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    y_0 = Parameter()
-    a = Parameter()
-    b = Parameter()
-    theta = Parameter()
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    a = Parameter(default=1)
+    b = Parameter(default=1)
+    theta = Parameter(default=0)
 
     @staticmethod
     def evaluate(x, y, amplitude, x_0, y_0, a, b, theta):
@@ -747,6 +1041,22 @@ class Ellipse2D(Fittable2DModel):
         in_ellipse = (((numerator1 / a) ** 2 + (numerator2 / b) ** 2) <= 1.)
         return np.select([in_ellipse], [amplitude])
 
+    @property
+    def bounding_box(self):
+        """
+        Tuple defining the default ``bounding_box`` limits.
+
+        ``((y_low, y_high), (x_low, x_high))``
+        """
+
+        a = self.a
+        b = self.b
+        theta = self.theta.value
+        dx, dy = ellipse_extent(a, b, theta)
+
+        return ((self.y_0 - dy, self.y_0 + dy),
+                (self.x_0 - dx, self.x_0 + dx))
+
 
 class Disk2D(Fittable2DModel):
     """
@@ -793,9 +1103,19 @@ class Disk2D(Fittable2DModel):
         rr = (x - x_0) ** 2 + (y - y_0) ** 2
         return np.select([rr <= R_0 ** 2], [amplitude])
 
+    @property
+    def bounding_box(self):
+        """
+        Tuple defining the default ``bounding_box`` limits.
+
+        ``((y_low, y_high), (x_low, x_high))``
+        """
+
+        return ((self.y_0 - self.R_0, self.y_0 + self.R_0),
+                (self.x_0 - self.R_0, self.x_0 + self.R_0))
+
 
 class Ring2D(Fittable2DModel):
-
     """
     Two dimensional radial symmetric Ring model.
 
@@ -863,6 +1183,19 @@ class Ring2D(Fittable2DModel):
         r_range = np.logical_and(rr >= r_in ** 2, rr <= (r_in + width) ** 2)
         return np.select([r_range], [amplitude])
 
+    @property
+    def bounding_box(self):
+        """
+        Tuple defining the default ``bounding_box``.
+
+        ``((y_low, y_high), (x_low, x_high))``
+        """
+
+        dr = self.r_in + self.width
+
+        return ((self.y_0 - dr, self.y_0 + dr),
+                (self.x_0 - dr, self.x_0 + dr))
+
 
 class Delta1D(Fittable1DModel):
     """One dimensional Dirac delta function."""
@@ -930,6 +1263,18 @@ class Box1D(Fittable1DModel):
         d_width = np.zeros_like(x)
         return [d_amplitude, d_x_0, d_width]
 
+    @property
+    def bounding_box(self):
+        """
+        Tuple defining the default ``bounding_box`` limits.
+
+        ``(x_low, x_high))``
+        """
+
+        dx = self.width / 2
+
+        return (self.x_0 - dx, self.x_0 + dx)
+
 
 class Box2D(Fittable2DModel):
     """
@@ -984,6 +1329,20 @@ class Box2D(Fittable2DModel):
                                  y <= y_0 + y_width / 2.)
         return np.select([np.logical_and(x_range, y_range)], [amplitude], 0)
 
+    @property
+    def bounding_box(self):
+        """
+        Tuple defining the default ``bounding_box``.
+
+        ``((y_low, y_high), (x_low, x_high))``
+        """
+
+        dx = self.x_width / 2
+        dy = self.y_width / 2
+
+        return ((self.y_0 - dy, self.y_0 + dy),
+                (self.x_0 - dx, self.x_0 + dx))
+
 
 class Trapezoid1D(Fittable1DModel):
     """
@@ -1030,6 +1389,18 @@ class Trapezoid1D(Fittable1DModel):
         val_c = slope * (x4 - x)
         return np.select([range_a, range_b, range_c], [val_a, val_b, val_c])
 
+    @property
+    def bounding_box(self):
+        """
+        Tuple defining the default ``bounding_box`` limits.
+
+        ``(x_low, x_high))``
+        """
+
+        dx = self.width / 2 + self.amplitude / self.slope
+
+        return (self.x_0 - dx, self.x_0 + dx)
+
 
 class TrapezoidDisk2D(Fittable2DModel):
     """
@@ -1065,11 +1436,24 @@ class TrapezoidDisk2D(Fittable2DModel):
 
         r = np.sqrt((x - x_0) ** 2 + (y - y_0) ** 2)
         range_1 = r <= R_0
-        range_2 = np.logical_and(r > R_0,  r <= R_0 + amplitude / slope)
+        range_2 = np.logical_and(r > R_0, r <= R_0 + amplitude / slope)
         val_1 = amplitude
         val_2 = amplitude + slope * (R_0 - r)
         return np.select([range_1, range_2], [val_1, val_2])
 
+    @property
+    def bounding_box(self):
+        """
+        Tuple defining the default ``bounding_box``.
+
+        ``((y_low, y_high), (x_low, x_high))``
+        """
+
+        dr = self.R_0 + self.amplitude / self.slope
+
+        return ((self.y_0 - dr, self.y_0 + dr),
+                (self.x_0 - dr, self.x_0 + dr))
+
 
 class MexicanHat1D(Fittable1DModel):
     """
@@ -1240,7 +1624,7 @@ class AiryDisk2D(Fittable2DModel):
         # separately so as not to raise a numpy warning
         z = np.ones(r.shape)
         rt = np.pi * r[r > 0]
-        z[r > 0] = (2.0 * cls._j1(rt) / rt)**2
+        z[r > 0] = (2.0 * cls._j1(rt) / rt) ** 2
         z *= amplitude
         return z
 
@@ -1356,16 +1740,121 @@ class Moffat2D(Fittable2DModel):
         return [d_A, d_x_0, d_y_0, d_gamma, d_alpha]
 
 
+class Sersic2D(Fittable2DModel):
+    r"""
+    Two dimensional Sersic surface brightness profile.
+
+    Parameters
+    ----------
+    amplitude : float
+        Central surface brightness, within r_eff.
+    r_eff : float
+        Effective (half-light) radius
+    n : float
+        Sersic Index.
+    x_0 : float, optional
+        x position of the center.
+    y_0 : float, optional
+        y position of the center.
+    ellip : float, optional
+        Ellipticity.
+    theta : float, optional
+        Rotation angle in radians, counterclockwise from
+        the positive x-axis.
+
+    See Also
+    --------
+    Gaussian2D, Moffat2D
+
+    Notes
+    -----
+    Model formula:
+
+    .. math::
+
+        I(x,y) = I(r) = I_e\exp\left[-b_n\left(\frac{r}{r_{e}}\right)^{(1/n)}-1\right]
+
+    The constant :math:`b_n` is defined such that :math:`r_e` contains half the total
+    luminosity, and can be solved for numerically.
+
+    .. math::
+
+        \Gamma(2n) = 2\gamma (b_n,2n)
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        from astropy.modeling.models import Sersic2D
+        import matplotlib.pyplot as plt
+
+        x,y = np.meshgrid(np.arange(100), np.arange(100))
+
+        mod = Sersic2D(amplitude = 1, r_eff = 25, n=4, x_0=50, y_0=50,
+                       ellip=.5, theta=-1)
+        img = mod(x, y)
+        log_img = np.log10(img)
+
+
+        plt.figure()
+        plt.imshow(log_img, origin='lower', interpolation='nearest',
+                   vmin=-1, vmax=2)
+        plt.xlabel('x')
+        plt.ylabel('y')
+        cbar = plt.colorbar()
+        cbar.set_label('Log Brightness', rotation=270, labelpad=25)
+        cbar.set_ticks([-1, 0, 1, 2], update_ticks=True)
+        plt.show()
+
+    References
+    ----------
+    .. [1] http://ned.ipac.caltech.edu/level5/March05/Graham/Graham2.html
+    """
+
+    amplitude = Parameter(default=1)
+    r_eff = Parameter(default=1)
+    n = Parameter(default=4)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    ellip = Parameter(default=0)
+    theta = Parameter(default=0)
+    _gammaincinv = None
+
+    def __init__(self, amplitude=amplitude.default, r_eff=r_eff.default,
+                 n=n.default, x_0=x_0.default, y_0=y_0.default, ellip=ellip.default,
+                 theta=theta.default, **kwargs):
+        try:
+            from scipy.special import gammaincinv
+            self.__class__._gammaincinv = gammaincinv
+        except ValueError:
+            raise ImportError("Sersic2D model requires scipy > 0.11.")
+
+        super(Sersic2D, self).__init__(
+            amplitude=amplitude, r_eff=r_eff, n=n, x_0=x_0, y_0=y_0,
+            ellip=ellip, theta=theta, **kwargs)
+
+    @classmethod
+    def evaluate(cls, x, y, amplitude, r_eff, n, x_0, y_0, ellip, theta):
+        """Two dimensional Sersic profile function."""
+
+        bn = cls._gammaincinv(2. * n, 0.5)
+        a, b = r_eff, (1 - ellip) * r_eff
+        cos_theta, sin_theta = np.cos(theta), np.sin(theta)
+        x_maj = (x - x_0) * cos_theta + (y - y_0) * sin_theta
+        x_min = -(x - x_0) * sin_theta + (y - y_0) * cos_theta
+        z = np.sqrt((x_maj / a) ** 2 + (x_min / b) ** 2)
+
+        return amplitude * np.exp(-bn * (z ** (1 / n) - 1))
+
+
 @deprecated('1.0', alternative='astropy.modeling.models.custom_model',
             pending=True)
 def custom_model_1d(func, func_fit_deriv=None):
-    argspec = inspect.getargspec(func)
-    param_values = argspec.defaults
-    nparams = len(param_values)
+    inputs, params = get_inputs_and_params(func)
 
-    if len(argspec.args) == nparams + 1:
-        param_names = argspec.args[-nparams:]
-    else:
+    if len(inputs) != 1:
         raise ModelDefinitionError(
             "All parameters must be keyword arguments")
 
