@@ -10,16 +10,17 @@ define their own models.
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
-import inspect
 import functools
 import numbers
+import types
 
 import numpy as np
 
-from ..utils import isiterable
-from ..utils.compat import ignored
+from ..utils import isiterable, OrderedDescriptor
 from ..extern import six
 
+from .utils import get_inputs_and_params
+
 __all__ = ['Parameter', 'InputParameterError']
 
 
@@ -54,7 +55,7 @@ def _tofloat(value):
     return value
 
 
-class Parameter(object):
+class Parameter(OrderedDescriptor):
     """
     Wraps individual parameters.
 
@@ -64,22 +65,31 @@ class Parameter(object):
     "unbound parameter"), or it can act as a proxy for the parameter values on
     an individual model instance (called a "bound parameter").
 
-    Parameter instances never store the actual value of the parameter
-    directly.  Rather, each instance of a model stores its own parameters
-    as either hidden attributes or (in the case of
-    `~astropy.modeling.FittableModel`) in an array.  A *bound*
-    Parameter simply wraps the value in a Parameter proxy which provides some
-    additional information about the parameter such as its constraints.
+    Parameter instances never store the actual value of the parameter directly.
+    Rather, each instance of a model stores its own parameters parameter values
+    in an array.  A *bound* Parameter simply wraps the value in a Parameter
+    proxy which provides some additional information about the parameter such
+    as its constraints.  In other words, this is a high-level interface to a
+    model's adjustable parameter values.
 
     *Unbound* Parameters are not associated with any specific model instance,
     and are merely used by model classes to determine the names of their
     parameters and other information about each parameter such as their default
     values and default constraints.
 
+    See :ref:`modeling-parameters` for more details.
+
     Parameters
     ----------
     name : str
         parameter name
+
+        .. warning::
+
+            The fact that `Parameter` accepts ``name`` as an argument is an
+            implementation detail, and should not be used directly.  When
+            defining a new `Model` class, parameter names are always
+            automatically defined by the class attribute they're assigned to.
     description : str
         parameter description
     default : float or array
@@ -104,9 +114,11 @@ class Parameter(object):
     bounds : tuple
         specify min and max as a single tuple--bounds may not be specified
         simultaneously with min or max
-    model : object
-        an instance of a Model class; this should only be used internally for
-        creating bound Parameters
+    model : `Model` instance
+        binds the the `Parameter` instance to a specific model upon
+        instantiation; this should only be used internally for creating bound
+        Parameters, and should not be used for `Parameter` descriptors defined
+        as class attributes
     """
 
     constraints = ('fixed', 'tied', 'bounds')
@@ -116,17 +128,15 @@ class Parameter(object):
     constraint (which is represented as a 2-tuple).
     """
 
-    # See the _nextid classmethod
-    _nextid = 1
+    # Settings for OrderedDescriptor
+    _class_attribute_ = '_parameters_'
+    _name_attribute_ = '_name'
 
     def __init__(self, name='', description='', default=None, getter=None,
                  setter=None, fixed=False, tied=False, min=None, max=None,
                  bounds=None, model=None):
         super(Parameter, self).__init__()
 
-        if model is not None and not name:
-            raise TypeError('Bound parameters must have a name specified.')
-
         self._name = name
         self.__doc__ = self._description = description.strip()
         self._default = default
@@ -137,7 +147,7 @@ class Parameter(object):
         if bounds is not None:
             if min is not None or max is not None:
                 raise ValueError(
-                    'bounds may not be specified simulatenously with min or '
+                    'bounds may not be specified simultaneously with min or '
                     'or max when instantiating Parameter {0}'.format(name))
         else:
             bounds = (min, max)
@@ -147,44 +157,41 @@ class Parameter(object):
         self._bounds = bounds
 
         self._order = None
-        self._shape = None
-        self._model = model
+        self._model = None
 
         # The getter/setter functions take one or two arguments: The first
         # argument is always the value itself (either the value returned or the
         # value being set).  The second argument is optional, but if present
         # will contain a reference to the model object tied to a parameter (if
         # it exists)
-        if getter is not None:
-            self._getter = self._create_value_wrapper(getter, model)
-        else:
-            self._getter = None
-        if setter is not None:
-            self._setter = self._create_value_wrapper(setter, model)
-        else:
-            self._setter = None
+        self._getter = self._create_value_wrapper(getter, None)
+        self._setter = self._create_value_wrapper(setter, None)
 
+        self._validator = None
+
+        # Only Parameters declared as class-level descriptors require
+        # and ordering ID
         if model is not None:
-            with ignored(AttributeError):
-                # This can only work if the parameter's value has been set by
-                # the model
-                _, self._shape = self._validate_value(model, self.value)
-        else:
-            # Only Parameters declared as class-level descriptors require
-            # and ordering ID
-            self._order = self._get_nextid()
+            self._bind(model)
 
     def __get__(self, obj, objtype):
         if obj is None:
             return self
 
-        return self.__class__(self._name, default=self._default,
-                              getter=self._getter, setter=self._setter,
-                              fixed=self._fixed, tied=self._tied,
-                              bounds=self._bounds, model=obj)
+        # All of the Parameter.__init__ work should already have been done for
+        # the class-level descriptor; we can skip that stuff and just copy the
+        # existing __dict__ and then bind to the model instance
+        parameter = self.__class__.__new__(self.__class__)
+        parameter.__dict__.update(self.__dict__)
+        parameter._bind(obj)
+        return parameter
 
     def __set__(self, obj, value):
-        value, shape = self._validate_value(obj, value)
+        value = _tofloat(value)
+
+        # Call the validator before the setter
+        if self._validator is not None:
+            self._validator(obj, value)
 
         if self._setter is not None:
             setter = self._create_value_wrapper(self._setter, obj)
@@ -311,12 +318,31 @@ class Parameter(object):
     def shape(self):
         """The shape of this parameter's value array."""
 
-        return self._shape
+        if self._model is None:
+            raise AttributeError('Parameter definition does not have a '
+                                 'shape.')
+
+        shape = self._model._param_metrics[self._name]['shape']
+
+        if len(self._model) > 1:
+            # If we are dealing with a model *set* the shape is the shape of
+            # the parameter within a single model in the set
+            model_axis = self._model._model_set_axis
+
+            if model_axis < 0:
+                model_axis = len(shape) + model_axis
+
+            shape = shape[:model_axis] + shape[model_axis + 1:]
+
+        return shape
 
     @property
     def size(self):
         """The size of this parameter's value array."""
 
+        # TODO: Rather than using self.value this could be determined from the
+        # size of the parameter in _param_metrics
+
         return np.size(self.value)
 
     @property
@@ -432,6 +458,83 @@ class Parameter(object):
             raise AttributeError("can't set attribute 'max' on Parameter "
                                  "definition")
 
+    @property
+    def validator(self):
+        """
+        Used as a decorator to set the validator method for a `Parameter`.
+        The validator method validates any value set for that parameter.
+        It takes two arguments--``self``, which refers to the `Model`
+        instance (remember, this is a method defined on a `Model`), and
+        the value being set for this parameter.  The validator method's
+        return value is ignored, but it may raise an exception if the value
+        set on the parameter is invalid (typically an `InputParameterError`
+        should be raised, though this is not currently a requirement).
+
+        The decorator *returns* the `Parameter` instance that the validator
+        is set on, so the underlying validator method should have the same
+        name as the `Parameter` itself (think of this as analogous to
+        ``property.setter``).  For example::
+
+            >>> from astropy.modeling import Fittable1DModel
+            >>> class TestModel(Fittable1DModel):
+            ...     a = Parameter()
+            ...     b = Parameter()
+            ...
+            ...     @a.validator
+            ...     def a(self, value):
+            ...         # Remember, the value can be an array
+            ...         if np.any(value < self.b):
+            ...             raise InputParameterError(
+            ...                 "parameter 'a' must be greater than or equal "
+            ...                 "to parameter 'b'")
+            ...
+            ...     @staticmethod
+            ...     def evaluate(x, a, b):
+            ...         return a * x + b
+            ...
+            >>> m = TestModel(a=1, b=2)  # doctest: +IGNORE_EXCEPTION_DETAIL
+            Traceback (most recent call last):
+            ...
+            InputParameterError: parameter 'a' must be greater than or equal
+            to parameter 'b'
+            >>> m = TestModel(a=2, b=2)
+            >>> m.a = 0  # doctest: +IGNORE_EXCEPTION_DETAIL
+            Traceback (most recent call last):
+            ...
+            InputParameterError: parameter 'a' must be greater than or equal
+            to parameter 'b'
+
+        On bound parameters this property returns the validator method itself,
+        as a bound method on the `Parameter`.  This is not often as useful, but
+        it allows validating a parameter value without setting that parameter::
+
+            >>> m.a.validator(42)  # Passes
+            >>> m.a.validator(-42)  # doctest: +IGNORE_EXCEPTION_DETAIL
+            Traceback (most recent call last):
+            ...
+            InputParameterError: parameter 'a' must be greater than or equal
+            to parameter 'b'
+        """
+
+        if self._model is None:
+            # For unbound parameters return the validator setter
+            def validator(func, self=self):
+                self._validator = func
+                return self
+
+            return validator
+        else:
+            # Return the validator method, bound to the Parameter instance with
+            # the name "validator"
+            def validator(self, value):
+                if self._validator is not None:
+                    return self._validator(self._model, value)
+
+            if six.PY2:
+                return types.MethodType(validator, self, type(self))
+            else:
+                return types.MethodType(validator, self)
+
     def copy(self, name=None, description=None, default=None, getter=None,
              setter=None, fixed=False, tied=False, min=None, max=None,
              bounds=None):
@@ -482,18 +585,16 @@ class Parameter(object):
 
         return self._get_model_value(self._model)
 
-    @classmethod
-    def _get_nextid(cls):
-        """Returns a monotonically increasing ID used to order Parameter
-        descriptors declared at the class-level of Model subclasses.
-
-        This allows the desired parameter order to be determined without
-        having to list it manually in the param_names class attribute.
+    def _bind(self, model):
+        """
+        Bind the `Parameter` to a specific `Model` instance; don't use this
+        directly on *unbound* parameters, i.e. `Parameter` descriptors that
+        are defined in class bodies.
         """
 
-        nextid = cls._nextid
-        cls._nextid += 1
-        return nextid
+        self._model = model
+        self._getter = self._create_value_wrapper(self._getter, model)
+        self._setter = self._create_value_wrapper(self._setter, model)
 
     def _get_model_value(self, model):
         """
@@ -543,26 +644,9 @@ class Parameter(object):
 
         model._parameters[param_slice] = np.array(value).ravel()
 
-    def _validate_value(self, model, value):
-        if model is None:
-            return
-
-        n_models = len(model)
-        value = _tofloat(value)
-        if n_models == 1:
-            # Just validate the value with _tofloat
-            return value, np.shape(value)
-        else:
-            shape = np.shape(value)
-            model_axis = model._model_set_axis
-            if model_axis < 0:
-                model_axis = len(shape) + model_axis
-            shape = shape[:model_axis] + shape[model_axis + 1:]
-
-            return value, shape
-
-    def _create_value_wrapper(self, wrapper, model):
-        """Wrappers a getter/setter function to support optionally passing in
+    @staticmethod
+    def _create_value_wrapper(wrapper, model):
+        """Wraps a getter/setter function to support optionally passing in
         a reference to the model object as the second argument.
 
         If a model is tied to this parameter and its getter/setter supports
@@ -575,9 +659,12 @@ class Parameter(object):
                 raise TypeError("A numpy.ufunc used for Parameter "
                                 "getter/setter may only take one input "
                                 "argument")
+        elif wrapper is None:
+            # Just allow non-wrappers to fall through silently, for convenience
+            return None
         else:
-            wrapper_args = inspect.getargspec(wrapper)
-            nargs = len(wrapper_args.args)
+            inputs, params = get_inputs_and_params(wrapper)
+            nargs = len(inputs)
 
             if nargs == 1:
                 pass
@@ -585,7 +672,7 @@ class Parameter(object):
                 if model is not None:
                     # Don't make a partial function unless we're tied to a
                     # specific model instance
-                    model_arg = wrapper_args.args[1]
+                    model_arg = inputs.args[1].name
                     wrapper = functools.partial(wrapper, **{model_arg: model})
             else:
                 raise TypeError("Parameter getter/setter must be a function "
@@ -595,10 +682,7 @@ class Parameter(object):
 
     def __array__(self, dtype=None):
         # Make np.asarray(self) work a little more straightforwardly
-        if self._model is None:
-            return np.array([], dtype=np.float)
-        else:
-            return np.asarray(self.value, dtype=dtype)
+        return np.asarray(self.value, dtype=dtype)
 
     def __nonzero__(self):
         if self._model is None:
@@ -609,61 +693,113 @@ class Parameter(object):
     __bool__ = __nonzero__
 
     def __add__(self, val):
+        if self._model is None:
+            # If we don't do this, __add__ will raise an AttributeError instead
+            # (from self.value) which is strange and unexpected
+            return NotImplemented
         return self.value + val
 
     def __radd__(self, val):
+        if self._model is None:
+            return NotImplemented
         return val + self.value
 
     def __sub__(self, val):
+        if self._model is None:
+            return NotImplemented
         return self.value - val
 
     def __rsub__(self, val):
+        if self._model is None:
+            return NotImplemented
         return val - self.value
 
     def __mul__(self, val):
+        if self._model is None:
+            return NotImplemented
         return self.value * val
 
     def __rmul__(self, val):
+        if self._model is None:
+            return NotImplemented
         return val * self.value
 
     def __pow__(self, val):
+        if self._model is None:
+            return NotImplemented
         return self.value ** val
 
     def __rpow__(self, val):
+        if self._model is None:
+            return NotImplemented
         return val ** self.value
 
     def __div__(self, val):
+        if self._model is None:
+            return NotImplemented
         return self.value / val
 
     def __rdiv__(self, val):
+        if self._model is None:
+            return NotImplemented
         return val / self.value
 
     def __truediv__(self, val):
+        if self._model is None:
+            return NotImplemented
         return self.value / val
 
     def __rtruediv__(self, val):
+        if self._model is None:
+            return NotImplemented
         return val / self.value
 
     def __eq__(self, val):
-        return (np.asarray(self) == np.asarray(val)).all()
+        if self._model is None:
+            return NotImplemented
+
+        return self.__array__() == val
 
     def __ne__(self, val):
-        return not (np.asarray(self) == np.asarray(val)).all()
+        if self._model is None:
+            return NotImplemented
+
+        return self.__array__() != val
 
     def __lt__(self, val):
-        return (np.asarray(self) < np.asarray(val)).all()
+        # Because OrderedDescriptor uses __lt__ to work, we need to call the
+        # super method, but only when not bound to an instance anyways
+        if self._model is None:
+            return super(Parameter, self).__lt__(val)
+
+        return self.__array__() < val
 
     def __gt__(self, val):
-        return (np.asarray(self) > np.asarray(val)).all()
+        if self._model is None:
+            return NotImplemented
+
+        return self.__array__() > val
 
     def __le__(self, val):
-        return (np.asarray(self) <= np.asarray(val)).all()
+        if self._model is None:
+            return NotImplemented
+
+        return self.__array__() <= val
 
     def __ge__(self, val):
-        return (np.asarray(self) >= np.asarray(val)).all()
+        if self._model is None:
+            return NotImplemented
+
+        return self.__array__() >= val
 
     def __neg__(self):
+        if self._model is None:
+            return NotImplemented
+
         return -self.value
 
     def __abs__(self):
+        if self._model is None:
+            return NotImplemented
+
         return np.abs(self.value)
