@@ -13,7 +13,7 @@ from .core import FittableModel, Model
 from .functional_models import Shift
 from .parameters import Parameter
 from .utils import poly_map_domain, comb, check_broadcast
-from ..utils import lazyproperty, indent
+from ..utils import indent
 
 
 __all__ = [
@@ -40,7 +40,7 @@ class PolynomialBase(FittableModel):
     linear = True
     col_fit_deriv = False
 
-    @lazyproperty
+    @property
     def param_names(self):
         """Coefficient names generated based on the model's polynomial degree
         and number of dimensions.
@@ -165,16 +165,14 @@ class OrthoPolynomialBase(PolynomialBase):
         degree in x
     y_degree : int
         degree in y
-    x_domain : list or None
+    x_domain : list or None, optional
         domain of the x independent variable
-    x_window : list or None
+    x_window : list or None, optional
         range of the x independent variable
-    y_domain : list or None
+    y_domain : list or None, optional
         domain of the y independent variable
-    y_window : list or None
+    y_window : list or None, optional
         range of the y independent variable
-    param_dim : int
-        number of parameter sets
     **params : dict
         {keyword: value} pairs, representing {parameter_name: value}
     """
@@ -326,12 +324,10 @@ class Chebyshev1D(PolynomialModel):
     ----------
     degree : int
         degree of the series
-    domain : list or None
-    window : list or None
+    domain : list or None, optional
+    window : list or None, optional
         If None, it is set to [-1,1]
         Fitters will remap the domain to this window
-    param_dim : int
-        number of parameter sets
     **params : dict
         keyword : value pairs, representing parameter_name: value
     """
@@ -416,12 +412,10 @@ class Legendre1D(PolynomialModel):
     ----------
     degree : int
         degree of the series
-    domain : list or None
-    window : list or None
+    domain : list or None, optional
+    window : list or None, optional
         If None, it is set to [-1,1]
         Fitters will remap the domain to this window
-    param_dim : int
-        number of parameter sets
     **params : dict
         keyword: value pairs, representing parameter_name: value
     """
@@ -504,12 +498,10 @@ class Polynomial1D(PolynomialModel):
     ----------
     degree : int
         degree of the series
-    domain : list or None
-    window : list or None
+    domain : list or None, optional
+    window : list or None, optional
         If None, it is set to [-1,1]
         Fitters will remap the domain to this window
-    param_dim : int
-        number of parameter sets
     **params : dict
         keyword: value pairs, representing parameter_name: value
     """
@@ -586,16 +578,14 @@ class Polynomial2D(PolynomialModel):
     degree : int
         highest power of the polynomial,
         the number of terms is degree+1
-    x_domain : list or None
+    x_domain : list or None, optional
         domain of the x independent variable
-    y_domain : list or None
+    y_domain : list or None, optional
         domain of the y independent variable
-    x_window : list or None
+    x_window : list or None, optional
         range of the x independent variable
-    y_window : list or None
+    y_window : list or None, optional
         range of the y independent variable
-    param_dim : int
-        number of parameter sets
     **params : dict
         keyword: value pairs, representing parameter_name: value
     """
@@ -736,16 +726,14 @@ class Chebyshev2D(OrthoPolynomialBase):
         degree in x
     y_degree : int
         degree in y
-    x_domain : list or None
+    x_domain : list or None, optional
         domain of the x independent variable
-    y_domain : list or None
+    y_domain : list or None, optional
         domain of the y independent variable
-    x_window : list or None
+    x_window : list or None, optional
         range of the x independent variable
-    y_window : list or None
+    y_window : list or None, optional
         range of the y independent variable
-    param_dim : int
-        number of parameter sets
     **params : dict
         keyword: value pairs, representing parameter_name: value
 
@@ -851,16 +839,14 @@ class Legendre2D(OrthoPolynomialBase):
         degree in x
     y_degree : int
         degree in y
-    x_domain : list or None
+    x_domain : list or None, optional
         domain of the x independent variable
-    y_domain : list or None
+    y_domain : list or None, optional
         domain of the y independent variable
-    x_window : list or None
+    x_window : list or None, optional
         range of the x independent variable
-    y_window : list or None
+    y_window : list or None, optional
         range of the y independent variable
-    param_dim : int
-        number of parameter sets
     **params : dict
         keyword: value pairs, representing parameter_name: value
     """
@@ -1060,8 +1046,6 @@ class SIP(Model):
         coefficients for the inverse transform
     bp_coeff : dict
         coefficients for the inverse transform
-    param_dim : int
-        number of parameter sets
 
     References
     ----------
@@ -1136,8 +1120,6 @@ class InverseSIP(Model):
         coefficients for the inverse transform
     bp_coeff : dict
         coefficients for the inverse transform
-    param_dim : int
-        number of parameter sets
 
     """
 
