@@ -42,16 +42,12 @@ class PowerLaw1D(Fittable1DModel):
 
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    alpha = Parameter()
-
-    def __init__(self, amplitude, x_0, alpha, **constraints):
-        super(PowerLaw1D, self).__init__(amplitude=amplitude, x_0=x_0,
-                                         alpha=alpha, **constraints)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=1)
+    alpha = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, x_0, alpha):
+    def evaluate(x, amplitude, x_0, alpha):
         """One dimensional power law model function"""
 
         xx = x / x_0
@@ -104,18 +100,13 @@ class BrokenPowerLaw1D(Fittable1DModel):
                    \\right.
     """
 
-    amplitude = Parameter()
-    x_break = Parameter()
-    alpha_1 = Parameter()
-    alpha_2 = Parameter()
-
-    def __init__(self, amplitude, x_break, alpha_1, alpha_2, **constraints):
-        super(BrokenPowerLaw1D, self).__init__(
-            amplitude=amplitude, x_break=x_break, alpha_1=alpha_1,
-            alpha_2=alpha_2, **constraints)
+    amplitude = Parameter(default=1)
+    x_break = Parameter(default=1)
+    alpha_1 = Parameter(default=1)
+    alpha_2 = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, x_break, alpha_1, alpha_2):
+    def evaluate(x, amplitude, x_break, alpha_1, alpha_2):
         """One dimensional broken power law model function"""
 
         alpha = np.where(x < x_break, alpha_1, alpha_2)
@@ -165,18 +156,13 @@ class ExponentialCutoffPowerLaw1D(Fittable1DModel):
 
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    alpha = Parameter()
-    x_cutoff = Parameter()
-
-    def __init__(self, amplitude, x_0, alpha, x_cutoff, **constraints):
-        super(ExponentialCutoffPowerLaw1D, self).__init__(
-            amplitude=amplitude, x_0=x_0, alpha=alpha, x_cutoff=x_cutoff,
-            **constraints)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=1)
+    alpha = Parameter(default=1)
+    x_cutoff = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, x_0, alpha, x_cutoff):
+    def evaluate(x, amplitude, x_0, alpha, x_cutoff):
         """One dimensional exponential cutoff power law model function"""
 
         xx = x / x_0
@@ -224,19 +210,14 @@ class LogParabola1D(Fittable1DModel):
 
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    alpha = Parameter()
-    beta = Parameter()
-
-    def __init__(self, amplitude, x_0, alpha, beta, **constraints):
-        super(LogParabola1D, self).__init__(
-            amplitude=amplitude, x_0=x_0, alpha=alpha, beta=beta,
-            **constraints)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=1)
+    alpha = Parameter(default=1)
+    beta = Parameter(default=0)
 
     @staticmethod
-    def eval(x, amplitude, x_0, alpha, beta):
-        """One dimenional log parabola model function"""
+    def evaluate(x, amplitude, x_0, alpha, beta):
+        """One dimensional log parabola model function"""
 
         xx = x / x_0
         exponent = -alpha - beta * np.log(xx)
@@ -244,7 +225,7 @@ class LogParabola1D(Fittable1DModel):
 
     @staticmethod
     def fit_deriv(x, amplitude, x_0, alpha, beta):
-        """One dimensional log parabola derivative with repsect to parameters"""
+        """One dimensional log parabola derivative with respect to parameters"""
 
         xx = x / x_0
         log_xx = np.log(xx)
