@@ -98,8 +98,8 @@ class TestFitting(object):
     # TODO: Most of these test cases have some pretty repetitive setup that we
     # could probably factor out
 
-    @pytest.mark.parametrize('model_class,constraints',
-                             product(linear1d.keys(), (False, True)))
+    @pytest.mark.parametrize(('model_class','constraints'),
+                             list(product(linear1d.keys(), (False, True))))
     def test_linear_fitter_1D(self, model_class, constraints):
         """Test fitting with LinearLSQFitter"""
 
@@ -128,8 +128,8 @@ class TestFitting(object):
             assert_allclose(model_lin.parameters, model.parameters,
                             atol=0.2)
 
-    @pytest.mark.parametrize('model_class,constraints',
-                             product(linear1d.keys(), (False, True)))
+    @pytest.mark.parametrize(('model_class','constraints'),
+                             list(product(linear1d.keys(), (False, True))))
     def test_non_linear_fitter_1D(self, model_class, constraints):
         """Test fitting with non-linear LevMarLSQFitter"""
 
@@ -156,8 +156,8 @@ class TestFitting(object):
             assert_allclose(model_nlin.parameters, model.parameters,
                             atol=0.2)
 
-    @pytest.mark.parametrize('model_class,constraints',
-                             product(linear2d.keys(), (False, True)))
+    @pytest.mark.parametrize(('model_class','constraints'),
+                             list(product(linear2d.keys(), (False, True))))
     def test_linear_fitter_2D(self, model_class, constraints):
         """Test fitting with LinearLSQFitter"""
 
@@ -184,8 +184,8 @@ class TestFitting(object):
             assert_allclose(model_lin.parameters, model.parameters,
                             atol=0.2)
 
-    @pytest.mark.parametrize('model_class,constraints',
-                             product(linear2d.keys(), (False, True)))
+    @pytest.mark.parametrize(('model_class','constraints'),
+                             list(product(linear2d.keys(), (False, True))))
     def test_non_linear_fitter_2D(self, model_class, constraints):
         """Test fitting with non-linear LevMarLSQFitter"""
 
