@@ -1,11 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-"""
-Test sky projections defined in WCS Paper II
-"""
+
+"""Test sky projections defined in WCS Paper II"""
 
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
-import os.path
+import os
 import numpy as np
 from numpy.testing import utils
 from .. import projections
@@ -18,55 +17,63 @@ from ...tests.helper import pytest
 
 
 def test_Projection_properties():
-    projection = projections.Sky2Pix_CAR()
+    projection = projections.Sky2Pix_PlateCarree()
     assert projection.n_inputs == 2
     assert projection.n_outputs == 2
 
+
 PIX_COORDINATES = [-10, 30]
 
-pars = [
-    ('TAN', projections.Sky2Pix_TAN, {}),
-    ('STG', projections.Sky2Pix_STG, {}),
-    ('SIN', projections.Sky2Pix_SIN, {}),
-    ('CEA', projections.Sky2Pix_CEA, {}),
-    ('CAR', projections.Sky2Pix_CAR, {}),
-    ('MER', projections.Sky2Pix_MER, {})
-]
+
+pars = [(x,) for x in projections.projcodes]
+# There is no groundtruth file for the XPH projection available here:
+#   http://www.atnf.csiro.au/people/mcalabre/WCS/example_data.html
+pars.remove(('XPH',))
 
 
-@pytest.mark.parametrize(('ID', 'model', 'args'), pars)
-def test_Sky2Pix(ID, model, args):
+@pytest.mark.parametrize(('code',), pars)
+def test_Sky2Pix(code):
     """Check astropy model eval against wcslib eval"""
-    wcs_map = os.path.join("../../wcs/tests/maps", "1904-66_{0}.hdr".format(ID))
+
+    wcs_map = os.path.join(os.pardir, os.pardir, "wcs", "tests", "maps",
+                           "1904-66_{0}.hdr".format(code))
     test_file = get_pkg_data_filename(wcs_map)
     header = fits.Header.fromfile(test_file, endcard=False, padding=False)
+
+    params = []
+    for i in range(3):
+        key = 'PV2_{0}'.format(i + 1)
+        if key in header:
+            params.append(header[key])
+
     w = wcs.WCS(header)
     w.wcs.crval = [0., 0.]
     w.wcs.crpix = [0, 0]
     w.wcs.cdelt = [1, 1]
     wcslibout = w.wcs.p2s([PIX_COORDINATES], 1)
     wcs_pix = w.wcs.s2p(wcslibout['world'], 1)['pixcrd']
-    tinv = model(**args)
+    model = getattr(projections, 'Sky2Pix_' + code)
+    tinv = model(*params)
     x, y = tinv(wcslibout['phi'], wcslibout['theta'])
     utils.assert_almost_equal(np.asarray(x), wcs_pix[:, 0])
     utils.assert_almost_equal(np.asarray(y), wcs_pix[:, 1])
 
-pars = [
-    ('TAN', projections.Pix2Sky_TAN, {}),
-    ('STG', projections.Pix2Sky_STG, {}),
-    ('SIN', projections.Pix2Sky_SIN, {}),
-    ('CEA', projections.Pix2Sky_CEA, {}),
-    ('CAR', projections.Pix2Sky_CAR, {}),
-    ('MER', projections.Pix2Sky_MER, {}),
-]
 
-
-@pytest.mark.parametrize(('ID', 'model', 'args'), pars)
-def test_Pix2Sky(ID, model, args):
+@pytest.mark.parametrize(('code',), pars)
+def test_Pix2Sky(code):
     """Check astropy model eval against wcslib eval"""
-    wcs_map = os.path.join("../../wcs/tests/maps", "1904-66_{0}.hdr".format(ID))
+
+    wcs_map = os.path.join(os.pardir, os.pardir, "wcs", "tests", "maps",
+                           "1904-66_{0}.hdr".format(code))
     test_file = get_pkg_data_filename(wcs_map)
     header = fits.Header.fromfile(test_file, endcard=False, padding=False)
+
+    params = []
+    for i in range(3):
+        key = 'PV2_{0}'.format(i + 1)
+        if key in header:
+            params.append(header[key])
+
     w = wcs.WCS(header)
     w.wcs.crval = [0., 0.]
     w.wcs.crpix = [0, 0]
@@ -74,20 +81,35 @@ def test_Pix2Sky(ID, model, args):
     wcslibout = w.wcs.p2s([PIX_COORDINATES], 1)
     wcs_phi = wcslibout['phi']
     wcs_theta = wcslibout['theta']
-    tanprj = model(**args)
+    model = getattr(projections, 'Pix2Sky_' + code)
+    tanprj = model(*params)
     phi, theta = tanprj(*PIX_COORDINATES)
     utils.assert_almost_equal(np.asarray(phi), wcs_phi)
     utils.assert_almost_equal(np.asarray(theta), wcs_theta)
 
 
-class TestAZP(object):
+@pytest.mark.parametrize(('code',), pars)
+def test_projection_default(code):
+    """Check astropy model eval with default parameters"""
+    # Just makes sure that the default parameter values are reasonable
+    # and accepted by wcslib.
+
+    model = getattr(projections, 'Sky2Pix_' + code)
+    tinv = model()
+    x, y = tinv(45, 45)
+
+    model = getattr(projections, 'Pix2Sky_' + code)
+    tinv = model()
+    x, y = tinv(0, 0)
+
+
+class TestZenithalPerspective(object):
+    """Test Zenithal Perspective projection"""
 
-    """
-    Test AZP projection
-    """
     def setup_class(self):
         ID = 'AZP'
-        wcs_map = os.path.join("../../wcs/tests/maps", "1904-66_{0}.hdr".format(ID))
+        wcs_map = os.path.join(os.pardir, os.pardir, "wcs", "tests", "maps",
+                               "1904-66_{0}.hdr".format(ID))
         test_file = get_pkg_data_filename(wcs_map)
         header = fits.Header.fromfile(test_file, endcard=False, padding=False)
         self.wazp = wcs.WCS(header)
@@ -95,9 +117,7 @@ class TestAZP(object):
         self.wazp.wcs.crval = np.array([0., 0.])
         self.wazp.wcs.cdelt = np.array([1., 1.])
         self.pv_kw = [kw[2] for kw in self.wazp.wcs.get_pv()]
-        proj = projections.__getattribute__("Pix2Sky_{0}".format(ID))
-        self.azp = proj(*self.pv_kw)
-        self.azpinv = self.azp.inverse()
+        self.azp = projections.Pix2Sky_ZenithalPerspective(*self.pv_kw)
 
     def test_AZP_p2s(self):
         wcslibout = self.wazp.wcs.p2s([[-10, 30]], 1)
@@ -110,19 +130,18 @@ class TestAZP(object):
     def test_AZP_s2p(self):
         wcslibout = self.wazp.wcs.p2s([[-10, 30]], 1)
         wcs_pix = self.wazp.wcs.s2p(wcslibout['world'], 1)['pixcrd']
-        x, y = self.azpinv(wcslibout['phi'], wcslibout['theta'])
+        x, y = self.azp.inverse(wcslibout['phi'], wcslibout['theta'])
         utils.assert_almost_equal(np.asarray(x), wcs_pix[:, 0])
         utils.assert_almost_equal(np.asarray(y), wcs_pix[:, 1])
 
 
-class TestCYP(object):
+class TestCylindricalPerspective(object):
+    """Test cylindrical perspective projection"""
 
-    """
-    Test CYP projection
-    """
     def setup_class(self):
         ID = "CYP"
-        wcs_map = os.path.join("../../wcs/tests/maps", "1904-66_{0}.hdr".format(ID))
+        wcs_map = os.path.join(os.pardir, os.pardir, "wcs", "tests", "maps",
+                               "1904-66_{0}.hdr".format(ID))
         test_file = get_pkg_data_filename(wcs_map)
         header = fits.Header.fromfile(test_file, endcard=False, padding=False)
         self.wazp = wcs.WCS(header)
@@ -130,9 +149,7 @@ class TestCYP(object):
         self.wazp.wcs.crval = np.array([0., 0.])
         self.wazp.wcs.cdelt = np.array([1., 1.])
         self.pv_kw = [kw[2] for kw in self.wazp.wcs.get_pv()]
-        proj = projections.__getattribute__("Pix2Sky_{0}".format(ID))
-        self.azp = proj(*self.pv_kw)
-        self.azpinv = self.azp.inverse()
+        self.azp = projections.Pix2Sky_CylindricalPerspective(*self.pv_kw)
 
     def test_CYP_p2s(self):
         wcslibout = self.wazp.wcs.p2s([[-10, 30]], 1)
@@ -145,7 +162,7 @@ class TestCYP(object):
     def test_CYP_s2p(self):
         wcslibout = self.wazp.wcs.p2s([[-10, 30]], 1)
         wcs_pix = self.wazp.wcs.s2p(wcslibout['world'], 1)['pixcrd']
-        x, y = self.azpinv(wcslibout['phi'], wcslibout['theta'])
+        x, y = self.azp.inverse(wcslibout['phi'], wcslibout['theta'])
         utils.assert_almost_equal(np.asarray(x), wcs_pix[:, 0])
         utils.assert_almost_equal(np.asarray(y), wcs_pix[:, 1])
 
@@ -171,17 +188,55 @@ def test_AffineTransformation2D_inverse():
         matrix=[[1, 1], [1, 1]])
 
     with pytest.raises(InputParameterError):
-        model1.inverse()
+        model1.inverse
 
     model2 = projections.AffineTransformation2D(
         matrix=[[1.2, 3.4], [5.6, 7.8]], translation=[9.1, 10.11])
-    inverse = model2.inverse()
 
     # Coordinates for vertices of a rectangle
     rect = [[0, 0], [1, 0], [0, 3], [1, 3]]
 
     x, y = zip(*rect)
 
-    x_new, y_new = inverse(*model2(x, y))
+    x_new, y_new = model2.inverse(*model2(x, y))
 
     utils.assert_allclose([x, y], [x_new, y_new], atol=1e-10)
+
+
+def test_c_projection_striding():
+    # This is just a simple test to make sure that the striding is
+    # handled correctly in the projection C extension
+    coords = np.arange(10).reshape((5, 2))
+
+    model = projections.Sky2Pix_ZenithalPerspective(2, 30)
+
+    phi, theta = model(coords[:, 0], coords[:, 1])
+
+    utils.assert_almost_equal(
+        phi,
+        [0., 2.2790416, 4.4889294, 6.6250643, 8.68301])
+
+    utils.assert_almost_equal(
+        theta,
+        [-76.4816918, -75.3594654, -74.1256332, -72.784558, -71.3406629])
+
+
+def test_c_projections_shaped():
+    nx, ny = (5, 2)
+    x = np.linspace(0, 1, nx)
+    y = np.linspace(0, 1, ny)
+    xv, yv = np.meshgrid(x, y)
+
+    model = projections.Pix2Sky_TAN()
+
+    phi, theta = model(xv, yv)
+
+    utils.assert_allclose(
+        phi,
+        [[0., 90., 90., 90., 90.,],
+         [180., 165.96375653, 153.43494882, 143.13010235, 135.]])
+
+    utils.assert_allclose(
+        theta,
+        [[90., 89.75000159, 89.50001269, 89.25004283, 89.00010152],
+         [89.00010152, 88.96933478, 88.88210788, 88.75019826,  88.58607353]])
