@@ -14,9 +14,12 @@ import numpy as np
 from ..extern import six
 from ..extern.six.moves import xrange, zip_longest
 
+from ..utils import isiterable
+from ..utils.compat.funcsigs import signature
+
 
 __all__ = ['ExpressionTree', 'AliasDict', 'check_broadcast',
-           'poly_map_domain', 'comb']
+           'poly_map_domain', 'comb', 'ellipse_extent']
 
 
 class ExpressionTree(object):
@@ -33,6 +36,15 @@ class ExpressionTree(object):
 
         self.right = right
 
+    def __getstate__(self):
+        # For some reason the default pickle protocol on Python 2 does not just
+        # do this.  On Python 3 it's not a problem.
+        return dict((slot, getattr(self, slot)) for slot in self.__slots__)
+
+    def __setstate__(self, state):
+        for slot, value in state.items():
+            setattr(self, slot, value)
+
     @property
     def isleaf(self):
         return self.left is None and self.right is None
@@ -338,6 +350,77 @@ class AliasDict(MutableMapping):
         return repr(store_copy)
 
 
+class _BoundingBox(tuple):
+    """
+    Base class for models with custom bounding box templates (methods that
+    return an actual bounding box tuple given some adjustable parameters--see
+    for example `~astropy.modeling.models.Gaussian1D.bounding_box`).
+
+    On these classes the ``bounding_box`` property still returns a `tuple`
+    giving the default bounding box for that instance of the model.  But that
+    tuple may also be a subclass of this class that is callable, and allows
+    a new tuple to be returned using a user-supplied value for any adjustable
+    parameters to the bounding box.
+    """
+
+    _model = None
+
+    def __new__(cls, input_, _model=None):
+        self = super(_BoundingBox, cls).__new__(cls, input_)
+        if _model is not None:
+            # Bind this _BoundingBox (most likely a subclass) to a Model
+            # instance so that its __call__ can access the model
+            self._model = _model
+
+        return self
+
+    def __call__(self, *args, **kwargs):
+        raise NotImplementedError(
+            "This bounding box is fixed by the model and does not have "
+            "adjustable parameters.")
+
+    @classmethod
+    def validate(cls, model, bounding_box):
+        """
+        Validate a given bounding box sequence against the given model (which
+        may be either a subclass of `~astropy.modeling.Model` or an instance
+        thereof, so long as the ``.inputs`` attribute is defined.
+
+        Currently this just checks that the bounding_box is either a 2-tuple
+        of lower and upper bounds for 1-D models, or an N-tuple of 2-tuples
+        for N-D models.
+
+        This also returns a normalized version of the bounding_box input to
+        ensure it is always an N-tuple (even for the 1-D case).
+        """
+
+        nd = model.n_inputs
+
+        if nd == 1:
+            msg = ("Bounding box for {0} model must be a sequence of length "
+                   "2 consisting of a lower and upper bound, or a 1-tuple "
+                   "containing such a sequence as its sole element.").format(
+                       model.name)
+
+            assert (isiterable(bounding_box) and
+                        np.shape(bounding_box) in ((2,), (1, 2))), msg
+
+            if len(bounding_box) == 1:
+                return cls((tuple(bounding_box[0]),))
+            else:
+                return cls((tuple(bounding_box),))
+        else:
+            msg = ("Bounding box for {0} model must be a sequence of length "
+                   "{1} (the number of model inputs) consisting of pairs of "
+                   "lower and upper bounds for those inputs on which to "
+                   "evaluate the model.").format(model.name, nd)
+
+            assert (isiterable(bounding_box) and
+                        np.shape(bounding_box) == (nd, 2)), msg
+
+            return cls(tuple(bounds) for bounds in bounding_box)
+
+
 def make_binary_operator_eval(oper, f, g):
     """
     Given a binary operator (as a callable of two arguments) ``oper`` and
@@ -484,3 +567,91 @@ def combine_labels(left, right):
 
     return left + right
 
+
+def ellipse_extent(a, b, theta):
+    """
+    Calculates the extent of a box encapsulating a rotated 2D ellipse.
+
+    Parameters
+    ----------
+    a : float
+        Major axis.
+    b : float
+        Minor axis.
+    theta : float
+        Rotation angle in radians.
+
+    Returns
+    -------
+    offsets : tuple
+        The absolute value of the offset distances from the ellipse center that
+        define its bounding box region, ``(dx, dy)``.
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+        from astropy.modeling.models import Ellipse2D
+        from astropy.modeling.utils import ellipse_extent, render_model
+
+        amplitude = 1
+        x0 = 50
+        y0 = 50
+        a = 30
+        b = 10
+        theta = np.pi/4
+
+        model = Ellipse2D(amplitude, x0, y0, a, b, theta)
+
+        dx, dy = ellipse_extent(a, b, theta)
+
+        limits = [x0 - dx, x0 + dx, y0 - dy, y0 + dy]
+
+        model.bounding_box = limits
+
+        image = render_model(model)
+
+        plt.imshow(image, cmap='binary', interpolation='nearest', alpha=.5,
+                  extent = limits)
+        plt.show()
+    """
+
+    t = np.arctan2(-b * np.tan(theta), a)
+    dx = a * np.cos(t) * np.cos(theta) - b * np.sin(t) * np.sin(theta)
+
+    t = np.arctan2(b, a * np.tan(theta))
+    dy = b * np.sin(t) * np.cos(theta) + a * np.cos(t) * np.sin(theta)
+
+    return np.abs([dx, dy])
+
+
+def get_inputs_and_params(func):
+    """
+    Given a callable, determine the input variables and the
+    parameters.
+
+    Parameters
+    ----------
+    func : callable
+
+    Returns
+    -------
+    inputs, params : tuple
+        Each entry is a list of inspect.Parameter objects
+    """
+    sig = signature(func)
+
+    inputs = []
+    params = []
+    for param in sig.parameters.values():
+        if param.kind in (param.VAR_POSITIONAL, param.VAR_KEYWORD):
+            raise ValueError("Signature must not have *args or **kwargs")
+        if param.default == param.empty:
+            inputs.append(param)
+        else:
+            params.append(param)
+
+    return inputs, params
