@@ -20,8 +20,11 @@ conf = Conf()
 
 from .column import Column, MaskedColumn
 from .groups import TableGroups, ColumnGroups
-from .table import Table, QTable, TableColumns, Row, TableFormatter
+from .table import Table, QTable, TableColumns, Row, TableFormatter, NdarrayMixin
 from .operations import join, hstack, vstack, unique, TableMergeError
+from .jsviewer import JSViewer
+from .bst import BST, FastBST, FastRBT
+from .sorted_array import SortedArray
 
 # Import routines that connect readers/writers to astropy.table
 from ..io.ascii import connect
