@@ -14,10 +14,13 @@ from ..units import Unit, Quantity
 from ..utils.compat import NUMPY_LT_1_8
 from ..utils.console import color_print
 from ..utils.metadata import MetaData
+from ..utils.data_info import BaseColumnInfo, dtype_info_name
 from . import groups
 from . import pprint
 from .np_utils import fix_column_name
 
+# These "shims" provide __getitem__ implementations for Column and MaskedColumn
+from ._column_mixins import _ColumnGetitemShim, _MaskedColumnGetitemShim
 from ..config import ConfigAlias
 
 
@@ -44,108 +47,33 @@ _comparison_functions = set(
      np.isfinite, np.isinf, np.isnan, np.sign, np.signbit])
 
 
-COLUMN_ATTRS = set(['name', 'unit', 'dtype', 'format', 'description', 'meta', 'parent_table'])
-
-def col_setattr(col, attr, value):
-    """
-    Set one of the column attributes.
-
-    Warning: this function is subject to change or removal.
-    """
-    if attr not in COLUMN_ATTRS:
-        raise AttributeError("attribute must be one of {0}".format(COLUMN_ATTRS))
-
-    # The unit and dtype attributes are considered universal and do NOT get
-    # stored in _astropy_column_attrs.  For BaseColumn instances use the usual setattr.
-    if isinstance(col, BaseColumn):
-        setattr(col, attr, value)
-    else:
-        # If no _astropy_column_attrs or it is None then convert to dict
-        if getattr(col, '_astropy_column_attrs', None) is None:
-            col._astropy_column_attrs = {}
-        if attr == 'parent_table':
-            value = None if value is None else weakref.ref(value)
-        col._astropy_column_attrs[attr] = value
-
-def col_getattr(col, attr, default=None):
-    """
-    Get one of the column attributes
-
-    Warning: this function is subject to change or removal.
-    """
-    if attr not in COLUMN_ATTRS:
-        raise AttributeError("attribute must be one of {0}".format(COLUMN_ATTRS))
-
-    # The unit and dtype attributes are considered universal and do NOT get
-    # stored in _astropy_column_attrs.  For BaseColumn instances use the usual setattr.
-    if (isinstance(col, BaseColumn) or
-            (isinstance(col, Quantity) and attr in ('dtype', 'unit'))):
-        value = getattr(col, attr, default)
-    else:
-        # If col does not have _astropy_column_attrs or it is None (meaning
-        # nothing has been set yet) then return default, otherwise look for
-        # the attribute in the astropy_column_attrs dict.
-        if getattr(col, '_astropy_column_attrs', None) is None:
-            value = default
-        else:
-            value = col._astropy_column_attrs.get(attr, default)
-
-        # Weak ref for parent table
-        if attr == 'parent_table' and callable(value):
-            value = value()
-
-        # Mixins have a default dtype of Object if nothing else was set
-        if attr == 'dtype' and value is None:
-            value = np.dtype('O')
-
-    return value
-
-def _col_update_attrs_from(newcol, col, exclude_attrs=['name', 'parent_table']):
-    """
-    Update _astropy_column_attrs from mixin `col` to `newcol`.  Does nothing
-    for BaseColumn cols
-
-    Warning: this function is subject to change or removal.
-    """
-    if isinstance(newcol, BaseColumn):
-        return
-
-    attrs = COLUMN_ATTRS - set(exclude_attrs)
-    for attr in attrs:
-        val = col_getattr(col, attr)
-        if val is not None:
-            col_setattr(newcol, attr, deepcopy(val))
-
-def col_iter_str_vals(col):
-    """
-    This is a mixin-safe version of Column.iter_str_vals.
-
-    Warning: this function is subject to change or removal.
-    """
-    parent_table = col_getattr(col, 'parent_table')
-    formatter = FORMATTER if parent_table is None else parent_table.formatter
-    _pformat_col_iter = formatter._pformat_col_iter
-    for str_val in _pformat_col_iter(col, -1, False, False, {}):
-        yield str_val
-
-def col_copy(col):
+def col_copy(col, copy_indices=True):
     """
     This is a mixin-safe version of Column.copy() (with copy_data=True).
-
-    Warning: this function is subject to change or removal.
     """
     if isinstance(col, BaseColumn):
         return col.copy()
 
-    if hasattr(col, '_astropy_column_attrs'):
-        col_setattr(col, 'parent_table', None)  # Don't copy weakref to parent table
-    newcol = col.copy() if hasattr(col, 'copy') else deepcopy(col)
+    # The new column should have None for the parent_table ref.  If the
+    # original parent_table weakref there at the point of copying then it
+    # generates an infinite recursion.  Instead temporarily remove the weakref
+    # on the original column and restore after the copy in an exception-safe
+    # manner.
 
-    # Copy old attributes.  Even deepcopy above may not get this (e.g. pandas).
-    if (not hasattr(newcol, '_astropy_column_attrs') or
-            newcol._astropy_column_attrs is None):
-        _column_attrs = deepcopy(getattr(col, '_astropy_column_attrs', {}))
-        newcol._astropy_column_attrs = _column_attrs
+    parent_table = col.info.parent_table
+    indices = col.info.indices
+    col.info.parent_table = None
+    col.info.indices = []
+
+    try:
+        newcol = col.copy() if hasattr(col, 'copy') else deepcopy(col)
+        newcol.info = col.info
+        newcol.info.indices = deepcopy(indices or []) if copy_indices else []
+        for index in newcol.info.indices:
+            index.replace_col(col, newcol)
+    finally:
+        col.info.parent_table = parent_table
+        col.info.indices = indices
 
     return newcol
 
@@ -168,14 +96,19 @@ class FalseArray(np.ndarray):
                              .format(self.__class__.__name__))
 
 
-class BaseColumn(np.ndarray):
+class ColumnInfo(BaseColumnInfo):
+    attrs_from_parent = BaseColumnInfo.attr_names
+    _supports_indexing = True
+
+
+class BaseColumn(_ColumnGetitemShim, np.ndarray):
 
     meta = MetaData()
 
     def __new__(cls, data=None, name=None,
                 dtype=None, shape=(), length=0,
-                description=None, unit=None, format=None, meta=None, copy=False):
-
+                description=None, unit=None, format=None, meta=None,
+                copy=False, copy_indices=True):
         if data is None:
             dtype = (np.dtype(dtype).str, shape)
             self_data = np.zeros(length, dtype=dtype)
@@ -202,11 +135,11 @@ class BaseColumn(np.ndarray):
             else:
                 self_data = np.array(data.to(unit), dtype=dtype, copy=copy)
             if description is None:
-                description = col_getattr(data, 'description')
+                description = data.info.description
             if format is None:
-                format = col_getattr(data, 'format')
+                format = data.info.format
             if meta is None:
-                meta = deepcopy(col_getattr(data, 'meta'))
+                meta = deepcopy(data.info.meta)
 
         else:
             self_data = np.array(data, dtype=dtype, copy=copy)
@@ -218,6 +151,10 @@ class BaseColumn(np.ndarray):
         self.description = description
         self.meta = meta
         self._parent_table = None
+        self.indices = deepcopy(getattr(data, 'indices', [])) if \
+                       copy_indices else []
+        for index in self.indices:
+            index.replace_col(data, self)
 
         return self
 
@@ -239,6 +176,7 @@ class BaseColumn(np.ndarray):
         else:
             self._parent_table = weakref.ref(table)
 
+    info = ColumnInfo()
 
     def copy(self, order='C', data=None, copy_data=True):
         """
@@ -306,6 +244,7 @@ class BaseColumn(np.ndarray):
         self.format = format
         self.description = description
         self.meta = meta
+        self._parent_table = None
 
     def __reduce__(self):
         """
@@ -323,12 +262,6 @@ class BaseColumn(np.ndarray):
 
         return reconstruct_func, reconstruct_func_args, state
 
-    def __getitem__(self, item):
-        if isinstance(item, INTEGER_TYPES):
-            return self.data[item]  # Return as plain ndarray or ma.MaskedArray
-        else:
-            return super(BaseColumn, self).__getitem__(item)
-
     # avoid == and != to be done based on type of subclass
     # (helped solve #1446; see also __array_wrap__)
     def __eq__(self, other):
@@ -349,6 +282,8 @@ class BaseColumn(np.ndarray):
         # or viewcast e.g. obj.view(Column).  In either case we want to
         # init Column attributes for self from obj if possible.
         self.parent_table = None
+        if not hasattr(self, 'indices'): # may have been copied in __new__
+            self.indices = []
         self._copy_attrs(obj)
 
     def __array_wrap__(self, out_arr, context=None):
@@ -764,22 +699,28 @@ class Column(BaseColumn):
 
     def __new__(cls, data=None, name=None,
                 dtype=None, shape=(), length=0,
-                description=None, unit=None, format=None, meta=None, copy=False):
+                description=None, unit=None, format=None, meta=None,
+                copy=False, copy_indices=True):
 
         if isinstance(data, MaskedColumn) and np.any(data.mask):
             raise TypeError("Cannot convert a MaskedColumn with masked value to a Column")
 
         self = super(Column, cls).__new__(cls, data=data, name=name, dtype=dtype,
                                           shape=shape, length=length, description=description,
-                                          unit=unit, format=format, meta=meta, copy=copy)
+                                          unit=unit, format=format, meta=meta,
+                                          copy=copy, copy_indices=copy_indices)
         return self
 
     def _base_repr_(self, html=False):
+        # If scalar then just convert to correct numpy type and use numpy repr
+        if self.ndim == 0:
+            return repr(self.item())
+
         descr_vals = [self.__class__.__name__]
         unit = None if self.unit is None else str(self.unit)
         shape = None if self.ndim <= 1 else self.shape[1:]
         for attr, val in (('name', self.name),
-                          ('dtype', self.dtype.name),
+                          ('dtype', dtype_info_name(self.dtype)),
                           ('shape', shape),
                           ('unit', unit),
                           ('format', self.format),
@@ -811,6 +752,10 @@ class Column(BaseColumn):
         return self._base_repr_(html=False)
 
     def __unicode__(self):
+        # If scalar then just convert to correct numpy type and use numpy repr
+        if self.ndim == 0:
+            return str(self.item())
+
         lines, outs = self._formatter._pformat_col(self)
         return '\n'.join(lines)
     if six.PY3:
@@ -824,11 +769,14 @@ class Column(BaseColumn):
     # Set items using a view of the underlying data, as it gives an
     # order-of-magnitude speed-up. [#2994]
     def __setitem__(self, index, value):
+        # update indices
+        self.info.adjust_indices(index, value, len(self))
         self.data[index] = value
 
     # # Set slices using a view of the underlying data, as it gives an
     # # order-of-magnitude speed-up.  Only gets called in Python 2.  [#3020]
     def __setslice__(self, start, stop, value):
+        self.info.adjust_indices(slice(start, stop), value, len(self))
         self.data.__setslice__(start, stop, value)
 
     def insert(self, obj, values):
@@ -879,7 +827,7 @@ class Column(BaseColumn):
     to = BaseColumn.to
 
 
-class MaskedColumn(Column, ma.MaskedArray):
+class MaskedColumn(Column, _MaskedColumnGetitemShim, ma.MaskedArray):
     """Define a masked data column for use in a Table object.
 
     Parameters
@@ -953,7 +901,8 @@ class MaskedColumn(Column, ma.MaskedArray):
 
     def __new__(cls, data=None, name=None, mask=None, fill_value=None,
                 dtype=None, shape=(), length=0,
-                description=None, unit=None, format=None, meta=None, copy=False):
+                description=None, unit=None, format=None, meta=None,
+                copy=False, copy_indices=True):
 
         if mask is None and hasattr(data, 'mask'):
             mask = data.mask
@@ -969,17 +918,24 @@ class MaskedColumn(Column, ma.MaskedArray):
         # First just pass through all args and kwargs to BaseColumn, then wrap that object
         # with MaskedArray.
         self_data = BaseColumn(data, dtype=dtype, shape=shape, length=length, name=name,
-                               unit=unit, format=format, description=description, meta=meta, copy=copy)
+                               unit=unit, format=format, description=description,
+                               meta=meta, copy=copy, copy_indices=copy_indices)
         self = ma.MaskedArray.__new__(cls, data=self_data, mask=mask)
 
         # Note: do not set fill_value in the MaskedArray constructor because this does not
         # go through the fill_value workarounds (see _fix_fill_value below).
-        if fill_value is None and hasattr(data, 'fill_value'):
-            fill_value = data.fill_value
+        if fill_value is None and hasattr(data, 'fill_value') and data.fill_value is not None:
+            # Coerce the fill_value to the correct type since `data` may be a
+            # different dtype than self.
+            fill_value = self.dtype.type(data.fill_value)
         self.fill_value = fill_value
 
         self.parent_table = None
 
+        # needs to be done here since self doesn't come from BaseColumn.__new__
+        for index in self.indices:
+            index.replace_col(self_data, self)
+
         return self
 
     def _fix_fill_value(self, val):
@@ -1108,28 +1064,29 @@ class MaskedColumn(Column, ma.MaskedArray):
 
         out = new_ma.view(self.__class__)
         out.parent_table = None
+        out.indices = []
         out._copy_attrs(self)
 
         return out
 
-    def __getitem__(self, item):
-        out = super(MaskedColumn, self).__getitem__(item)
-
+    def _copy_attrs_slice(self, out):
         # Fixes issue #3023: when calling getitem with a MaskedArray subclass
         # the original object attributes are not copied.
         if out.__class__ is self.__class__:
             out.parent_table = None
+            # we need this because __getitem__ does a shallow copy of indices
+            if out.indices is self.indices:
+                out.indices = []
             out._copy_attrs(self)
-
         return out
 
-    # Set items and slices using MaskedArray method, instead of falling through
-    # to the (faster) Column version which uses an ndarray view.  This doesn't
-    # copy the mask properly. See test_setting_from_masked_column test.
     def __setitem__(self, index, value):
+        # update indices
+        self.info.adjust_indices(index, value, len(self))
         ma.MaskedArray.__setitem__(self, index, value)
 
     def __setslice__(self, start, stop, value):
+        # defers to __setitem__, so we don't adjust indices here
         ma.MaskedArray.__setslice__(self, start, stop, value)
 
     # We do this to make the methods show up in the API docs
