@@ -8,6 +8,7 @@ from ..extern.six.moves import xrange
 
 import os
 import sys
+import re
 
 import numpy as np
 
@@ -15,6 +16,7 @@ from .. import log
 # Note, in numpy <= 1.6, some classes do not properly represent themselves.
 from ..utils.compat import NUMPY_LT_1_6_1
 from ..utils.console import Getch, color_print, terminal_size, conf
+from ..utils.data_info import dtype_info_name
 
 if six.PY3:
     def default_format_func(format_, val):
@@ -71,6 +73,9 @@ def _auto_format_func(format_, val):
 
     Returns the formatted value.
     """
+    if format_ in _format_funcs:
+        return _format_funcs[format_](format_, val)
+
     if six.callable(format_):
         format_func = lambda format_, val: format_(val)
         if NUMPY_LT_1_6_1:
@@ -183,9 +188,8 @@ class TableFormatter(object):
 
         return max_lines, max_width
 
-
     def _pformat_col(self, col, max_lines=None, show_name=True, show_unit=None,
-                     show_dtype=False, show_length=None, html=False):
+                     show_dtype=False, show_length=None, html=False, align=None):
         """Return a list of formatted string representation of column values.
 
         Parameters
@@ -211,23 +215,33 @@ class TableFormatter(object):
         html : bool
             Output column as HTML
 
+        align : str
+            Left/right alignment of columns. Default is '>' (right) for all
+            columns. Other allowed values are '<', '^', and '0=' for left,
+            centered, and 0-padded, respectively.
+
         Returns
         -------
         lines : list
             List of lines with formatted column values
 
-        n_header : int
-            Number of lines in the header
+        outs : dict
+            Dict which is used to pass back additional values
+            defined within the iterator.
 
         """
         if show_unit is None:
-            show_unit = getattr(col, 'unit', None) is not None
+            show_unit = col.info.unit is not None
 
         outs = {}  # Some values from _pformat_col_iter iterator that are needed here
-        col_strs_iter = self._pformat_col_iter(col, max_lines, show_name=show_name, show_unit=show_unit,
-                                               show_dtype=show_dtype, show_length=show_length,
+        col_strs_iter = self._pformat_col_iter(col, max_lines, show_name=show_name,
+                                               show_unit=show_unit,
+                                               show_dtype=show_dtype,
+                                               show_length=show_length,
                                                outs=outs)
         col_strs = list(col_strs_iter)
+        if len(col_strs) > 0:
+            col_width = max(len(x) for x in col_strs)
 
         if html:
             from ..utils.xml.writer import xml_escape
@@ -249,25 +263,53 @@ class TableFormatter(object):
             col_strs.insert(0, '<table>')
             col_strs.append('</table>')
 
+        # Now bring all the column string values to the same fixed width
         else:
             col_width = max(len(x) for x in col_strs) if col_strs else 1
 
-            # Center line content and generate dashed headerline
+            # Center line header content and generate dashed headerline
             for i in outs['i_centers']:
                 col_strs[i] = col_strs[i].center(col_width)
             if outs['i_dashes'] is not None:
                 col_strs[outs['i_dashes']] = '-' * col_width
 
-            # Now bring all the column string values to the same fixed width
+            # Format columns according to alignment.  `align` arg has precedent, otherwise
+            # use `col.format` if it starts as a legal alignment string.  If neither applies
+            # then right justify.
+            re_fill_align = re.compile(r'(?P<fill>.?)(?P<align>[<^>=])')
+            match = None
+            if align:
+                # If there is an align specified then it must match
+                match = re_fill_align.match(align)
+                if not match:
+                    raise ValueError("column align must be one of '<', '^', '>', or '='")
+            elif isinstance(col.info.format, six.string_types):
+                # col.info.format need not match, in which case rjust gets used
+                match = re_fill_align.match(col.info.format)
+
+            if match:
+                fill_char = match.group('fill')
+                align_char = match.group('align')
+                if align_char == '=':
+                    if fill_char != '0':
+                        raise ValueError("fill character must be '0' for '=' align")
+                    fill_char = ''  # str.zfill gets used which does not take fill char arg
+            else:
+                fill_char = ''
+                align_char = '>'
+
+            justify_methods = {'<': 'ljust', '^': 'center', '>': 'rjust', '=': 'zfill'}
+            justify_method = justify_methods[align_char]
+            justify_args = (col_width, fill_char) if fill_char else (col_width,)
+
             for i, col_str in enumerate(col_strs):
-                col_strs[i] = col_str.rjust(col_width)
+                col_strs[i] = getattr(col_str, justify_method)(*justify_args)
 
         if outs['show_length']:
             col_strs.append('Length = {0} rows'.format(len(col)))
 
         return col_strs, outs
 
-
     def _pformat_col_iter(self, col, max_lines, show_name, show_unit, outs,
                           show_dtype=False, show_length=None):
         """Iterator which yields formatted string representation of column values.
@@ -285,18 +327,17 @@ class TableFormatter(object):
             for units only if one or more columns has a defined value
             for the unit.
 
+        outs : dict
+            Must be a dict which is used to pass back additional values
+            defined within the iterator.
+
         show_dtype : bool
             Include column dtype (default=False)
 
         show_length : bool
             Include column length at end.  Default is to show this only
             if the column is not shown completely.
-
-        out : dict
-            Must be a dict which is used to pass back additional values
-            defined within the iterator.
         """
-        from .column import col_getattr
         max_lines, _ = self._get_pprint_size(max_lines, -1)
 
         multidims = getattr(col, 'shape', [0])[1:]
@@ -311,7 +352,7 @@ class TableFormatter(object):
         if show_name:
             i_centers.append(n_header)
             # Get column name (or 'None' if not set)
-            col_name = six.text_type(col_getattr(col, 'name'))
+            col_name = six.text_type(col.info.name)
             if multidims:
                 col_name += ' [{0}]'.format(
                     ','.join(six.text_type(n) for n in multidims))
@@ -320,12 +361,12 @@ class TableFormatter(object):
         if show_unit:
             i_centers.append(n_header)
             n_header += 1
-            yield six.text_type(getattr(col, 'unit', None) or '')
+            yield six.text_type(col.info.unit or '')
         if show_dtype:
             i_centers.append(n_header)
             n_header += 1
             try:
-                dtype = col.dtype.name
+                dtype = dtype_info_name(col.dtype)
             except AttributeError:
                 dtype = 'object'
             yield six.text_type(dtype)
@@ -338,7 +379,7 @@ class TableFormatter(object):
         n_print2 = max_lines // 2
         n_rows = len(col)
 
-        col_format = col_getattr(col, 'format')
+        col_format = col.info.format or getattr(col.info, 'default_format', None)
         format_func = _format_funcs.get(col_format, _auto_format_func)
         if len(col) > max_lines:
             if show_length is None:
@@ -374,10 +415,9 @@ class TableFormatter(object):
         outs['i_centers'] = i_centers
         outs['i_dashes'] = i_dashes
 
-
-    def _pformat_table(self, table, max_lines=None, max_width=None, show_name=True,
-                       show_unit=None, show_dtype=False,
-                       html=False, tableid=None):
+    def _pformat_table(self, table, max_lines=None, max_width=None,
+                       show_name=True, show_unit=None, show_dtype=False,
+                       html=False, tableid=None, tableclass=None, align=None):
         """Return a list of lines for the formatted string representation of
         the table.
 
@@ -408,13 +448,25 @@ class TableFormatter(object):
             "table{id}", where id is the unique integer id of the table object,
             id(table)
 
+        tableclass : str or list of str or `None`
+            CSS classes for the table; only used if html is set.  Default is
+            none
+
+        align : str or list or tuple
+            Left/right alignment of columns. Default is '>' (right) for all
+            columns. Other allowed values are '<', '^', and '0=' for left,
+            centered, and 0-padded, respectively. A list of strings can be
+            provided for alignment of tables with multiple columns.
+
         Returns
         -------
-        out : str
-            Formatted table as a single string
+        rows : list
+            Formatted table as a list of strings
+
+        outs : dict
+            Dict which is used to pass back additional values
+            defined within the iterator.
 
-        n_header : int
-            Number of lines in the header
         """
         # "Print" all the values into temporary lists by column for subsequent
         # use and to determine the width
@@ -422,11 +474,26 @@ class TableFormatter(object):
         cols = []
 
         if show_unit is None:
-            show_unit = any([getattr(col, 'unit', None) for col in six.itervalues(table.columns)])
+            show_unit = any([col.info.unit for col in six.itervalues(table.columns)])
 
-        for col in six.itervalues(table.columns):
+        # Coerce align into a correctly-sized list of alignments (if possible)
+        n_cols = len(table.columns)
+        if align is None or isinstance(align, six.string_types):
+            align = [align] * n_cols
+
+        elif isinstance(align, (list, tuple)):
+            if len(align) != n_cols:
+                raise ValueError('got {0} alignment values instead of '
+                                 'the number of columns ({1})'
+                                 .format(len(align), n_cols))
+        else:
+            raise TypeError('align keyword must be str or list or tuple (got {0})'
+                            .format(type(align)))
+
+        for align_, col in izip(align, table.columns.values()):
             lines, outs = self._pformat_col(col, max_lines, show_name=show_name,
-                                            show_unit=show_unit, show_dtype=show_dtype)
+                                            show_unit=show_unit, show_dtype=show_dtype,
+                                            align=align_)
             if outs['show_length']:
                 lines = lines[:-1]
             cols.append(lines)
@@ -460,7 +527,15 @@ class TableFormatter(object):
 
             if tableid is None:
                 tableid = 'table{id}'.format(id=id(table))
-            rows.append('<table id="{tid}">'.format(tid=tableid))
+
+            if tableclass is not None:
+                if isinstance(tableclass, list):
+                    tableclass = ' '.join(tableclass)
+                rows.append('<table id="{tid}" class="{tcls}">'.format(
+                    tid=tableid, tcls=tableclass))
+            else:
+                rows.append('<table id="{tid}">'.format(tid=tableid))
+
             for i in range(n_rows):
                 # _pformat_col output has a header line '----' which is not needed here
                 if i == n_header - 1:
@@ -480,9 +555,8 @@ class TableFormatter(object):
 
         return rows, outs
 
-
-    def _more_tabcol(self, tabcol, max_lines=None, max_width=None, show_name=True,
-                     show_unit=None, show_dtype=False):
+    def _more_tabcol(self, tabcol, max_lines=None, max_width=None,
+                     show_name=True, show_unit=None, show_dtype=False):
         """Interactive "more" of a table or column.
 
         Parameters
