@@ -1,4 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
+# TEST_UNICODE_LITERALS
+
 """ Verify item access API in:
 https://github.com/astropy/astropy/wiki/Table-item-access-definition
 """
@@ -7,31 +10,7 @@ import numpy as np
 
 from ...tests.helper import pytest
 from ... import table
-
-
-class MaskedTable(table.Table):
-    def __init__(self, *args, **kwargs):
-        kwargs['masked'] = True
-        table.Table.__init__(self, *args, **kwargs)
-
-
-# Fixture to run all the Column tests for both an unmasked (ndarray)
-# and masked (MaskedArray) column.
-@pytest.fixture(params=[False, True])
-def table_data(request):
-    class TableData:
-        def __init__(self, request):
-            self.Table = MaskedTable if request.param else table.Table
-            self.Column = table.MaskedColumn if request.param else table.Column
-            self.COLS = [
-                self.Column(name='a', data=[1, 2, 3], description='da',
-                            format='fa', meta={'ma': 1}, unit='ua'),
-                self.Column(name='b', data=[4, 5, 6], description='db',
-                            format='fb', meta={'mb': 1}, unit='ub'),
-                self.Column(name='c', data=[7, 8, 9], description='dc',
-                            format='fc', meta={'mc': 1}, unit='ub')]
-            self.DATA = self.Table(self.COLS)
-    return TableData(request)
+from .conftest import MaskedTable
 
 
 @pytest.mark.usefixtures('table_data')
@@ -137,11 +116,34 @@ class TestTableItems(BaseTestItems):
         assert row.columns['a'].attrs_equal(table_data.COLS[0])
         assert row.columns['b'].attrs_equal(table_data.COLS[1])
         assert row.columns['c'].attrs_equal(table_data.COLS[2])
+
+        # Check that setting by col index sets the table and row value
         row[1] = 0
         assert row[1] == 0
-        if table_data.Table is not MaskedTable:
-            # numpy.core.ma.mvoid makes a copy so this test is skipped for masked table
-            assert self.t['b'][1] == 0
+        assert row['b'] == 0
+        assert self.t['b'][1] == 0
+        assert self.t[1]['b'] == 0
+
+        # Check that setting by col name sets the table and row value
+        row['a'] = 0
+        assert row[0] == 0
+        assert row['a'] == 0
+        assert self.t['a'][1] == 0
+        assert self.t[1]['a'] == 0
+
+    def test_empty_iterable_item(self, table_data):
+        """
+        Table item access with [], (), or np.array([]) returns the same table
+        with no rows.
+        """
+        self.t = table_data.Table(table_data.COLS)
+        for item in [], (), np.array([]):
+            t2 = self.t[item]
+            assert not t2
+            assert len(t2) == 0
+            assert t2['a'].attrs_equal(table_data.COLS[0])
+            assert t2['b'].attrs_equal(table_data.COLS[1])
+            assert t2['c'].attrs_equal(table_data.COLS[2])
 
     def test_table_slice(self, table_data):
         """Table slice returns REFERENCE to data"""
@@ -172,7 +174,7 @@ class TestTableItems(BaseTestItems):
         assert t2['c'].attrs_equal(table_data.COLS[2])
         t2['a'][0] = 0
 
-        assert np.all(self.t._data == table_data.DATA)
+        assert np.all(self.t.as_array() == table_data.DATA)
         assert np.any(t2['a'] != table_data.DATA['a'][slice])
         assert t2.masked == self.t.masked
         assert t2._column_class == self.t._column_class
@@ -191,7 +193,7 @@ class TestTableItems(BaseTestItems):
         assert t2['c'].attrs_equal(table_data.COLS[2])
         t2['a'][0] = 0
 
-        assert np.all(self.t._data == table_data.DATA)
+        assert np.all(self.t.as_array() == table_data.DATA)
         assert np.any(t2['a'] != table_data.DATA['a'][slice])
         assert t2.masked == self.t.masked
         assert t2._column_class == self.t._column_class
@@ -211,11 +213,23 @@ class TestTableItems(BaseTestItems):
             assert t2['a'].attrs_equal(table_data.COLS[0])
             assert t2['c'].attrs_equal(table_data.COLS[2])
             t2['a'][0] = 0
-            assert np.all(self.t._data == table_data.DATA)
+            assert np.all(self.t.as_array() == table_data.DATA)
             assert np.any(t2['a'] != table_data.DATA['a'])
             assert t2.masked == self.t.masked
             assert t2._column_class == self.t._column_class
 
+    def test_select_columns_fail(self, table_data):
+        """Selecting a column that doesn't exist fails"""
+        self.t = table_data.Table(table_data.COLS)
+
+        with pytest.raises(ValueError) as err:
+            self.t[['xxxx']]
+        assert 'Slice name(s) xxxx not valid column name(s)' in str(err)
+
+        with pytest.raises(ValueError) as err:
+            self.t[['xxxx', 'yyyy']]
+        assert 'Slice name(s) xxxx, yyyy not valid column name(s)' in str(err)
+
     def test_np_where(self, table_data):
         """Select rows using output of np.where"""
         t = table_data.Table(table_data.COLS)
@@ -232,6 +246,15 @@ class TestTableItems(BaseTestItems):
         assert len(t2) == 0
         assert isinstance(t2, table_data.Table)
 
+    def test_np_integers(self, table_data):
+        """
+        Select rows using numpy integers.  This is a regression test for a
+        py 3.3 failure mode
+        """
+        t = table_data.Table(table_data.COLS)
+        idxs = np.random.randint(len(t), size=2)
+        item = t[idxs[1]]
+
     def test_select_bad_column(self, table_data):
         """Select column name that does not exist"""
         self.t = table_data.Table(table_data.COLS)
