@@ -7,11 +7,12 @@ import numpy as np
 from ...tests.helper import pytest
 from ... import table
 from ...table import Table
+from ...table.table_helpers import simple_table
 from ...extern.six import PY3
 from ...utils import console
 
-BIG_WIDE_ARR = np.arange(2000, dtype=np.float).reshape(100, 20)
-SMALL_ARR = np.arange(12, dtype=np.int).reshape(4, 3)
+BIG_WIDE_ARR = np.arange(2000, dtype=np.float64).reshape(100, 20)
+SMALL_ARR = np.arange(18, dtype=np.int64).reshape(6, 3)
 
 
 @pytest.mark.usefixtures('table_type')
@@ -20,14 +21,13 @@ class TestMultiD():
     def test_multidim(self, table_type):
         """Test printing with multidimensional column"""
         arr = [np.array([[1, 2],
-                         [10, 20]]),
+                         [10, 20]], dtype=np.int64),
                np.array([[3, 4],
-                         [30, 40]]),
+                         [30, 40]], dtype=np.int64),
                np.array([[5, 6],
-                         [50, 60]])]
+                         [50, 60]], dtype=np.int64)]
         t = table_type(arr)
         lines = t.pformat()
-        print(lines)
         assert lines == ['col0 [2] col1 [2] col2 [2]',
                          '-------- -------- --------',
                          '  1 .. 2   3 .. 4   5 .. 6',
@@ -39,14 +39,17 @@ class TestMultiD():
                          '<tr><td>1 .. 2</td><td>3 .. 4</td><td>5 .. 6</td></tr>',
                          '<tr><td>10 .. 20</td><td>30 .. 40</td><td>50 .. 60</td></tr>',
                          '</table>']
-        assert t._repr_html_() == ('<table id="table{tid}"><thead><tr><th>col0 [2]</th><th>col1 [2]</th><th>col2 [2]'.format(tid=id(t)) +
-                                   '</th></tr></thead><tr><td>1 .. 2</td><td>3 .. 4</td><td>5 .. 6</td>'
-                                   '</tr><tr><td>10 .. 20</td><td>30 .. 40</td><td>50 .. 60</td>'
-                                   '</tr></table>')
+        assert t._repr_html_().splitlines() == [
+            '&lt;{0} masked={1} length=2&gt;'.format(table_type.__name__, t.masked),
+            '<table id="table{tid}">'.format(tid=id(t)),
+            '<thead><tr><th>col0 [2]</th><th>col1 [2]</th><th>col2 [2]</th></tr></thead>',
+            '<thead><tr><th>int64</th><th>int64</th><th>int64</th></tr></thead>',
+            '<tr><td>1 .. 2</td><td>3 .. 4</td><td>5 .. 6</td></tr>',
+            '<tr><td>10 .. 20</td><td>30 .. 40</td><td>50 .. 60</td></tr>',
+            '</table>']
 
         t = table_type([arr])
         lines = t.pformat()
-        print(lines)
         assert lines == ['col0 [2,2]',
                          '----------',
                          '   1 .. 20',
@@ -56,14 +59,13 @@ class TestMultiD():
     def test_fake_multidim(self, table_type):
         """Test printing with 'fake' multidimensional column"""
         arr = [np.array([[(1,)],
-                         [(10,)]]),
+                         [(10,)]], dtype=np.int64),
                np.array([[(3,)],
-                         [(30,)]]),
+                         [(30,)]], dtype=np.int64),
                np.array([[(5,)],
-                         [(50,)]])]
+                         [(50,)]], dtype=np.int64)]
         t = table_type(arr)
         lines = t.pformat()
-        print(lines)
         assert lines == ['col0 [1,1] col1 [1,1] col2 [1,1]',
                          '---------- ---------- ----------',
                          '         1          3          5',
@@ -75,13 +77,16 @@ class TestMultiD():
                          '<tr><td>1</td><td>3</td><td>5</td></tr>',
                          '<tr><td>10</td><td>30</td><td>50</td></tr>',
                          '</table>']
-        assert t._repr_html_() == ('<table id="table{tid}"><thead><tr><th>col0 [1,1]</th><th>col1 [1,1]</th><th>col2'.format(tid=id(t)) +
-                                   ' [1,1]</th></tr></thead><tr><td>1</td><td>3</td><td>5</td>'
-                                   '</tr><tr><td>10</td><td>30</td><td>50</td></tr></table>')
+        assert t._repr_html_().splitlines() == [
+            '&lt;{0} masked={1} length=2&gt;'.format(table_type.__name__, t.masked),
+            '<table id="table{id}">'.format(id=id(t)),
+            '<thead><tr><th>col0 [1,1]</th><th>col1 [1,1]</th><th>col2 [1,1]</th></tr></thead>',
+            '<thead><tr><th>int64</th><th>int64</th><th>int64</th></tr></thead>',
+            '<tr><td>1</td><td>3</td><td>5</td></tr>', u'<tr><td>10</td><td>30</td><td>50</td></tr>',
+            '</table>']
 
         t = table_type([arr])
         lines = t.pformat()
-        print(lines)
         assert lines == ['col0 [2,1,1]',
                          '------------',
                          '     1 .. 10',
@@ -90,12 +95,16 @@ class TestMultiD():
 
 
 def test_html_escaping():
-    t = table.Table([('<script>alert("gotcha");</script>', 2, 3)])
-    assert t._repr_html_() == (
-        '<table id="table{id}"><thead><tr><th>col0</th></tr></thead>'.format(id=id(t)) +
-        '<tr><td>&lt;script&gt;alert(&quot;gotcha&quot;);&lt;/script&gt;</td>'
-        '</tr><tr><td>2</td></tr><tr><td>3</td></tr></table>')
-
+    t = table.Table([(str('<script>alert("gotcha");</script>'), 2, 3)])
+    assert t._repr_html_().splitlines() == [
+        '&lt;Table length=3&gt;',
+        '<table id="table{id}">'.format(id=id(t)),
+        '<thead><tr><th>col0</th></tr></thead>',
+        '<thead><tr><th>str33</th></tr></thead>',
+        '<tr><td>&lt;script&gt;alert(&quot;gotcha&quot;);&lt;/script&gt;</td></tr>',
+        '<tr><td>2</td></tr>',
+        '<tr><td>3</td></tr>',
+        '</table>']
 
 @pytest.mark.usefixtures('table_type')
 class TestPprint():
@@ -109,18 +118,24 @@ class TestPprint():
         self.tb['col19'].unit = 'kg s m**-2'
         self.ts = table_type(SMALL_ARR)
 
+    def test_empty_table(self, table_type):
+        t = table_type()
+        lines = t.pformat()
+        assert lines == ['<No columns>']
+        c = repr(t)
+        assert c.splitlines() == ['<{0} masked={1} length=0>'.format(table_type.__name__, t.masked),
+                                  '<No columns>']
+
     def test_format0(self, table_type):
         """Try getting screen size but fail to defaults because testing doesn't
         have access to screen (fcntl.ioctl fails).
         """
-        from ... import conf
-
         self._setup(table_type)
-        arr = np.arange(4000, dtype=np.float).reshape(100, 40)
+        arr = np.arange(4000, dtype=np.float64).reshape(100, 40)
         lines = table_type(arr).pformat()
         nlines, width = console.terminal_size()
         assert len(lines) == nlines
-        for line in lines:
+        for line in lines[:-1]:  # skip last "Length = .. rows" line
             assert (len(line) > width - 10 and
                     len(line) <= width)
 
@@ -132,10 +147,10 @@ class TestPprint():
                          '    km2                  ... kg s / m2',
                          '------------ ----------- ... ---------',
                          '0.000000e+00    1.000000 ...      19.0',
-                         '2.000000e+01   21.000000 ...      39.0',
                          '         ...         ... ...       ...',
                          '1.960000e+03 1961.000000 ...    1979.0',
-                         '1.980000e+03 1981.000000 ...    1999.0']
+                         '1.980000e+03 1981.000000 ...    1999.0',
+                         'Length = 100 rows']
 
     def test_format2(self, table_type):
         """Basic test of formatting, unit header row excluded"""
@@ -145,25 +160,24 @@ class TestPprint():
                          '------------ ----------- ... ------',
                          '0.000000e+00    1.000000 ...   19.0',
                          '2.000000e+01   21.000000 ...   39.0',
-                         '4.000000e+01   41.000000 ...   59.0',
                          '         ...         ... ...    ...',
                          '1.960000e+03 1961.000000 ... 1979.0',
-                         '1.980000e+03 1981.000000 ... 1999.0']
+                         '1.980000e+03 1981.000000 ... 1999.0',
+                         'Length = 100 rows']
 
     def test_format3(self, table_type):
         """Include the unit header row"""
         self._setup(table_type)
         lines = self.tb.pformat(max_lines=8, max_width=40, show_unit=True)
 
-        print(lines)
         assert lines == ['    col0         col1    ...   col19  ',
                          '    km2                  ... kg s / m2',
                          '------------ ----------- ... ---------',
                          '0.000000e+00    1.000000 ...      19.0',
-                         '2.000000e+01   21.000000 ...      39.0',
                          '         ...         ... ...       ...',
                          '1.960000e+03 1961.000000 ...    1979.0',
-                         '1.980000e+03 1981.000000 ...    1999.0']
+                         '1.980000e+03 1981.000000 ...    1999.0',
+                         'Length = 100 rows']
 
     def test_format4(self, table_type):
         """Do not include the name header row"""
@@ -173,10 +187,10 @@ class TestPprint():
                          '------------ ----------- ... ---------',
                          '0.000000e+00    1.000000 ...      19.0',
                          '2.000000e+01   21.000000 ...      39.0',
-                         '4.000000e+01   41.000000 ...      59.0',
                          '         ...         ... ...       ...',
                          '1.960000e+03 1961.000000 ...    1979.0',
-                         '1.980000e+03 1981.000000 ...    1999.0']
+                         '1.980000e+03 1981.000000 ...    1999.0',
+                         'Length = 100 rows']
 
     def test_noclip(self, table_type):
         """Basic table print"""
@@ -187,10 +201,12 @@ class TestPprint():
                          '   0    1    2',
                          '   3    4    5',
                          '   6    7    8',
-                         '   9   10   11']
+                         '   9   10   11',
+                         '  12   13   14',
+                         '  15   16   17']
 
     def test_clip1(self, table_type):
-        """max lines below hard limit of 6
+        """max lines below hard limit of 8
         """
         self._setup(table_type)
         lines = self.ts.pformat(max_lines=3, max_width=-1)
@@ -199,22 +215,27 @@ class TestPprint():
                          '   0    1    2',
                          '   3    4    5',
                          '   6    7    8',
-                         '   9   10   11']
+                         '   9   10   11',
+                         '  12   13   14',
+                         '  15   16   17']
 
     def test_clip2(self, table_type):
-        """max lines below hard limit of 6 and output longer than 6
+        """max lines below hard limit of 8 and output longer than 8
         """
         self._setup(table_type)
-        lines = self.ts.pformat(max_lines=3, max_width=-1, show_unit=True)
-        assert lines == ['col0 col1 col2',
-                         '              ',
-                         '---- ---- ----',
-                         '   0    1    2',
-                         ' ...  ...  ...',
-                         '   9   10   11']
+        lines = self.ts.pformat(max_lines=3, max_width=-1, show_unit=True, show_dtype=True)
+        assert lines == [' col0  col1  col2',
+                         '                 ',
+                         'int64 int64 int64',
+                         '----- ----- -----',
+                         '    0     1     2',
+                         '  ...   ...   ...',
+                         '   15    16    17',
+                         'Length = 6 rows']
+
 
     def test_clip3(self, table_type):
-        """Max lines below hard limit of 6 and max width below hard limit
+        """Max lines below hard limit of 8 and max width below hard limit
         of 10
         """
         self._setup(table_type)
@@ -224,14 +245,16 @@ class TestPprint():
                          '---- ...',
                          '   0 ...',
                          ' ... ...',
-                         '   9 ...']
+                         '  12 ...',
+                         '  15 ...',
+                         'Length = 6 rows']
 
     def test_clip4(self, table_type):
         """Test a range of max_lines"""
         self._setup(table_type)
-        for max_lines in range(130):
+        for max_lines in (0, 1, 4, 5, 6, 7, 8, 100, 101, 102, 103, 104, 130):
             lines = self.tb.pformat(max_lines=max_lines, show_unit=False)
-            assert len(lines) == max(6, min(102, max_lines))
+            assert len(lines) == max(8, min(102, max_lines))
 
 
 
@@ -270,13 +293,26 @@ class TestFormat():
 
     def test_column_format_with_threshold(self, table_type):
         from ... import conf
-        with conf.set_temp('max_lines', 6):
+        with conf.set_temp('max_lines', 8):
             t = table_type([np.arange(20)], names=['a'])
             t['a'].format = '%{0:}'
-            assert str(t['a']) == ' a \n---\n %0\n %1\n...\n%19'
+            assert str(t['a']).splitlines() == [' a ',
+                                                '---',
+                                                ' %0',
+                                                ' %1',
+                                                '...',
+                                                '%18',
+                                                '%19',
+                                                'Length = 20 rows']
             t['a'].format = '{ %4.2f }'
-            assert str(t['a']) == '    a    \n---------\n { 0.00 }\n' \
-                                  ' { 1.00 }\n      ...\n{ 19.00 }'
+            assert str(t['a']).splitlines() == ['    a    ',
+                                                '---------',
+                                                ' { 0.00 }',
+                                                ' { 1.00 }',
+                                                '      ...',
+                                                '{ 18.00 }',
+                                                '{ 19.00 }',
+                                                'Length = 20 rows']
 
     def test_column_format_func(self, table_type):
         # run most of functions twice
@@ -335,6 +371,19 @@ class TestFormat():
         with pytest.raises(ValueError):
             str(t['a'])
 
+    def test_column_alignment(self, table_type):
+        t = table_type([[1], [2], [3], [4]],
+                       names=('long title a', 'long title b',
+                              'long title c', 'long title d'))
+        t['long title a'].format = '<'
+        t['long title b'].format = '^'
+        t['long title c'].format = '>'
+        t['long title d'].format = '0='
+        assert str(t['long title a']) == 'long title a\n------------\n1           '
+        assert str(t['long title b']) == 'long title b\n------------\n     2      '
+        assert str(t['long title c']) == 'long title c\n------------\n           3'
+        assert str(t['long title d']) == 'long title d\n------------\n000000000004'
+
 
 class TestFormatWithMaskedElements():
 
@@ -364,18 +413,30 @@ class TestFormatWithMaskedElements():
         t['a'].format = '%4.2f {0:}'
         assert str(t['a']) == '   a   \n-------\n     --\n%4.2f 2\n     --'
 
-    def test_column_format_with_threshold(self):
+    def test_column_format_with_threshold(self, table_type):
         from ... import conf
-        with conf.set_temp('max_lines', 7):
-            t = Table([np.arange(20)], names=['a'], masked=True)
+        with conf.set_temp('max_lines', 8):
+            t = table_type([np.arange(20)], names=['a'])
+            t['a'].format = '%{0:}'
             t['a'].mask[0] = True
             t['a'].mask[-1] = True
-            t['a'].format = '%{0:}'
-            assert str(t['a']) == ' a \n---\n --\n %1\n...\n%18\n --'
+            assert str(t['a']).splitlines() == [' a ',
+                                                '---',
+                                                ' --',
+                                                ' %1',
+                                                '...',
+                                                '%18',
+                                                ' --',
+                                                'Length = 20 rows']
             t['a'].format = '{ %4.2f }'
-            assert str(t['a']) == '    a    \n---------\n       --\n' \
-                                  ' { 1.00 }\n      ...\n{ 18.00 }\n' \
-                                  '       --'
+            assert str(t['a']).splitlines() == ['    a    ',
+                                                '---------',
+                                                '       --',
+                                                ' { 1.00 }',
+                                                '      ...',
+                                                '{ 18.00 }',
+                                                '       --',
+                                                'Length = 20 rows']
 
     def test_column_format_func(self):
         # run most of functions twice
@@ -442,8 +503,8 @@ class TestFormatWithMaskedElements():
         arr = [np.array([[1, 2],
                          [10, 20]])]
         t = Table(arr, names=['a'], masked=True)
-        t['a'].mask[0,1] = True
-        t['a'].mask[1,1] = True
+        t['a'].mask[0, 1] = True
+        t['a'].mask[1, 1] = True
         # mathematical function
         t['a'].format = lambda x: str(x * 3.)
         outstr = '  a [2]   \n----------\n 3.0 .. --\n30.0 .. --'
@@ -477,5 +538,132 @@ def test_pprint_nameless_col():
     """Regression test for #2213, making sure a nameless column can be printed
     using None as the name.
     """
-    col = table.Column([1.,2.])
+    col = table.Column([1., 2.])
     assert str(col).startswith('None')
+
+
+def test_html():
+    """Test HTML printing"""
+    dat = np.array([1., 2.], dtype=np.float32)
+    t = Table([dat], names=['a'])
+
+    lines = t.pformat(html=True)
+    assert lines == ['<table id="table{id}">'.format(id=id(t)),
+                     u'<thead><tr><th>a</th></tr></thead>',
+                     u'<tr><td>1.0</td></tr>',
+                     u'<tr><td>2.0</td></tr>',
+                     u'</table>']
+
+    lines = t.pformat(html=True, tableclass='table-striped')
+    assert lines == [
+        '<table id="table{id}" class="table-striped">'.format(id=id(t)),
+        u'<thead><tr><th>a</th></tr></thead>',
+        u'<tr><td>1.0</td></tr>',
+        u'<tr><td>2.0</td></tr>',
+        u'</table>']
+
+    lines = t.pformat(html=True, tableclass=['table', 'table-striped'])
+    assert lines == [
+        '<table id="table{id}" class="table table-striped">'.format(id=id(t)),
+        u'<thead><tr><th>a</th></tr></thead>',
+        u'<tr><td>1.0</td></tr>',
+        u'<tr><td>2.0</td></tr>',
+        u'</table>']
+
+def test_align():
+    t = simple_table(2, kinds='iS')
+    assert t.pformat() == [' a   b ',
+                           '--- ---',
+                           '  1   b',
+                           '  2   c']
+    # Use column format attribute
+    t['a'].format = '<'
+    assert t.pformat() == [' a   b ',
+                           '--- ---',
+                           '1     b',
+                           '2     c']
+
+    # Now override column format attribute with various combinations of align
+    tpf = [' a   b ',
+           '--- ---',
+           ' 1   b ',
+           ' 2   c ']
+    for align in ('^', ['^', '^'], ('^', '^')):
+        assert tpf == t.pformat(align=align)
+
+    assert t.pformat(align='<') == [' a   b ',
+                                    '--- ---',
+                                    '1   b  ',
+                                    '2   c  ']
+    assert t.pformat(align='0=') == [' a   b ',
+                                     '--- ---',
+                                     '001 00b',
+                                     '002 00c']
+
+    assert t.pformat(align=['<', '^']) == [' a   b ',
+                                           '--- ---',
+                                           '1    b ',
+                                           '2    c ']
+
+    # Now use fill characters.  Stress the system using a fill
+    # character that is the same as an align character.
+    t = simple_table(2, kinds='iS')
+
+    assert t.pformat(align='^^') == [' a   b ',
+                                     '--- ---',
+                                     '^1^ ^b^',
+                                     '^2^ ^c^']
+
+    assert t.pformat(align='^>') == [' a   b ',
+                                     '--- ---',
+                                     '^^1 ^^b',
+                                     '^^2 ^^c']
+
+    assert t.pformat(align='^<') == [' a   b ',
+                                     '--- ---',
+                                     '1^^ b^^',
+                                     '2^^ c^^']
+
+    # Complicated interaction (same as narrative docs example)
+    t1 = Table([[1.0, 2.0], [1, 2]], names=['column1', 'column2'])
+    t1['column1'].format = '#^.2f'
+
+    assert t1.pformat() == ['column1 column2',
+                            '------- -------',
+                            '##1.00#       1',
+                            '##2.00#       2']
+
+    assert t1.pformat(align='!<') ==  ['column1 column2',
+                                       '------- -------',
+                                       '1.00!!! 1!!!!!!',
+                                       '2.00!!! 2!!!!!!']
+
+    assert t1.pformat(align=[None, '!<']) == ['column1 column2',
+                                              '------- -------',
+                                              '##1.00# 1!!!!!!',
+                                              '##2.00# 2!!!!!!']
+
+    # Zero fill
+    t['a'].format = '+d'
+    assert t.pformat(align='0=') == [' a   b ',
+                                     '--- ---',
+                                     '+01 00b',
+                                     '+02 00c']
+
+    with pytest.raises(ValueError):
+        t.pformat(align=['fail'])
+
+    with pytest.raises(TypeError):
+        t.pformat(align=0)
+
+    with pytest.raises(TypeError):
+        t.pprint(align=0)
+
+    # Make sure pprint() does not raise an exception
+    t.pprint()
+
+    with pytest.raises(ValueError):
+        t.pprint(align=['<', '<', '<'])
+
+    with pytest.raises(ValueError):
+        t.pprint(align='x=')
