@@ -5,15 +5,30 @@
 
 import copy
 import gc
+import sys
 
 import numpy as np
+from numpy.testing import assert_allclose
 
 from ...extern import six
-from ...tests.helper import pytest, assert_follows_unicode_guidelines
+from ...io import fits
+from ...tests.helper import (pytest, assert_follows_unicode_guidelines,
+                             ignore_warnings)
+from ...utils.data import get_pkg_data_filename
 from ... import table
 from ... import units as u
 from .conftest import MaskedTable
 
+try:
+    with ignore_warnings(DeprecationWarning):
+        # Ignore DeprecationWarning on pandas import in Python 3.5--see
+        # https://github.com/astropy/astropy/issues/4380
+        import pandas
+except ImportError:
+    HAS_PANDAS = False
+else:
+    HAS_PANDAS = True
+
 
 class SetupData(object):
     def _setup(self, table_types):
@@ -264,12 +279,13 @@ class TestNewFromColumns():
             table_types.Table(cols)
 
     def test_name_none(self, table_types):
-        """Column with name=None can init a table IFF names are supplied"""
-        c = table_types.Column(data=[1, 2])
-        table_types.Table([c], names=('c',))
-        with pytest.raises(TypeError):
-            table_types.Table([c])
-
+        """Column with name=None can init a table whether or not names are supplied"""
+        c = table_types.Column(data=[1, 2], name='c')
+        d = table_types.Column(data=[3, 4])
+        t = table_types.Table([c, d], names=(None, 'd'))
+        assert t.colnames == ['c', 'd']
+        t = table_types.Table([c, d])
+        assert t.colnames == ['c', 'col1']
 
 @pytest.mark.usefixtures('table_types')
 class TestReverse():
@@ -476,17 +492,39 @@ class TestAddColumns(SetupData):
         t.add_column(self.a)
         with pytest.raises(ValueError):
             t.add_column(table_types.Column(name='a', data=[0, 1, 2]))
+        t.add_column(table_types.Column(name='a', data=[0, 1, 2]),
+                     rename_duplicate=True)
         t.add_column(self.b)
         t.add_column(self.c)
-        assert t.colnames == ['a', 'b', 'c']
+        assert t.colnames == ['a', 'a_1', 'b', 'c']
+        t.add_column(table_types.Column(name='a', data=[0, 1, 2]),
+                     rename_duplicate=True)
+        assert t.colnames == ['a', 'a_1', 'b', 'c', 'a_2']
+
+        # test adding column from a separate Table
+        t1 = table_types.Table()
+        t1.add_column(self.a)
+        with pytest.raises(ValueError):
+            t.add_column(t1['a'])
+        t.add_column(t1['a'], rename_duplicate=True)
+
+        t1['a'][0] = 100  # Change original column
+        assert t.colnames == ['a', 'a_1', 'b', 'c', 'a_2', 'a_3']
+        assert t1.colnames == ['a']
+
+        # Check new column didn't change (since name conflict forced a copy)
+        assert t['a_3'][0] == self.a[0]
 
     def test_add_duplicate_columns(self, table_types):
         self._setup(table_types)
         t = table_types.Table([self.a, self.b, self.c])
         with pytest.raises(ValueError):
             t.add_columns([table_types.Column(name='a', data=[0, 1, 2]), table_types.Column(name='b', data=[0, 1, 2])])
+        t.add_columns([table_types.Column(name='a', data=[0, 1, 2]),
+                       table_types.Column(name='b', data=[0, 1, 2])],
+                      rename_duplicate=True)
         t.add_column(self.d)
-        assert t.colnames == ['a', 'b', 'c', 'd']
+        assert t.colnames == ['a', 'b', 'c', 'a_1', 'b_1', 'd']
 
 
 @pytest.mark.usefixtures('table_types')
@@ -1057,6 +1095,49 @@ class TestConvertNumpyArray():
         with pytest.raises(ValueError):
             np_data = np.array(d, dtype=[(str('c'), 'i8'), (str('d'), 'i8')])
 
+    def test_as_array_byteswap(self, table_types):
+        """Test for https://github.com/astropy/astropy/pull/4080"""
+
+        byte_orders = ('>', '<')
+        native_order = byte_orders[sys.byteorder == 'little']
+
+        for order in byte_orders:
+            col = table_types.Column([1.0, 2.0], name='a', dtype=order + 'f8')
+            t = table_types.Table([col])
+            arr = t.as_array()
+            assert arr['a'].dtype.byteorder in (native_order, '=')
+            arr = t.as_array(keep_byteorder=True)
+            if order == native_order:
+                assert arr['a'].dtype.byteorder in (order, '=')
+            else:
+                assert arr['a'].dtype.byteorder == order
+
+    def test_byteswap_fits_array(self, table_types):
+        """
+        Test for https://github.com/astropy/astropy/pull/4080, demonstrating
+        that FITS tables are converted to native byte order.
+        """
+
+        non_native_order = ('>', '<')[sys.byteorder != 'little']
+
+        filename = get_pkg_data_filename('data/tb.fits',
+                                         'astropy.io.fits.tests')
+        t = table_types.Table.read(filename)
+        arr = t.as_array()
+
+        for idx in range(len(arr.dtype)):
+            assert arr.dtype[idx].byteorder != non_native_order
+
+        with fits.open(filename) as hdul:
+            data = hdul[1].data
+            for colname in data.columns.names:
+                assert np.all(data[colname] == arr[colname])
+
+            arr2 = t.as_array(keep_byteorder=True)
+            for colname in data.columns.names:
+                assert (data[colname].dtype.byteorder ==
+                        arr2[colname].dtype.byteorder)
+
 
 def _assert_copies(t, t2, deep=True):
     assert t.colnames == t2.colnames
@@ -1375,3 +1456,155 @@ def test_table_init_from_degenerate_arrays(table_types):
 
     t = table_types.Table(np.array([1, 2, 3]))
     assert len(t.columns) == 3
+
+
+@pytest.mark.skipif('not HAS_PANDAS')
+class TestPandas(object):
+
+    def test_simple(self):
+
+        t = table.Table()
+
+        for endian in ['<', '>']:
+            for kind in ['f', 'i']:
+                for byte in ['2','4','8']:
+                    dtype = np.dtype(endian + kind + byte)
+                    x = np.array([1,2,3], dtype=dtype)
+                    t[endian + kind + byte] = x
+
+        t['u'] = ['a','b','c']
+        t['s'] = [b'a', b'b', b'c']
+
+        d = t.to_pandas()
+
+        for column in t.columns:
+            if column == 'u':
+                assert np.all(t['u'] == np.array(['a','b','c']))
+                assert d[column].dtype == np.dtype("O")  # upstream feature of pandas
+            elif column == 's':
+                assert np.all(t['s'] == np.array([b'a',b'b',b'c']))
+                assert d[column].dtype == np.dtype("O")  # upstream feature of pandas
+            else:
+                # We should be able to compare exact values here
+                assert np.all(t[column] == d[column])
+                if t[column].dtype.byteorder in ('=', '|'):
+                    assert d[column].dtype == t[column].dtype
+                else:
+                    assert d[column].dtype == t[column].byteswap().newbyteorder().dtype
+
+
+        # Regression test for astropy/astropy#1156 - the following code gave a
+        # ValueError: Big-endian buffer not supported on little-endian
+        # compiler. We now automatically swap the endian-ness to native order
+        # upon adding the arrays to the data frame.
+        d[['<i4','>i4']]
+        d[['<f4','>f4']]
+
+        t2 = table.Table.from_pandas(d)
+
+        for column in t.columns:
+            if column in ('u', 's'):
+                assert np.all(t[column] == t2[column])
+            else:
+                assert_allclose(t[column], t2[column])
+            if t[column].dtype.byteorder in ('=', '|'):
+                assert t[column].dtype == t2[column].dtype
+            else:
+                assert t[column].byteswap().newbyteorder().dtype == t2[column].dtype
+
+    def test_2d(self):
+
+        t = table.Table()
+        t['a'] = [1,2,3]
+        t['b'] = np.ones((3,2))
+
+        with pytest.raises(ValueError) as exc:
+            t.to_pandas()
+        assert exc.value.args[0] == "Cannot convert a table with multi-dimensional columns to a pandas DataFrame"
+
+    def test_mixin(self):
+
+        from ...coordinates import SkyCoord
+
+        t = table.Table()
+        t['c'] = SkyCoord([1,2,3], [4,5,6], unit='deg')
+
+        with pytest.raises(ValueError) as exc:
+            t.to_pandas()
+        assert exc.value.args[0] == "Cannot convert a table with mixin columns to a pandas DataFrame"
+
+    def test_masking(self):
+
+        t = table.Table(masked=True)
+
+        t['a'] = [1, 2, 3]
+        t['a'].mask = [True, False, True]
+
+        t['b'] = [1., 2., 3.]
+        t['b'].mask = [False, False, True]
+
+        t['u'] = ['a','b','c']
+        t['u'].mask = [False, True, False]
+
+        t['s'] = [b'a', b'b', b'c']
+        t['s'].mask = [False, True, False]
+
+        d = t.to_pandas()
+
+        t2 = table.Table.from_pandas(d)
+
+        for name, column in t.columns.items():
+            assert np.all(column.data == t2[name].data)
+            assert np.all(column.mask == t2[name].mask)
+            # Masked integer type comes back as float.  Nothing we can do about this.
+            if column.dtype.kind == 'i':
+                assert t2[name].dtype.kind == 'f'
+            else:
+                if column.dtype.byteorder in ('=', '|'):
+                    assert column.dtype == t2[name].dtype
+                else:
+                    assert column.byteswap().newbyteorder().dtype == t2[name].dtype
+
+
+@pytest.mark.usefixtures('table_types')
+class TestReplaceColumn(SetupData):
+    def test_fail_replace_column(self, table_types):
+        """Raise exception when trying to replace column via table.columns object"""
+        self._setup(table_types)
+        t = table_types.Table([self.a, self.b])
+
+        with pytest.raises(ValueError):
+            t.columns['a'] = [1, 2, 3]
+
+        with pytest.raises(ValueError):
+            t.replace_column('not there', [1, 2, 3])
+
+    def test_replace_column(self, table_types):
+        """Replace existing column with a new column"""
+        self._setup(table_types)
+        t = table_types.Table([self.a, self.b])
+        ta = t['a']
+        tb = t['b']
+
+        vals = [1.2, 3.4, 5.6]
+        for col in (vals,
+                    table_types.Column(vals),
+                    table_types.Column(vals, name='a'),
+                    table_types.Column(vals, name='b')):
+            t.replace_column('a', col)
+            assert np.all(t['a'] == vals)
+            assert t['a'] is not ta  # New a column
+            assert t['b'] is tb  # Original b column unchanged
+            assert t.colnames == ['a', 'b']
+            assert t['a'].meta == {}
+            assert t['a'].format is None
+
+    def test_replace_index_column(self, table_types):
+        """Replace index column and generate expected exception"""
+        self._setup(table_types)
+        t = table_types.Table([self.a, self.b])
+        t.add_index('a')
+
+        with pytest.raises(ValueError) as err:
+            t.replace_column('a', [1, 2, 3])
+        assert err.value.args[0] == 'cannot replace a table index column'
