@@ -5,16 +5,22 @@
 import copy
 import functools
 import sys
-
-from datetime import datetime, tzinfo, timedelta
+import datetime
 
 import numpy as np
 
 from ...tests.helper import pytest, catch_warnings
 from ...extern import six
-from .. import Time, ScaleValueError, erfa_time, TIME_SCALES
+from ...utils import isiterable
+from .. import (Time, ScaleValueError, erfa_time, TIME_SCALES, TimeString,
+                TimezoneInfo)
 from ...coordinates import EarthLocation
-
+from ... import units as u
+try:
+    import pytz
+    HAS_PYTZ = True
+except ImportError:
+    HAS_PYTZ = False
 
 allclose_jd = functools.partial(np.allclose, rtol=2. ** -52, atol=0)
 allclose_jd2 = functools.partial(np.allclose, rtol=2. ** -52,
@@ -25,6 +31,8 @@ allclose_year = functools.partial(np.allclose, rtol=2. ** -52,
                                   atol=0.)  # 14 microsec at current epoch
 
 
+
+
 class TestBasic():
     """Basic tests stemming from initial example and API reference"""
 
@@ -157,31 +165,48 @@ class TestBasic():
         assert np.all(t7[:, 2]._time.jd2 == t7._time.jd2[:, 2])
         assert np.all(t7[:, 0]._time.jd1 == t._time.jd1)
         assert np.all(t7[:, 0]._time.jd2 == t._time.jd2)
+        # Get tdb to check that delta_tdb_tt attribute is sliced properly.
+        t7_tdb = t7.tdb
+        assert t7_tdb[0, 0].delta_tdb_tt == t7_tdb.delta_tdb_tt[0, 0]
+        assert np.all(t7_tdb[5].delta_tdb_tt == t7_tdb.delta_tdb_tt[5])
+        assert np.all(t7_tdb[:, 2].delta_tdb_tt == t7_tdb.delta_tdb_tt[:, 2])
+        # Explicitly set delta_tdb_tt attribute. Now it should not be sliced.
+        t7.delta_tdb_tt = 0.1
+        t7_tdb2 = t7.tdb
+        assert t7_tdb2[0, 0].delta_tdb_tt == 0.1
+        assert t7_tdb2[5].delta_tdb_tt == 0.1
+        assert t7_tdb2[:, 2].delta_tdb_tt == 0.1
+        # Check broadcasting of location.
+        t8 = Time(mjd[:, np.newaxis] + frac, format='mjd', scale='utc',
+                  location=(np.arange(len(frac)), np.arange(len(frac))))
+        assert t8[0, 0].location == t8.location[0, 0]
+        assert np.all(t8[5].location == t8.location[5])
+        assert np.all(t8[:, 2].location == t8.location[:, 2])
         # Finally check empty array.
-        t8 = t[:0]
-        assert t8.isscalar is False
-        assert t8.shape == (0,)
-        assert t8.size == 0
+        t9 = t[:0]
+        assert t9.isscalar is False
+        assert t9.shape == (0,)
+        assert t9.size == 0
 
     def test_properties(self):
         """Use properties to convert scales and formats.  Note that the UT1 to
         UTC transformation requires a supplementary value (``delta_ut1_utc``)
         that can be obtained by interpolating from a table supplied by IERS.
-        This will be included in the package later."""
+        This is tested separately."""
 
         t = Time('2010-01-01 00:00:00', format='iso', scale='utc')
         t.delta_ut1_utc = 0.3341  # Explicitly set one part of the xform
         assert allclose_jd(t.jd, 2455197.5)
         assert t.iso == '2010-01-01 00:00:00.000'
         assert t.tt.iso == '2010-01-01 00:01:06.184'
-        assert t.tai.iso == '2010-01-01 00:00:34.000'
+        assert t.tai.fits == '2010-01-01T00:00:34.000(TAI)'
         assert allclose_jd(t.utc.jd, 2455197.5)
         assert allclose_jd(t.ut1.jd, 2455197.500003867)
         assert t.tcg.isot == '2010-01-01T00:01:06.910'
         assert allclose_sec(t.unix, 1262304000.0)
         assert allclose_sec(t.cxcsec, 378691266.184)
         assert allclose_sec(t.gps, 946339215.0)
-        assert t.datetime == datetime(2010, 1, 1)
+        assert t.datetime == datetime.datetime(2010, 1, 1)
 
     def test_precision(self):
         """Set the output precision which is used for some formats.  This is
@@ -277,18 +302,18 @@ class TestBasic():
         t3 = Time(mjd, format='mjd', scale='utc', location=(lon, lat))
         assert t3.shape == (4, 2)
         assert t3.location.shape == ()
-        assert t3.tdb.shape == (4, 2)
+        assert t3.tdb.shape == t3.shape
         t4 = Time(mjd, format='mjd', scale='utc',
                   location=(np.array([lon, 0]), np.array([lat, 0])))
         assert t4.shape == (4, 2)
-        assert t4.location.shape == (2,)
-        assert t4.tdb.shape == (4, 2)
+        assert t4.location.shape == t4.shape
+        assert t4.tdb.shape == t4.shape
         t5 = Time(mjd, format='mjd', scale='utc',
                   location=(np.array([[lon], [0], [0], [0]]),
                             np.array([[lat], [0], [0], [0]])))
         assert t5.shape == (4, 2)
-        assert t5.location.shape == (4, 1)
-        assert t5.tdb.shape == (4, 2)
+        assert t5.location.shape == t5.shape
+        assert t5.tdb.shape == t5.shape
 
     def test_all_transforms(self):
         """Test that all transforms work.  Does not test correctness,
@@ -317,11 +342,14 @@ class TestBasic():
         Time('2000-01-01 12:23:34.0Z', format='iso', scale='utc')
         Time('2000-01-01T12:23:34.0', format='isot', scale='tai')
         Time('2000-01-01T12:23:34.0Z', format='isot', scale='utc')
+        Time('2000-01-01T12:23:34.0', format='fits')
+        Time('2000-01-01T12:23:34.0', format='fits', scale='tdb')
+        Time('2000-01-01T12:23:34.0(TDB)', format='fits')
         Time(2400000.5, 51544.0333981, format='jd', scale='tai')
         Time(0.0, 51544.0333981, format='mjd', scale='tai')
         Time('2000:001:12:23:34.0', format='yday', scale='tai')
         Time('2000:001:12:23:34.0Z', format='yday', scale='utc')
-        dt = datetime(2000, 1, 2, 3, 4, 5, 123456)
+        dt = datetime.datetime(2000, 1, 2, 3, 4, 5, 123456)
         Time(dt, format='datetime', scale='tai')
         Time([dt, dt], format='datetime', scale='tai')
 
@@ -330,8 +358,8 @@ class TestBasic():
         Test datetime format, including guessing the format from the input type
         by not providing the format keyword to Time.
         """
-        dt = datetime(2000, 1, 2, 3, 4, 5, 123456)
-        dt2 = datetime(2001, 1, 1)
+        dt = datetime.datetime(2000, 1, 2, 3, 4, 5, 123456)
+        dt2 = datetime.datetime(2001, 1, 1)
         t = Time(dt, scale='utc', precision=9)
         assert t.iso == '2000-01-02 03:04:05.123456000'
         assert t.datetime == dt
@@ -343,7 +371,7 @@ class TestBasic():
         assert np.all(t.value == [dt, dt2])
 
         t = Time('2000-01-01 01:01:01.123456789', scale='tai')
-        assert t.datetime == datetime(2000, 1, 1, 1, 1, 1, 123457)
+        assert t.datetime == datetime.datetime(2000, 1, 1, 1, 1, 1, 123457)
 
         # broadcasting
         dt3 = (dt + (dt2-dt)*np.arange(12)).reshape(4, 3)
@@ -389,6 +417,12 @@ class TestBasic():
         # regression test against #3396
         with pytest.raises(ValueError):
             Time(np.nan, format='jd', scale='utc')
+        with pytest.raises(ValueError):
+            Time('2000-01-02T03:04:05(TAI)', scale='utc')
+        with pytest.raises(ValueError):
+            Time('2000-01-02T03:04:05(TAI')
+        with pytest.raises(ValueError):
+            Time('2000-01-02T03:04:05(UT(NIST)')
 
     def test_utc_leap_sec(self):
         """Time behaves properly near or in UTC leap second.  This
@@ -525,6 +559,32 @@ class TestSubFormat():
                                          '2000-01-01 01:01',
                                          '2000-01-01 01:01']))
 
+    def test_fits_format(self):
+        """FITS format includes bigger years."""
+        # Heterogeneous input formats with in_subfmt='*' (default)
+        times = ['2000-01-01', '2000-01-01T01:01:01', '2000-01-01T01:01:01.123']
+        t = Time(times, format='fits', scale='tai')
+        assert np.all(t.fits == np.array(['2000-01-01T00:00:00.000(TAI)',
+                                          '2000-01-01T01:01:01.000(TAI)',
+                                          '2000-01-01T01:01:01.123(TAI)']))
+        # Explicit long format for output, default scale is UTC.
+        t2 = Time(times, format='fits', out_subfmt='long*')
+        assert np.all(t2.fits == np.array(['+02000-01-01T00:00:00.000(UTC)',
+                                           '+02000-01-01T01:01:01.000(UTC)',
+                                           '+02000-01-01T01:01:01.123(UTC)']))
+        # Implicit long format for output, because of negative year.
+        times[2] = '-00594-01-01'
+        t3 = Time(times, format='fits', scale='tai')
+        assert np.all(t3.fits == np.array(['+02000-01-01T00:00:00.000(TAI)',
+                                           '+02000-01-01T01:01:01.000(TAI)',
+                                           '-00594-01-01T00:00:00.000(TAI)']))
+        # Implicit long format for output, because of large positive year.
+        times[2] = '+10594-01-01'
+        t4 = Time(times, format='fits', scale='tai')
+        assert np.all(t4.fits == np.array(['+02000-01-01T00:00:00.000(TAI)',
+                                           '+02000-01-01T01:01:01.000(TAI)',
+                                           '+10594-01-01T00:00:00.000(TAI)']))
+
     def test_yday_format(self):
         """Year:Day_of_year format"""
         # Heterogeneous input formats with in_subfmt='*' (default)
@@ -556,6 +616,24 @@ class TestSubFormat():
         with pytest.raises(ScaleValueError):
             Time('2000:001:00:00:00', scale='bad scale')
 
+    def test_fits_scale(self):
+        """Test that scale gets interpreted correctly for FITS strings."""
+        t = Time('2000-01-02(TAI)')
+        assert t.scale == 'tai'
+        # Test deprecated scale.
+        t = Time('2000-01-02(IAT)')
+        assert t.scale == 'tai'
+        # Check that inconsistent scales lead to errors.
+        with pytest.raises(ValueError):
+            Time('2000-01-02(TAI)', scale='utc')
+        with pytest.raises(ValueError):
+            Time(['2000-01-02(TAI)', '2001-02-03(UTC)'])
+
+    def test_fits_scale_representation(self):
+        t = Time('1960-01-02T03:04:05.678(ET(NIST))')
+        assert t.scale == 'tt'
+        assert t.value == '1960-01-02T03:04:05.678(ET(NIST))'
+
     def test_scale_default(self):
         """Test behavior when no scale is provided"""
         # These first three are TimeFromEpoch and have an intrinsic time scale
@@ -583,11 +661,16 @@ class TestSubFormat():
         assert t2.value == '1998-01-01 00:00:00.000'
 
         # Value take from Chandra.Time.DateTime('2010:001:00:00:00').secs
-        t = Time(378691266.184, format='cxcsec', scale='utc')
+        t_cxcsec = 378691266.184
+        t = Time(t_cxcsec, format='cxcsec', scale='utc')
+        assert allclose_sec(t.value, t_cxcsec)
+        assert allclose_sec(t.cxcsec, t_cxcsec)
+        assert allclose_sec(t.tt.value, t_cxcsec)
+        assert allclose_sec(t.tt.cxcsec, t_cxcsec)
         assert t.yday == '2010:001:00:00:00.000'
         t = Time('2010:001:00:00:00.000', scale='utc')
-        assert allclose_sec(t.cxcsec, 378691266.184)
-        assert allclose_sec(t.tt.cxcsec, 378691266.184)
+        assert allclose_sec(t.cxcsec, t_cxcsec)
+        assert allclose_sec(t.tt.cxcsec, t_cxcsec)
 
         # Value from:
         #   d = datetime.datetime(2000, 1, 1)
@@ -709,7 +792,7 @@ def test_now():
     Tests creating a Time object with the `now` class method.
     """
 
-    now = datetime.utcnow()
+    now = datetime.datetime.utcnow()
     t = Time.now()
 
     assert t.format == 'datetime'
@@ -745,6 +828,24 @@ def test_decimalyear():
                            jd0 + 0.75 * d_jd])
 
 
+def test_fits_year0():
+    t = Time(1721425.5, format='jd')
+    assert t.fits == '0001-01-01T00:00:00.000(UTC)'
+    t = Time(1721425.5 - 366., format='jd')
+    assert t.fits == '+00000-01-01T00:00:00.000(UTC)'
+    t = Time(1721425.5 - 366. - 365., format='jd')
+    assert t.fits == '-00001-01-01T00:00:00.000(UTC)'
+
+
+def test_fits_year10000():
+    t = Time(5373484.5, format='jd', scale='tai')
+    assert t.fits == '+10000-01-01T00:00:00.000(TAI)'
+    t = Time(5373484.5 - 365., format='jd', scale='tai')
+    assert t.fits == '9999-01-01T00:00:00.000(TAI)'
+    t = Time(5373484.5, -1./24./3600., format='jd', scale='tai')
+    assert t.fits == '9999-12-31T23:59:59.000(TAI)'
+
+
 def test_dir():
     t = Time('2000:001', format='yday', scale='tai')
     assert 'utc' in dir(t)
@@ -807,10 +908,146 @@ def test_datetime_tzinfo():
     """
     Test #3160 that time zone info in datetime objects is respected.
     """
-    class TZm6(tzinfo):
+    class TZm6(datetime.tzinfo):
         def utcoffset(self, dt):
-            return timedelta(hours=-6)
+            return datetime.timedelta(hours=-6)
 
-    d = datetime(2002, 1, 2, 10, 3, 4, tzinfo=TZm6())
+    d = datetime.datetime(2002, 1, 2, 10, 3, 4, tzinfo=TZm6())
     t = Time(d)
-    assert t.value == datetime(2002, 1, 2, 16, 3, 4)
+    assert t.value == datetime.datetime(2002, 1, 2, 16, 3, 4)
+
+def test_subfmts_regex():
+    """
+    Test having a custom subfmts with a regular expression
+    """
+    class TimeLongYear(TimeString):
+        name = 'longyear'
+        subfmts = (('date',
+                    r'(?P<year>[+-]\d{5})-%m-%d',  # hybrid
+                    '{year:+06d}-{mon:02d}-{day:02d}'),)
+    t = Time('+02000-02-03', format='longyear')
+    assert t.value == '+02000-02-03'
+    assert t.jd == Time('2000-02-03').jd
+
+def test_set_format_basic():
+    """
+    Test basics of setting format attribute.
+    """
+    for format, value in (('jd', 2451577.5),
+                          ('mjd', 51577.0),
+                          ('cxcsec', 65923264.184),  # confirmed with Chandra.Time
+                          ('datetime', datetime.datetime(2000, 2, 3, 0, 0)),
+                          ('iso', '2000-02-03 00:00:00.000')):
+        t = Time('+02000-02-03', format='fits')
+        t0 = t.replicate()
+        t.format = format
+        assert t.value == value
+        # Internal jd1 and jd2 are preserved
+        assert t._time.jd1 is t0._time.jd1
+        assert t._time.jd2 is t0._time.jd2
+
+def test_set_format_shares_subfmt():
+    """
+    Set format and round trip through a format that shares out_subfmt
+    """
+    t = Time('+02000-02-03', format='fits', out_subfmt='date_hms', precision=5)
+    tc = t.copy()
+
+    t.format = 'isot'
+    assert t.precision == 5
+    assert t.out_subfmt == 'date_hms'
+    assert t.value == '2000-02-03T00:00:00.00000'
+
+    t.format = 'fits'
+    assert t.value == tc.value
+    assert t.precision == 5
+
+def test_set_format_does_not_share_subfmt():
+    """
+    Set format and round trip through a format that does not share out_subfmt
+    """
+    t = Time('+02000-02-03', format='fits', out_subfmt='longdate')
+
+    t.format = 'isot'
+    assert t.out_subfmt == '*'  # longdate_hms not there, goes to default
+    assert t.value == '2000-02-03T00:00:00.000'
+
+    t.format = 'fits'
+    assert t.out_subfmt == '*'
+    assert t.value == '2000-02-03T00:00:00.000(UTC)'  # date_hms
+
+def test_replicate_value_error():
+    """
+    Passing a bad format to replicate should raise ValueError, not KeyError.
+    PR #3857.
+    """
+    t1 = Time('2007:001', scale='tai')
+    with pytest.raises(ValueError) as err:
+        t1.replicate(format='definitely_not_a_valid_format')
+    assert 'format must be one of' in str(err)
+
+def test_remove_astropy_time():
+    """
+    Make sure that 'astropy_time' format is really gone after #3857.  Kind of
+    silly test but just to be sure.
+    """
+    t1 = Time('2007:001', scale='tai')
+    assert 'astropy_time' not in t1.FORMATS
+    with pytest.raises(ValueError) as err:
+        Time(t1, format='astropy_time')
+    assert 'format must be one of' in str(err)
+
+def test_isiterable():
+    """
+    Ensure that scalar `Time` instances are not reported as iterable by the
+    `isiterable` utility.
+
+    Regression test for https://github.com/astropy/astropy/issues/4048
+    """
+
+    t1 = Time.now()
+    assert not isiterable(t1)
+
+    t2 = Time(['1999-01-01 00:00:00.123456789', '2010-01-01 00:00:00'],
+              format='iso', scale='utc')
+    assert isiterable(t2)
+
+def test_to_datetime():
+    tz = TimezoneInfo(utc_offset=-10*u.hour, tzname='US/Hawaii')
+    # The above lines produces a `datetime.tzinfo` object similar to:
+    #     tzinfo = pytz.timezone('US/Hawaii')
+    time = Time('2010-09-03 00:00:00')
+    tz_aware_datetime = time.to_datetime(tz)
+    assert tz_aware_datetime.time() == datetime.time(14, 0)
+    forced_to_astropy_time = Time(tz_aware_datetime)
+    assert tz.tzname(time.datetime) == tz_aware_datetime.tzname()
+    assert time == forced_to_astropy_time
+
+    # Test non-scalar time inputs:
+    time = Time(['2010-09-03 00:00:00', '2005-09-03 06:00:00',
+                 '1990-09-03 06:00:00'])
+    tz_aware_datetime = time.to_datetime(tz)
+    forced_to_astropy_time = Time(tz_aware_datetime)
+    for dt, tz_dt in zip(time.datetime, tz_aware_datetime):
+        assert tz.tzname(dt) == tz_dt.tzname()
+    assert np.all(time == forced_to_astropy_time)
+
+@pytest.mark.skipif('not HAS_PYTZ')
+def test_to_datetime_pytz():
+
+    tz = pytz.timezone('US/Hawaii')
+    time = Time('2010-09-03 00:00:00')
+    tz_aware_datetime = time.to_datetime(tz)
+    forced_to_astropy_time = Time(tz_aware_datetime)
+    assert tz_aware_datetime.time() == datetime.time(14, 0)
+    assert tz.tzname(time.datetime) == tz_aware_datetime.tzname()
+    assert time == forced_to_astropy_time
+
+    # Test non-scalar time inputs:
+    time = Time(['2010-09-03 00:00:00', '2005-09-03 06:00:00',
+                 '1990-09-03 06:00:00'])
+    tz_aware_datetime = time.to_datetime(tz)
+    forced_to_astropy_time = Time(tz_aware_datetime)
+    for dt, tz_dt in zip(time.datetime, tz_aware_datetime):
+        assert tz.tzname(dt) == tz_dt.tzname()
+    assert np.all(time == forced_to_astropy_time)
