@@ -6,6 +6,7 @@ import numpy as np
 from ...tests.helper import pytest
 from .. import Time, TimeDelta, OperandTypeError
 from ... import units as u
+from ...table import Column
 
 allclose_sec = functools.partial(np.allclose, rtol=2. ** -52,
                                  atol=2. ** -52 * 24 * 3600)  # 20 ps atol
@@ -53,6 +54,20 @@ class TestTimeQuantity():
         with pytest.raises(u.UnitsError):
             Time(2450000.*u.dimensionless_unscaled, format='jd', scale='utc')
 
+    def test_column_with_and_without_units(self):
+        """Ensure a Column without a unit is treated as an array [#3648]"""
+        a = np.arange(50000., 50010.)
+        ta = Time(a, format='mjd')
+        c1 = Column(np.arange(50000., 50010.), name='mjd')
+        tc1 = Time(c1, format='mjd')
+        assert np.all(ta == tc1)
+        c2 = Column(np.arange(50000., 50010.), name='mjd', unit='day')
+        tc2 = Time(c2, format='mjd')
+        assert np.all(ta == tc2)
+        c3 = Column(np.arange(50000., 50010.), name='mjd', unit='m')
+        with pytest.raises(u.UnitsError):
+            Time(c3, format='mjd')
+
     def test_no_quantity_input_allowed(self):
         """Time formats that are not allowed to take Quantity input."""
         qy = 1990.*u.yr
@@ -171,3 +186,23 @@ class TestTimeDeltaQuantity():
         t0 = TimeDelta(np.arange(12.).reshape(4, 3), format='sec')
         with pytest.raises(ValueError):
             t0 + np.arange(4.) * u.s
+
+
+class TestDeltaAttributes():
+    def test_delta_ut1_utc(self):
+        t = Time('2010-01-01 00:00:00', format='iso', scale='utc', precision=6)
+        t.delta_ut1_utc = 0.3 * u.s
+        assert t.ut1.iso == '2010-01-01 00:00:00.300000'
+        t.delta_ut1_utc = 0.4 / 60. * u.minute
+        assert t.ut1.iso == '2010-01-01 00:00:00.400000'
+        with pytest.raises(u.UnitsError):
+            t.delta_ut1_utc = 0.4 * u.m
+
+    def test_delta_tdb_tt(self):
+        t = Time('2010-01-01 00:00:00', format='iso', scale='tt', precision=6)
+        t.delta_tdb_tt = 20. * u.second
+        assert t.tdb.iso == '2010-01-01 00:00:20.000000'
+        t.delta_tdb_tt = 30. / 60. * u.minute
+        assert t.tdb.iso == '2010-01-01 00:00:30.000000'
+        with pytest.raises(u.UnitsError):
+            t.delta_tdb_tt = 0.4 * u.m
