@@ -21,14 +21,14 @@ from ..utils.decorators import lazyproperty, deprecated
 from ..utils.exceptions import AstropyWarning
 from ..utils.misc import isiterable, InheritDocstrings
 from .utils import (is_effectively_unity, sanitize_scale, validate_power,
-                    add_powers)
+                    resolve_fractions)
 from . import format as unit_format
 
-# TODO: Support functional units, e.g. log(x), ln(x)
+# TODO: Support function units, e.g. log(x), ln(x)
 
 __all__ = [
-    'UnitsError', 'UnitsWarning', 'UnitBase', 'NamedUnit',
-    'IrreducibleUnit', 'Unit', 'def_unit', 'CompositeUnit',
+    'UnitsError', 'UnitsWarning', 'UnitConversionError', 'UnitBase',
+    'NamedUnit', 'IrreducibleUnit', 'Unit', 'def_unit', 'CompositeUnit',
     'PrefixUnit', 'UnrecognizedUnit', 'get_current_unit_registry',
     'set_enabled_units', 'add_enabled_units',
     'set_enabled_equivalencies', 'add_enabled_equivalencies',
@@ -95,8 +95,8 @@ def _normalize_equivalencies(equivalencies):
         else:
             raise ValueError(
                 "Invalid equivalence entry {0}: {1!r}".format(i, equiv))
-        if not (isinstance(funit, UnitBase) and
-                (isinstance(tunit, UnitBase) or tunit is None) and
+        if not (funit is Unit(funit) and
+                (tunit is None or tunit is Unit(tunit)) and
                 six.callable(a) and
                 six.callable(b)):
             raise ValueError(
@@ -359,11 +359,13 @@ def add_enabled_units(units):
       Angstrom     | 1e-10 m         | AA, angstrom          ,
       cm           | 0.01 m          | centimeter            ,
       ft           | 0.3048 m        | foot                  ,
+      fur          | 201.168 m       | furlong               ,
       inch         | 0.0254 m        |                       ,
       lyr          | 9.46073e+15 m   | lightyear             ,
       m            | irreducible     | meter                 ,
       mi           | 1609.34 m       | mile                  ,
       micron       | 1e-06 m         |                       ,
+      mil          | 2.54e-05 m      | thou                  ,
       nmi          | 1852 m          | nauticalmile, NM      ,
       pc           | 3.08568e+16 m   | parsec                ,
       solRad       | 6.95508e+08 m   | R_sun, Rsun           ,
@@ -449,9 +451,11 @@ class UnitScaleError(UnitsError, ValueError):
     pass
 
 
-# Maintain error in old location for backward compatibility
-from .format import fits as _fits
-_fits.UnitScaleError = UnitScaleError
+class UnitConversionError(UnitsError, ValueError):
+    """
+    Used specifically for errors related to converting between units or
+    interpreting units in terms of other units.
+    """
 
 
 class UnitsWarning(AstropyWarning):
@@ -489,22 +493,22 @@ class UnitBase(object):
         -------
         Latex string
         """
-        return unit_format.Latex().to_string(self)
+        return unit_format.Latex.to_string(self)
 
     def __bytes__(self):
         """Return string representation for unit"""
-        return unit_format.Generic().to_string(self).encode('ascii')
+        return unit_format.Generic.to_string(self).encode('ascii')
     if six.PY2:
         __str__ = __bytes__
 
     def __unicode__(self):
         """Return string representation for unit"""
-        return unit_format.Generic().to_string(self)
+        return unit_format.Generic.to_string(self)
     if six.PY3:
         __str__ = __unicode__
 
     def __repr__(self):
-        string = unit_format.Generic().to_string(self)
+        string = unit_format.Generic.to_string(self)
         if six.PY2:
             string = string.encode('unicode_escape')
 
@@ -532,7 +536,6 @@ class UnitBase(object):
         raise AttributeError(
             "Can not get names from unnamed units. "
             "Perhaps you meant to_string()?")
-        return self._names
 
     @property
     def name(self):
@@ -583,6 +586,7 @@ class UnitBase(object):
             The name of a format or a formatter object.  If not
             provided, defaults to the generic format.
         """
+
         f = unit_format.get_format(format)
         return f.to_string(self)
 
@@ -632,24 +636,30 @@ class UnitBase(object):
                 return self
             return CompositeUnit(1, [self, m], [1, -1], _error_check=False)
 
-        # Cannot handle this as Unit, re-try as Quantity
-        from .quantity import Quantity
-        return Quantity(1, self) / m
+        try:
+            # Cannot handle this as Unit, re-try as Quantity
+            from .quantity import Quantity
+            return Quantity(1, self) / m
+        except TypeError:
+            return NotImplemented
 
     def __rdiv__(self, m):
         if isinstance(m, (bytes, six.text_type)):
             return Unit(m) / self
 
-        # Cannot handle this as Unit.  Here, m cannot be a Quantity,
-        # so we make it into one, fasttracking when it does not have a unit,
-        # for the common case of <array> / <unit>.
-        from .quantity import Quantity
-        if hasattr(m, 'unit'):
-            result = Quantity(m)
-            result /= self
-            return result
-        else:
-            return Quantity(m, self**(-1))
+        try:
+            # Cannot handle this as Unit.  Here, m cannot be a Quantity,
+            # so we make it into one, fasttracking when it does not have a unit,
+            # for the common case of <array> / <unit>.
+            from .quantity import Quantity
+            if hasattr(m, 'unit'):
+                result = Quantity(m)
+                result /= self
+                return result
+            else:
+                return Quantity(m, self**(-1))
+        except TypeError:
+            return NotImplemented
 
     __truediv__ = __div__
 
@@ -667,8 +677,11 @@ class UnitBase(object):
             return CompositeUnit(1, [self, m], [1, 1], _error_check=False)
 
         # Cannot handle this as Unit, re-try as Quantity.
-        from .quantity import Quantity
-        return Quantity(1, self) * m
+        try:
+            from .quantity import Quantity
+            return Quantity(1, self) * m
+        except TypeError:
+            return NotImplemented
 
     def __rmul__(self, m):
         if isinstance(m, (bytes, six.text_type)):
@@ -677,13 +690,16 @@ class UnitBase(object):
         # Cannot handle this as Unit.  Here, m cannot be a Quantity,
         # so we make it into one, fasttracking when it does not have a unit
         # for the common case of <array> * <unit>.
-        from .quantity import Quantity
-        if hasattr(m, 'unit'):
-            result = Quantity(m)
-            result *= self
-            return result
-        else:
-            return Quantity(m, self)
+        try:
+            from .quantity import Quantity
+            if hasattr(m, 'unit'):
+                result = Quantity(m)
+                result *= self
+                return result
+            else:
+                return Quantity(m, self)
+        except TypeError:
+            return NotImplemented
 
     def __hash__(self):
         # This must match the hash used in CompositeUnit for a unit
@@ -698,6 +714,12 @@ class UnitBase(object):
             other = Unit(other, parse_strict='silent')
         except (ValueError, UnitsError, TypeError):
             return False
+
+        # Other is Unit-like, but the test below requires it is a UnitBase
+        # instance; if it is not, give up (so that other can try).
+        if not isinstance(other, UnitBase):
+            return NotImplemented
+
         try:
             return is_effectively_unity(self._to(other))
         except UnitsError:
@@ -749,8 +771,8 @@ class UnitBase(object):
         if isinstance(other, tuple):
             return any(self.is_equivalent(u, equivalencies=equivalencies)
                        for u in other)
-        else:
-            other = Unit(other, parse_strict='silent')
+
+        other = Unit(other, parse_strict='silent')
 
         return self._is_equivalent(other, equivalencies)
 
@@ -785,7 +807,7 @@ class UnitBase(object):
 
         return False
 
-    def _apply_equivalences(self, unit, other, equivalencies):
+    def _apply_equivalencies(self, unit, other, equivalencies):
         """
         Internal function (used from `_get_converter`) to apply
         equivalence pairs.
@@ -795,16 +817,14 @@ class UnitBase(object):
                 return func(_condition_arg(v) / scale1) * scale2
             return convert
 
-        orig_unit = unit
-        orig_other = other
-
-        unit = self.decompose()
-        other = other.decompose()
+        if hasattr(other, 'equivalencies'):
+            equivalencies += other.equivalencies
 
         for funit, tunit, a, b in equivalencies:
             if tunit is None:
                 try:
-                    ratio_in_funit = (other/unit).decompose([funit])
+                    ratio_in_funit = (other.decompose() /
+                                      unit.decompose()).decompose([funit])
                     return make_converter(ratio_in_funit.scale, a, 1.)
                 except UnitsError:
                     pass
@@ -832,10 +852,10 @@ class UnitBase(object):
                 unit_str = "'{0}'".format(unit_str)
             return unit_str
 
-        unit_str = get_err_str(orig_unit)
-        other_str = get_err_str(orig_other)
+        unit_str = get_err_str(unit)
+        other_str = get_err_str(other)
 
-        raise UnitsError(
+        raise UnitConversionError(
             "{0} and {1} are not convertible".format(
                 unit_str, other_str))
 
@@ -845,7 +865,7 @@ class UnitBase(object):
         try:
             scale = self._to(other)
         except UnitsError:
-            return self._apply_equivalences(
+            return self._apply_equivalencies(
                 self, other, self._normalize_equivalencies(equivalencies))
         return lambda val: scale * _condition_arg(val)
 
@@ -896,19 +916,22 @@ class UnitBase(object):
         if self is other:
             return 1.0
 
-        self_decomposed = self.decompose()
-        other_decomposed = other.decompose()
+        # Don't presume decomposition is possible; e.g.,
+        # conversion to function units is through equivalencies.
+        if isinstance(other, UnitBase):
+            self_decomposed = self.decompose()
+            other_decomposed = other.decompose()
 
-        # Check quickly whether equivalent.  This is faster than
-        # `is_equivalent`, because it doesn't generate the entire
-        # physical type list of both units.  In other words it "fails
-        # fast".
-        if(self_decomposed.powers == other_decomposed.powers and
-           all(self_base is other_base for (self_base, other_base)
-               in zip(self_decomposed.bases, other_decomposed.bases))):
-            return self_decomposed.scale / other_decomposed.scale
+            # Check quickly whether equivalent.  This is faster than
+            # `is_equivalent`, because it doesn't generate the entire
+            # physical type list of both units.  In other words it "fails
+            # fast".
+            if(self_decomposed.powers == other_decomposed.powers and
+               all(self_base is other_base for (self_base, other_base)
+                   in zip(self_decomposed.bases, other_decomposed.bases))):
+                return self_decomposed.scale / other_decomposed.scale
 
-        raise UnitsError(
+        raise UnitConversionError(
             "'{0!r}' is not a scaled version of '{1!r}'".format(self, other))
 
     def to(self, other, value=1.0, equivalencies=[]):
@@ -1480,9 +1503,9 @@ class NamedUnit(UnitBase):
 
         if doc is None:
             doc = self._generate_doc()
-
-        doc = textwrap.dedent(doc)
-        doc = textwrap.fill(doc)
+        else:
+            doc = textwrap.dedent(doc)
+            doc = textwrap.fill(doc)
 
         self.__doc__ = doc
 
@@ -1634,7 +1657,7 @@ class IrreducibleUnit(NamedUnit):
                         return CompositeUnit(scale, [base], [1],
                                              _error_check=False)
 
-            raise UnitsError(
+            raise UnitConversionError(
                 "Unit {0} can not be decomposed into the requested "
                 "bases".format(self))
 
@@ -1670,7 +1693,7 @@ class UnrecognizedUnit(IrreducibleUnit):
     if six.PY3:
         __str__ = __unicode__
 
-    def to_string(self, format=unit_format.Generic):
+    def to_string(self, format=None):
         return self.name
 
     def _unrecognized_operator(self, *args, **kwargs):
@@ -1718,7 +1741,7 @@ class _UnitMetaClass(InheritDocstrings):
                  doc=None, parse_strict='raise'):
 
         # Short-circuit if we're already a unit
-        if isinstance(s, UnitBase):
+        if hasattr(s, '_get_physical_type_id'):
             return s
 
         # turn possible Quantity input for s or represents into a Unit
@@ -1771,9 +1794,9 @@ class _UnitMetaClass(InheritDocstrings):
                 if parse_strict == 'silent':
                     pass
                 else:
-                    # Deliberately not isinstance here. Subclasses
+                    # Deliberately not issubclass here. Subclasses
                     # should use their name.
-                    if type(f) is not unit_format.Generic:
+                    if f is not unit_format.Generic:
                         format_clause = f.name + ' '
                     else:
                         format_clause = ''
@@ -2023,7 +2046,8 @@ class CompositeUnit(UnitBase):
                         break
 
             if unit in new_parts:
-                new_parts[unit] = add_powers(new_parts[unit], power)
+                a, b = resolve_fractions(new_parts[unit], power)
+                new_parts[unit] = a + b
             else:
                 new_parts[unit] = power
             return scale
@@ -2044,7 +2068,8 @@ class CompositeUnit(UnitBase):
                     # needed for the corner case of mag=-0.4*dex
                     scale *= cmath.exp(p * cmath.log(b._scale))
                 for b_sub, p_sub in zip(b._bases, b._powers):
-                    scale = add_unit(b_sub, p_sub * p, scale)
+                    a, b = resolve_fractions(p_sub, p)
+                    scale = add_unit(b_sub, a * b, scale)
             else:
                 scale = add_unit(b, p, scale)
 
@@ -2295,3 +2320,7 @@ def _condition_arg(value):
 dimensionless_unscaled = CompositeUnit(1, [], [], _error_check=False)
 # Abbreviation of the above, see #1980
 one = dimensionless_unscaled
+
+# Maintain error in old location for backward compatibility
+# TODO: Is this still needed? Should there be a deprecation warning?
+unit_format.fits.UnitScaleError = UnitScaleError
