@@ -13,20 +13,19 @@ from __future__ import (absolute_import, unicode_literals, division,
 from ...extern import six
 
 from numpy.testing.utils import assert_allclose
-from ...tests.helper import raises, pytest
+from ...tests.helper import raises, pytest, catch_warnings
 
 from ... import units as u
 from ...constants import si
 from .. import core
 from .. import format as u_format
 from ..utils import is_effectively_unity
-from ... import wcs
 
 
 def test_unit_grammar():
     def _test_unit_grammar(s, unit):
         print(s)
-        unit2 = u_format.Generic().parse(s)
+        unit2 = u_format.Generic.parse(s)
         assert unit2 == unit
 
     data = [
@@ -39,7 +38,12 @@ def test_unit_grammar():
         (["10+8m"], u.Unit(u.m * 1e8)),
         # This is the VOUnits documentation, but doesn't seem to follow the
         # unity grammar (["3.45 10**(-4)Jy"], 3.45 * 1e-4 * u.Jy)
-        (["sqrt(m)"], u.m ** 0.5)
+        (["sqrt(m)"], u.m ** 0.5),
+        (["dB(mW)", "dB (mW)"], u.DecibelUnit(u.mW)),
+        (["mag"], u.mag),
+        (["mag(ct/s)"], u.MagUnit(u.ct / u.s)),
+        (["dex"], u.dex),
+        (["dex(cm s**-2)", "dex(cm/s2)"], u.DexUnit(u.cm / u.s**2))
     ]
 
     for strings, unit in data:
@@ -47,10 +51,24 @@ def test_unit_grammar():
             yield _test_unit_grammar, s, unit
 
 
+def test_unit_grammar_fail():
+    @raises(ValueError)
+    def _test_unit_grammar_fail(s):
+        u_format.Generic.parse(s)
+
+    data = ['sin( /pixel /s)',
+            'mag(mag)',
+            'dB(dB(mW))',
+            'dex()']
+
+    for s in data:
+        yield _test_unit_grammar_fail, s
+
+
 def test_cds_grammar():
     def _test_cds_grammar(s, unit):
         print(s)
-        unit2 = u_format.CDS().parse(s)
+        unit2 = u_format.CDS.parse(s)
         assert unit2 == unit
 
     data = [
@@ -95,7 +113,7 @@ def test_cds_grammar_fail():
     @raises(ValueError)
     def _test_cds_grammar_fail(s):
         print(s)
-        u_format.CDS().parse(s)
+        u_format.CDS.parse(s)
 
     data = ['0.1 nm',
             'solMass(3/2)',
@@ -108,7 +126,10 @@ def test_cds_grammar_fail():
             '5x8+3m',
             '0.1---',
             '---m',
-            'm---']
+            'm---',
+            'mag(s-1)',
+            'dB(mW)',
+            'dex(cm s-2)']
 
     for s in data:
         yield _test_cds_grammar_fail, s
@@ -117,7 +138,7 @@ def test_cds_grammar_fail():
 def test_ogip_grammar():
     def _test_ogip_grammar(s, unit):
         print(s)
-        unit2 = u_format.OGIP().parse(s)
+        unit2 = u_format.OGIP.parse(s)
         assert unit2 == unit
 
     # These examples are taken from the EXAMPLES section of
@@ -159,12 +180,13 @@ def test_ogip_grammar():
 def test_ogip_grammar_fail():
     @raises(ValueError)
     def _test_ogip_grammar_fail(s):
-        u_format.OGIP().parse(s)
+        u_format.OGIP.parse(s)
 
     data = ['log(photon /m**2 /s /Hz)',
             'sin( /pixel /s)',
             'log(photon /cm**2 /s /Hz) /(sin( /pixel /s))',
-            'log(photon /cm**2 /s /Hz) (sin( /pixel /s))**(-1)']
+            'log(photon /cm**2 /s /Hz) (sin( /pixel /s))**(-1)',
+            'dB(mW)', 'dex(cm/s**2)']
 
     for s in data:
         yield _test_ogip_grammar_fail, s
@@ -193,7 +215,7 @@ def test_roundtrip_vo_unit():
         b = core.Unit(u, format='vounit')
         assert_allclose(b.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
-    x = u_format.VOUnit()
+    x = u_format.VOUnit
     for key, val in x._units.items():
         if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_vo_unit, val, val in (u.mag, u.dB)
@@ -205,7 +227,7 @@ def test_roundtrip_fits():
         a = core.Unit(s, format='fits')
         assert_allclose(a.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
-    for key, val in u_format.Fits()._units.items():
+    for key, val in u_format.Fits._units.items():
         if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_fits, val
 
@@ -220,7 +242,7 @@ def test_roundtrip_cds():
             return
         assert_allclose(b.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
-    x = u_format.CDS()
+    x = u_format.CDS
     for key, val in x._units.items():
         if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_cds, val
@@ -236,22 +258,22 @@ def test_roundtrip_ogip():
             return
         assert_allclose(b.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
-    x = u_format.OGIP()
+    x = u_format.OGIP
     for key, val in x._units.items():
         if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_ogip, val
 
 
 def test_fits_units_available():
-    u_format.Fits()
+    u_format.Fits._units
 
 
 def test_vo_units_available():
-    u_format.VOUnit()
+    u_format.VOUnit._units
 
 
 def test_cds_units_available():
-    u_format.CDS()
+    u_format.CDS._units
 
 
 def test_latex():
@@ -264,6 +286,19 @@ def test_new_style_latex():
     assert "{0:latex}".format(fluxunit) == r'$\mathrm{\frac{erg}{s\,cm^{2}}}$'
 
 
+def test_latex_scale():
+    fluxunit = u.Unit(1.e-24 * u.erg / (u.cm **2 * u.s * u.Hz))
+    latex = r'$\mathrm{1 \times 10^{-24}\,\frac{erg}{Hz\,s\,cm^{2}}}$'
+    assert fluxunit.to_string('latex') == latex
+
+
+def test_latex_inline_scale():
+    fluxunit = u.Unit(1.e-24 * u.erg / (u.cm **2 * u.s * u.Hz))
+    latex_inline = (r'$\mathrm{1 \times 10^{-24}\,erg'
+                    r'\,Hz^{-1}\,s^{-1}\,cm^{-2}}$')
+    assert fluxunit.to_string('latex_inline') == latex_inline
+
+
 def test_format_styles():
     fluxunit = u.erg / (u.cm ** 2 * u.s)
     def _test_format_styles(format_spec, s):
@@ -274,6 +309,7 @@ def test_format_styles():
         ('s', 'erg / (cm2 s)'),
         ('console', '  erg  \n ------\n s cm^2'),
         ('latex', '$\\mathrm{\\frac{erg}{s\\,cm^{2}}}$'),
+        ('latex_inline', '$\\mathrm{erg\\,s^{-1}\\,cm^{-2}}$'),
         ('>20s','       erg / (cm2 s)'),
     ]
 
@@ -350,7 +386,157 @@ def test_scaled_dimensionless():
     assert u.Unit('10+8').to_string('cds') == '10+8'
 
     with pytest.raises(ValueError):
-        u.Unit(0.1).to_string('fits')
+        u.Unit(0.15).to_string('fits')
+
+    assert u.Unit(0.1).to_string('fits') == '10**-1'
 
     with pytest.raises(ValueError):
         u.Unit(0.1).to_string('vounit')
+
+
+def test_deprecated_did_you_mean_units():
+    try:
+        u.Unit('ANGSTROM', format='fits')
+    except ValueError as e:
+        assert 'angstrom (deprecated)' in six.text_type(e)
+        assert 'Angstrom (deprecated)' in six.text_type(e)
+        assert '10**-1 nm' in six.text_type(e)
+
+    with catch_warnings() as w:
+        u.Unit('Angstrom', format='fits')
+    assert len(w) == 1
+    assert '10**-1 nm' in six.text_type(w[0].message)
+
+    try:
+        u.Unit('crab', format='ogip')
+    except ValueError as e:
+        assert 'Crab (deprecated)' in six.text_type(e)
+        assert 'mCrab (deprecated)' in six.text_type(e)
+
+    try:
+        u.Unit('ANGSTROM', format='vounit')
+    except ValueError as e:
+        assert 'angstrom (deprecated)' in six.text_type(e)
+        assert '0.1nm' in six.text_type(e)
+        assert six.text_type(e).count('0.1nm') == 1
+
+    with catch_warnings() as w:
+        u.Unit('angstrom', format='vounit')
+    assert len(w) == 1
+    assert '0.1nm' in six.text_type(w[0].message)
+
+
+def test_fits_function():
+    # Function units cannot be written, so ensure they're not parsed either.
+    @raises(ValueError)
+    def _test_fits_grammar_fail(s):
+        print(s)
+        u_format.Fits().parse(s)
+
+    data = ['mag(ct/s)',
+            'dB(mW)',
+            'dex(cm s**-2)']
+
+    for s in data:
+        yield _test_fits_grammar_fail, s
+
+
+def test_vounit_function():
+    # Function units cannot be written, so ensure they're not parsed either.
+    @raises(ValueError)
+    def _test_vounit_grammar_fail(s):
+        print(s)
+        u_format.VOUnit().parse(s)
+
+    data = ['mag(ct/s)',
+            'dB(mW)',
+            'dex(cm s**-2)']
+
+    for s in data:
+        yield _test_vounit_grammar_fail, s
+
+
+def test_vounit_binary_prefix():
+    u.Unit('KiB', format='vounit') == u.Unit('1024 B')
+    u.Unit('Kibyte', format='vounit') == u.Unit('1024 B')
+    u.Unit('Kibit', format='vounit') == u.Unit('1024 B')
+    with catch_warnings() as w:
+        u.Unit('kibibyte', format='vounit')
+    assert len(w) == 1
+
+
+def test_vounit_unknown():
+    assert u.Unit('unknown', format='vounit') is None
+    assert u.Unit('UNKNOWN', format='vounit') is None
+    assert u.Unit('', format='vounit') is u.dimensionless_unscaled
+
+
+def test_vounit_details():
+    assert u.Unit('Pa', format='vounit') is u.Pascal
+
+    # The da- prefix is not allowed, and the d- prefix is discouraged
+    assert u.dam.to_string('vounit') == '10m'
+    assert u.Unit('dam dag').to_string('vounit') == '100g m'
+
+
+def test_vounit_custom():
+    x = u.Unit("'foo' m", format='vounit')
+    x_vounit = x.to_string('vounit')
+    assert x_vounit == "'foo' m"
+    x_string = x.to_string()
+    assert x_string == "foo m"
+
+    x = u.Unit("m'foo' m", format='vounit')
+    assert x.bases[1]._represents.scale == 0.001
+    x_vounit = x.to_string('vounit')
+    assert x_vounit == "m m'foo'"
+    x_string = x.to_string()
+    assert x_string == 'm mfoo'
+
+
+def test_vounit_implicit_custom():
+    x = u.Unit("furlong/week", format="vounit")
+    assert x.bases[0]._represents.scale == 1e-15
+    assert x.bases[0]._represents.bases[0].name == 'urlong'
+
+
+def test_fits_scale_factor():
+    with pytest.raises(ValueError):
+        x = u.Unit('1000 erg/s/cm**2/Angstrom', format='fits')
+
+    with pytest.raises(ValueError):
+        x = u.Unit('12 erg/s/cm**2/Angstrom', format='fits')
+
+    x = u.Unit('10+2 erg/s/cm**2/Angstrom', format='fits')
+    assert x == 100 * (u.erg / u.s / u.cm ** 2 / u.Angstrom)
+    assert x.to_string(format='fits') == '10**2 Angstrom-1 cm-2 erg s-1'
+
+    x = u.Unit('10**(-20) erg/s/cm**2/Angstrom', format='fits')
+    assert x == 10**(-20) * (u.erg / u.s / u.cm ** 2 / u.Angstrom)
+    assert x.to_string(format='fits') == '10**-20 Angstrom-1 cm-2 erg s-1'
+
+    x = u.Unit('10**-20 erg/s/cm**2/Angstrom', format='fits')
+    assert x == 10**(-20) * (u.erg / u.s / u.cm ** 2 / u.Angstrom)
+    assert x.to_string(format='fits') == '10**-20 Angstrom-1 cm-2 erg s-1'
+
+    x = u.Unit('10^(-20) erg/s/cm**2/Angstrom', format='fits')
+    assert x == 10**(-20) * (u.erg / u.s / u.cm ** 2 / u.Angstrom)
+    assert x.to_string(format='fits') == '10**-20 Angstrom-1 cm-2 erg s-1'
+
+    x = u.Unit('10^-20 erg/s/cm**2/Angstrom', format='fits')
+    assert x == 10**(-20) * (u.erg / u.s / u.cm ** 2 / u.Angstrom)
+    assert x.to_string(format='fits') == '10**-20 Angstrom-1 cm-2 erg s-1'
+
+    x = u.Unit('10-20 erg/s/cm**2/Angstrom', format='fits')
+    assert x == 10**(-20) * (u.erg / u.s / u.cm ** 2 / u.Angstrom)
+    assert x.to_string(format='fits') == '10**-20 Angstrom-1 cm-2 erg s-1'
+
+    x = u.Unit('10**(-20)*erg/s/cm**2/Angstrom', format='fits')
+    assert x == 10**(-20) * (u.erg / u.s / u.cm ** 2 / u.Angstrom)
+
+    x = u.Unit(1.2 * u.erg)
+    with pytest.raises(ValueError):
+        x.to_string(format='fits')
+
+    x = u.Unit(100.0 * u.erg)
+    assert x.to_string(format='fits') == '10**2 erg'
