@@ -18,6 +18,7 @@ from numpy.testing import (assert_allclose, assert_array_equal,
 from ...tests.helper import raises, pytest
 from ...utils import isiterable, minversion
 from ...utils.compat import NUMPY_LT_1_7
+from ...utils.compat.fractions import Fraction
 from ... import units as u
 from ...units.quantity import _UNIT_NOT_INITIALISED
 from ...extern.six.moves import xrange
@@ -466,8 +467,12 @@ class TestQuantityOperations(object):
                 long(q1)
             assert exc.value.args[0] == converter_err_msg
 
+        # We used to test `q1 * ['a', 'b', 'c'] here, but that that worked
+        # at all was a really odd confluence of bugs.  Since it doesn't work
+        # in numpy >=1.10 any more, just go directly for `__index__` (which
+        # makes the test more similar to the `int`, `long`, etc., tests).
         with pytest.raises(TypeError) as exc:
-            q1 * ['a', 'b', 'c']
+            q1.__index__()
         assert exc.value.args[0] == index_err_msg
 
         # dimensionless but scaled is OK, however
@@ -480,7 +485,7 @@ class TestQuantityOperations(object):
             assert long(q2) == long(q2.to(u.dimensionless_unscaled).value)
 
         with pytest.raises(TypeError) as exc:
-            q2 * ['a', 'b', 'c']
+            q2.__index__()
         assert exc.value.args[0] == index_err_msg
 
         # dimensionless unscaled is OK, though for index needs to be int
@@ -492,7 +497,7 @@ class TestQuantityOperations(object):
             assert long(q3) == 1
 
         with pytest.raises(TypeError) as exc:
-            q1 * ['a', 'b', 'c']
+            q1.__index__()
         assert exc.value.args[0] == index_err_msg
 
         # integer dimensionless unscaled is good for all
@@ -503,7 +508,7 @@ class TestQuantityOperations(object):
         if six.PY2:
             assert long(q4) == 2
 
-        assert q4 * ['a', 'b', 'c'] == ['a', 'b', 'c', 'a', 'b', 'c']
+        assert q4.__index__() == 2
 
         # but arrays are not OK
         q5 = u.Quantity([1, 2], u.m)
@@ -521,7 +526,7 @@ class TestQuantityOperations(object):
             assert exc.value.args[0] == converter_err_msg
 
         with pytest.raises(TypeError) as exc:
-            q5 * ['a', 'b', 'c']
+            q5.__index__()
         assert exc.value.args[0] == index_err_msg
 
     def test_array_converters(self):
@@ -824,7 +829,7 @@ def test_arrays():
     assert qkpc0.value == a[0].item()
     assert qkpc0.unit == qkpc.unit
     assert isinstance(qkpc0, u.Quantity)
-    assert not qkpc0.isscalar
+    assert qkpc0.isscalar
     qkpcx = qkpc['x']
     assert np.all(qkpcx.value == a['x'])
     assert qkpcx.unit == qkpc.unit
@@ -1085,6 +1090,14 @@ def test_quantity_tuple_power():
     (5.0 * u.m) ** (1, 2)
 
 
+def test_quantity_fraction_power():
+    q = (25.0 * u.m**2) ** Fraction(1, 2)
+    assert q.value == 5.
+    assert q.unit == u.m
+    # Regression check to ensure we didn't create an object type by raising
+    # the value of the quantity to a Fraction. [#3922]
+    assert q.dtype.kind == 'f'
+
 def test_inherit_docstrings():
     assert u.Quantity.argmax.__doc__ == np.ndarray.argmax.__doc__
 
