@@ -688,3 +688,39 @@ def test_fractional_rounding_errors_simple():
     assert isinstance(x.powers[0], Fraction)
     assert x.powers[0].numerator == 6
     assert x.powers[0].denominator == 5
+
+
+def test_enable_unit_groupings():
+    from ...units import cds
+
+    with cds.enable():
+        assert cds.geoMass in u.kg.find_equivalent_units()
+
+    from ...units import imperial
+    with imperial.enable():
+        assert imperial.inch in u.m.find_equivalent_units()
+
+
+def test_unit_summary_prefixes():
+    """
+    Test for a few units that the unit summary table correctly reports
+    whether or not that unit supports prefixes.
+
+    Regression test for https://github.com/astropy/astropy/issues/3835
+    """
+
+    from .. import astrophys
+
+    for summary in utils._iter_unit_summary(astrophys.__dict__):
+        unit, _, _, _, prefixes = summary
+
+        if unit.name == 'lyr':
+            assert prefixes
+        elif unit.name == 'pc':
+            assert prefixes
+        elif unit.name == 'barn':
+            assert prefixes
+        elif unit.name == 'cycle':
+            assert not prefixes
+        elif unit.name == 'vox':
+            assert prefixes
