@@ -12,5 +12,9 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 
-from .compat.odict import OrderedDict
+from .codegen import *
+from .decorators import *
+from .introspection import *
 from .misc import *
+
+from .compat.odict import OrderedDict
