@@ -7,4 +7,4 @@ in all supported NumPy versions.  See docs/utils/numpy.rst for details.
 
 from __future__ import division, absolute_import, unicode_literals
 
-from .lib.stride_tricks import broadcast_arrays
+from .lib.stride_tricks import broadcast_arrays, broadcast_to
