@@ -25,19 +25,33 @@ try:
 except ImportError:
     _CAN_RESIZE_TERMINAL = False
 
+try:
+    from IPython import get_ipython
+except ImportError:
+    pass
 try:
     get_ipython()
 except NameError:
     OutStream = None
     IPythonIOStream = None
 else:
+    from IPython import version_info
+    ipython_major_version = version_info[0]
+
     try:
-        from IPython.zmq.iostream import OutStream
+        from ipykernel.iostream import OutStream
     except ImportError:
         try:
-            from IPython.kernel.zmq.iostream import OutStream
+            from IPython.zmq.iostream import OutStream
         except ImportError:
-            OutStream = None
+            if ipython_major_version < 4:
+                try:
+                    from IPython.kernel.zmq.iostream import OutStream
+                except ImportError:
+                    OutStream = None
+            else:
+                OutStream = None
+
 
     if OutStream is not None:
         from IPython.utils import io as ipyio
@@ -61,6 +75,7 @@ from ..config import ConfigAlias
 from ..extern import six
 from ..extern.six.moves import range
 from .. import conf
+from .. import units as u
 
 from .decorators import deprecated
 from .misc import isiterable
@@ -431,6 +446,9 @@ def human_file_size(size):
     size : str
         A human-friendly representation of the size of the file
     """
+    if hasattr(size, 'unit'):
+        size = size.to(u.byte).value
+
     suffixes = ' kMGTPEZY'
     if size == 0:
         num_scale = 0
@@ -490,7 +508,10 @@ class ProgressBar(six.Iterator):
         if ipython_widget:
             # Import only if ipython_widget, i.e., widget in IPython
             # notebook
-            from IPython.html import widgets
+            if ipython_major_version < 4:
+                from IPython.html import widgets
+            else:
+                from ipywidgets import widgets
             from IPython.display import display
 
         if file is None:
@@ -631,10 +652,14 @@ class ProgressBar(six.Iterator):
         # if none exists.
         if not hasattr(self, '_widget'):
             # Import only if an IPython widget, i.e., widget in iPython NB
-            from IPython.html import widgets
+            if ipython_major_version < 4:
+                from IPython.html import widgets
+                self._widget = widgets.FloatProgressWidget()
+            else:
+                from ipywidgets import widgets
+                self._widget = widgets.FloatProgress()
             from IPython.display import display
 
-            self._widget = widgets.FloatProgressWidget()
             display(self._widget)
             self._widget.value = 0
 
@@ -648,7 +673,7 @@ class ProgressBar(six.Iterator):
         pass
 
     @classmethod
-    def map(cls, function, items, multiprocess=False, file=None):
+    def map(cls, function, items, multiprocess=False, file=None, step=100):
         """
         Does a `map` operation while displaying a progress bar with
         percentage complete.
@@ -678,6 +703,13 @@ class ProgressBar(six.Iterator):
             `sys.stdout`.  If `file` is not a tty (as determined by
             calling its `isatty` member, if any), the scrollbar will
             be completely silent.
+
+        step : int, optional
+            Update the progress bar at least every *step* steps (default: 100).
+            If ``multiprocess`` is `True`, this will affect the size
+            of the chunks of ``items`` that are submitted as separate tasks
+            to the process pool.  A large step size may make the job
+            complete faster if ``items`` is very long.
         """
 
         results = []
@@ -686,17 +718,17 @@ class ProgressBar(six.Iterator):
             file = _get_stdout()
 
         with cls(len(items), file=file) as bar:
-            step_size = max(200, bar._bar_length)
-            steps = max(int(float(len(items)) / step_size), 1)
+            default_step = max(int(float(len(items)) / bar._bar_length), 1)
+            chunksize = min(default_step, step)
             if not multiprocess:
                 for i, item in enumerate(items):
                     results.append(function(item))
-                    if (i % steps) == 0:
+                    if (i % chunksize) == 0:
                         bar.update(i)
             else:
                 p = multiprocessing.Pool()
                 for i, result in enumerate(
-                    p.imap_unordered(function, items, steps)):
+                    p.imap_unordered(function, items, chunksize=chunksize)):
                     bar.update(i)
                     results.append(result)
                 p.close()
