@@ -42,8 +42,6 @@ NONCRIT_WARNINGS = ConfigAlias(
     '0.4', 'NONCRIT_WARNINGS', 'noncritical_warnings',
     'astropy.vo.validator.validate', 'astropy.vo.validator')
 
-_OUT_ROOT = None  # Set by check_conesearch_sites()
-
 
 @timefunc(1)
 def check_conesearch_sites(destdir=os.curdir, verbose=True, parallel=True,
@@ -96,7 +94,6 @@ def check_conesearch_sites(destdir=os.curdir, verbose=True, parallel=True,
 
     """
     from . import conf
-    global _OUT_ROOT
 
     if url_list == 'default':
         url_list = conf.conesearch_urls
@@ -109,10 +106,10 @@ def check_conesearch_sites(destdir=os.curdir, verbose=True, parallel=True,
         os.mkdir(destdir)
 
     # Output dir created by votable.validator
-    _OUT_ROOT = os.path.join(destdir, 'results')
+    out_dir = os.path.join(destdir, 'results')
 
-    if not os.path.exists(_OUT_ROOT):
-        os.mkdir(_OUT_ROOT)
+    if not os.path.exists(out_dir):
+        os.mkdir(out_dir)
 
     # Output files
     db_file = OrderedDict()
@@ -188,20 +185,20 @@ def check_conesearch_sites(destdir=os.curdir, verbose=True, parallel=True,
             warnings.warn(warn_str, AstropyUserWarning)
 
     all_urls = list(key_lookup_by_url)
+    timeout = data.conf.remote_timeout
+    map_args = [(out_dir, url, timeout) for url in all_urls]
 
     # Validate URLs
     if parallel:
-        mp_list = []
         pool = multiprocessing.Pool()
-        mp_proc = pool.map_async(_do_validation, all_urls,
-                                 callback=mp_list.append)
-        mp_proc.wait()
-        if len(mp_list) < 1:  # pragma: no cover
+        try:
+            mp_list = pool.map(_do_validation, map_args)
+        except Exception as exc:  # pragma: no cover
             raise ValidationMultiprocessingError(
-                'Multiprocessing pool callback returned empty list.')
-        mp_list = mp_list[0]
+                'An exception occurred during parallel processing '
+                'of validation results: {0}'.format(exc))
     else:
-        mp_list = [_do_validation(cur_url) for cur_url in all_urls]
+        mp_list = map(_do_validation, map_args)
 
     # Categorize validation results
     for r in mp_list:
@@ -212,16 +209,15 @@ def check_conesearch_sites(destdir=os.curdir, verbose=True, parallel=True,
         js_tree[db_key].add_catalog(cat_key, cur_cat)
 
     # Write to HTML
-    html_subsets = result.get_result_subsets(mp_list, _OUT_ROOT)
-    html.write_index(html_subsets, all_urls, _OUT_ROOT)
+    html_subsets = result.get_result_subsets(mp_list, out_dir)
+    html.write_index(html_subsets, all_urls, out_dir)
     if parallel:
-        html_subindex_args = [(html_subset, uniq_rows)
+        html_subindex_args = [(out_dir, html_subset, uniq_rows)
                               for html_subset in html_subsets]
-        mp_proc = pool.map_async(_html_subindex, html_subindex_args)
-        mp_proc.wait()
+        pool.map(_html_subindex, html_subindex_args)
     else:
         for html_subset in html_subsets:
-            _html_subindex((html_subset, uniq_rows))
+            _html_subindex((out_dir, html_subset, uniq_rows))
 
     # Write to JSON
     n = {}
@@ -242,11 +238,14 @@ def check_conesearch_sites(destdir=os.curdir, verbose=True, parallel=True,
             'No good sites available for Cone Search.', AstropyUserWarning)
 
 
-def _do_validation(url):
+def _do_validation(args):
     """Validation for multiprocessing support."""
+
+    root, url, timeout = args
+
     votable.table.reset_vo_warnings()
 
-    r = result.Result(url, root=_OUT_ROOT, timeout=data.conf.remote_timeout)
+    r = result.Result(url, root=root, timeout=timeout)
     r.validate_vo()
 
     _categorize_result(r)
@@ -332,8 +331,8 @@ def _categorize_result(r):
 
 def _html_subindex(args):
     """HTML writer for multiprocessing support."""
-    subset, total = args
-    html.write_index_table(_OUT_ROOT, *subset, total=total)
+    out_dir, subset, total = args
+    html.write_index_table(out_dir, *subset, total=total)
 
 
 def _copy_r_to_cat(r, cat):
