@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.25 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2014, Mark Calabretta
+  WCSLIB 5.10 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2015, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,38 +22,40 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: spc.h,v 4.25 2014/12/14 14:29:36 mcalabre Exp $
+  $Id: spc.h,v 5.10 2015/10/09 08:19:15 mcalabre Exp $
 *=============================================================================
 *
-* WCSLIB 4.25 - C routines that implement the spectral coordinate systems
-* recognized by the FITS World Coordinate System (WCS) standard.  Refer to
-*
-*   "Representations of world coordinates in FITS",
-*   Greisen, E.W., & Calabretta, M.R. 2002, A&A, 395, 1061 (Paper I)
-*
-*   "Representations of spectral coordinates in FITS",
-*   Greisen, E.W., Calabretta, M.R., Valdes, F.G., & Allen, S.L.
-*   2006, A&A, 446, 747 (Paper III)
-*
-* Refer to the README file provided with WCSLIB for an overview of the
-* library.
+* WCSLIB 5.10 - C routines that implement the FITS World Coordinate System
+* (WCS) standard.  Refer to the README file provided with WCSLIB for an
+* overview of the library.
 *
 *
 * Summary of the spc routines
 * ---------------------------
-* These routines implement the part of the FITS WCS standard that deals with
-* spectral coordinates.  They define methods to be used for computing spectral
-* world coordinates from intermediate world coordinates (a linear
-* transformation of image pixel coordinates), and vice versa.  They are based
-* on the spcprm struct which contains all information needed for the
-* computations.  The struct contains some members that must be set by the
-* user, and others that are maintained by these routines, somewhat like a
-* C++ class but with no encapsulation.
+* Routines in this suite implement the part of the FITS World Coordinate
+* System (WCS) standard that deals with spectral coordinates, as described in
+*
+=   "Representations of world coordinates in FITS",
+=   Greisen, E.W., & Calabretta, M.R. 2002, A&A, 395, 1061 (WCS Paper I)
+=
+=   "Representations of spectral coordinates in FITS",
+=   Greisen, E.W., Calabretta, M.R., Valdes, F.G., & Allen, S.L.
+=   2006, A&A, 446, 747 (WCS Paper III)
+*
+* These routines define methods to be used for computing spectral world
+* coordinates from intermediate world coordinates (a linear transformation
+* of image pixel coordinates), and vice versa.  They are based on the spcprm
+* struct which contains all information needed for the computations.  The
+* struct contains some members that must be set by the user, and others that
+* are maintained by these routines, somewhat like a C++ class but with no
+* encapsulation.
 *
 * Routine spcini() is provided to initialize the spcprm struct with default
 * values, spcfree() reclaims any memory that may have been allocated to store
 * an error message, and spcprt() prints its contents.
 *
+* spcperr() prints the error message(s) (if any) stored in a spcprm struct.
+*
 * A setup routine, spcset(), computes intermediate values in the spcprm struct
 * from parameters in it that were supplied by the user.  The struct always
 * needs to be set up by spcset() but it need not be called explicitly - refer
@@ -216,6 +218,25 @@
 *                         1: Null spcprm pointer passed.
 *
 *
+* spcperr() - Print error messages from a spcprm struct
+* -----------------------------------------------------
+* spcperr() prints the error message(s) (if any) stored in a spcprm struct.
+* If there are no errors then nothing is printed.  It uses wcserr_prt(), q.v.
+*
+* Given:
+*   spc       const struct spcprm*
+*                       Spectral transformation parameters.
+*
+*   prefix    const char *
+*                       If non-NULL, each output line will be prefixed with
+*                       this string.
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Success.
+*                         1: Null spcprm pointer passed.
+*
+*
 * spcset() - Setup routine for the spcprm struct
 * ----------------------------------------------
 * spcset() sets up a spcprm struct according to information supplied within
@@ -719,7 +740,7 @@
 *     (An unused variable inserted for alignment purposes only.)
 *
 *   struct wcserr *err
-*     (Returned) If enabled, when an error status is returned this structure
+*     (Returned) If enabled, when an error status is returned, this struct
 *     contains detailed information about the error, see wcserr_enable().
 *
 *   void *padding2
@@ -751,7 +772,6 @@
 #define WCSLIB_SPC
 
 #include "spx.h"
-#include "wcserr.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -831,6 +851,8 @@ int spcfree(struct spcprm *spc);
 
 int spcprt(const struct spcprm *spc);
 
+int spcperr(const struct spcprm *spc, const char *prefix);
+
 int spcset(struct spcprm *spc);
 
 int spcx2s(struct spcprm *spc, int nx, int sx, int sspec,
@@ -869,10 +891,10 @@ int spctyp(const char ctype[9], char stype[], char scode[], char sname[],
            char units[], char *ptype, char *xtype, int *restreq);
 int spcspx(const char ctypeS[9], double crvalS, double restfrq,
            double restwav, char *ptype, char *xtype, int *restreq,
-	   double *crvalX, double *dXdS);
+           double *crvalX, double *dXdS);
 int spcxps(const char ctypeS[9], double crvalX, double restfrq,
            double restwav, char *ptype, char *xtype, int *restreq,
-	   double *crvalS, double *dSdX);
+           double *crvalS, double *dSdX);
 int spctrn(const char ctypeS1[9], double crvalS1, double cdeltS1,
            double restfrq, double restwav, char ctypeS2[9], double *crvalS2,
            double *cdeltS2);
