@@ -13,10 +13,12 @@ from datetime import datetime
 
 import numpy as np
 from numpy.testing import (
-    assert_allclose, assert_array_almost_equal, assert_array_almost_equal_nulp)
+    assert_allclose, assert_array_almost_equal, assert_array_almost_equal_nulp,
+    assert_array_equal)
 
 from ...tests.helper import raises, catch_warnings, pytest
 from ... import wcs
+from .. import _wcs
 from ...utils.data import (
     get_pkg_data_filenames, get_pkg_data_contents, get_pkg_data_filename)
 from ...utils.misc import NumpyRNGContext
@@ -44,7 +46,7 @@ def test_maps():
         assert_array_almost_equal(pix, [[97, 97]], decimal=0)
 
     # get the list of the hdr files that we want to test
-    hdr_file_list = list(get_pkg_data_filenames("maps", "*.hdr"))
+    hdr_file_list = list(get_pkg_data_filenames("maps", pattern="*.hdr"))
 
     # actually perform a test for each one
     for filename in hdr_file_list:
@@ -89,7 +91,7 @@ def test_spectra():
         assert len(all_wcs) == 9
 
     # get the list of the hdr files that we want to test
-    hdr_file_list = list(get_pkg_data_filenames("spectra", "*.hdr"))
+    hdr_file_list = list(get_pkg_data_filenames("spectra", pattern="*.hdr"))
 
     # actually perform a test for each one
     for filename in hdr_file_list:
@@ -338,6 +340,15 @@ def test_warning_about_defunct_keywords():
     for item in w:
         assert 'PCi_ja' in str(item.message)
 
+    # Make sure the warnings come out every time...
+
+    with catch_warnings(wcs.FITSFixedWarning) as w:
+        run()
+
+    assert len(w) == 4
+    for item in w:
+        assert 'PCi_ja' in str(item.message)
+
 
 @raises(wcs.FITSFixedWarning)
 def test_warning_about_defunct_keywords_exception():
@@ -356,7 +367,13 @@ def test_to_header_string():
     WCSAXES =                    2 / Number of coordinate axes                      CRPIX1  =                  0.0 / Pixel coordinate of reference point            CRPIX2  =                  0.0 / Pixel coordinate of reference point            CDELT1  =                  1.0 / Coordinate increment at reference point        CDELT2  =                  1.0 / Coordinate increment at reference point        CRVAL1  =                  0.0 / Coordinate value at reference point            CRVAL2  =                  0.0 / Coordinate value at reference point            LATPOLE =                 90.0 / [deg] Native latitude of celestial pole        END"""
 
     w = wcs.WCS()
-    assert w.to_header_string().strip() == header_string.strip()
+    h0 = fits.Header.fromstring(w.to_header_string().strip())
+    if 'COMMENT' in h0:
+        del h0['COMMENT']
+    if '' in h0:
+        del h0['']
+    h1 = fits.Header.fromstring(header_string.strip())
+    assert dict(h0) == dict(h1)
 
 
 def test_to_fits():
@@ -368,6 +385,28 @@ def test_to_fits():
     assert header_string == wfits[0].header[-8:]
 
 
+def test_to_header_warning():
+    fits_name = get_pkg_data_filename('data/sip.fits')
+    x = wcs.WCS(fits_name)
+    with catch_warnings() as w:
+        x.to_header()
+    assert len(w) == 1
+    assert 'A_ORDER' in str(w[0])
+
+
+def test_no_comments_in_header():
+    w = wcs.WCS()
+    header = w.to_header()
+    assert w.wcs.alt not in header
+    assert 'COMMENT' + w.wcs.alt.strip() not in header
+    assert 'COMMENT' not in header
+    wkey = 'P'
+    header = w.to_header(key=wkey)
+    assert wkey not in header
+    assert 'COMMENT' not in header
+    assert 'COMMENT' + w.wcs.alt.strip() not in header
+
+
 @raises(wcs.InvalidTransformError)
 def test_find_all_wcs_crash():
     """
@@ -385,8 +424,13 @@ def test_validate():
     with catch_warnings():
         results = wcs.validate(get_pkg_data_filename("data/validate.fits"))
         results_txt = repr(results)
-        with open(get_pkg_data_filename("data/validate.txt"), "r") as fd:
-            assert set([x.strip() for x in fd.readlines()]) == set([
+        if wcs._wcs.__version__[0] == '5':
+            filename = 'data/validate.5.0.txt'
+        else:
+            filename = 'data/validate.txt'
+        with open(get_pkg_data_filename(filename), "r") as fd:
+            lines = fd.readlines()
+            assert set([x.strip() for x in lines]) == set([
                 x.strip() for x in results_txt.splitlines()])
 
 
@@ -712,3 +756,111 @@ def test_no_iteration():
     with pytest.raises(TypeError) as exc:
         iter(w)
     assert exc.value.args[0] == "'NewWCS' object is not iterable"
+
+
+@pytest.mark.skipif('_wcs.__version__[0] < "5"',
+                    reason="TPV only works with wcslib 5.x or later")
+def test_sip_tpv_agreement():
+    sip_header = get_pkg_data_contents(
+        os.path.join("data", "siponly.hdr"), encoding='binary')
+    tpv_header = get_pkg_data_contents(
+        os.path.join("data", "tpvonly.hdr"), encoding='binary')
+
+    w_sip = wcs.WCS(sip_header)
+    w_tpv = wcs.WCS(tpv_header)
+
+    assert_array_almost_equal(
+        w_sip.all_pix2world([w_sip.wcs.crpix], 1),
+        w_tpv.all_pix2world([w_tpv.wcs.crpix], 1))
+
+    w_sip2 = wcs.WCS(w_sip.to_header())
+    w_tpv2 = wcs.WCS(w_tpv.to_header())
+
+    assert_array_almost_equal(
+        w_sip.all_pix2world([w_sip.wcs.crpix], 1),
+        w_sip2.all_pix2world([w_sip.wcs.crpix], 1))
+    assert_array_almost_equal(
+        w_tpv.all_pix2world([w_sip.wcs.crpix], 1),
+        w_tpv2.all_pix2world([w_sip.wcs.crpix], 1))
+    assert_array_almost_equal(
+        w_sip2.all_pix2world([w_sip.wcs.crpix], 1),
+        w_tpv2.all_pix2world([w_tpv.wcs.crpix], 1))
+
+
+@pytest.mark.skipif('_wcs.__version__[0] < "5"',
+                    reason="TPV only works with wcslib 5.x or later")
+def test_tpv_copy():
+    # See #3904
+
+    tpv_header = get_pkg_data_contents(
+        os.path.join("data", "tpvonly.hdr"), encoding='binary')
+
+    w_tpv = wcs.WCS(tpv_header)
+
+    ra, dec = w_tpv.wcs_pix2world([0, 100, 200], [0, -100, 200], 0)
+    assert ra[0] != ra[1] and ra[1] != ra[2]
+    assert dec[0] != dec[1] and dec[1] != dec[2]
+
+
+def test_hst_wcs():
+    path = get_pkg_data_filename("data/dist_lookup.fits.gz")
+
+    hdulist = fits.open(path)
+    # wcslib will complain about the distortion parameters if they
+    # weren't correctly deleted from the header
+    w = wcs.WCS(hdulist[1].header, hdulist)
+
+    # Exercise the main transformation functions, mainly just for
+    # coverage
+    w.p4_pix2foc([0, 100, 200], [0, -100, 200], 0)
+    w.det2im([0, 100, 200], [0, -100, 200], 0)
+
+    w.cpdis1 = w.cpdis1
+    w.cpdis2 = w.cpdis2
+
+    w.det2im1 = w.det2im1
+    w.det2im2 = w.det2im2
+
+    w.sip = w.sip
+
+    w.cpdis1.cdelt = w.cpdis1.cdelt
+    w.cpdis1.crpix = w.cpdis1.crpix
+    w.cpdis1.crval = w.cpdis1.crval
+    w.cpdis1.data = w.cpdis1.data
+
+    assert w.sip.a_order == 4
+    assert w.sip.b_order == 4
+    assert w.sip.ap_order == 0
+    assert w.sip.bp_order == 0
+    assert_array_equal(w.sip.crpix, [2048., 1024.])
+    wcs.WCS(hdulist[1].header, hdulist)
+
+
+def test_list_naxis():
+    path = get_pkg_data_filename("data/dist_lookup.fits.gz")
+
+    hdulist = fits.open(path)
+    # wcslib will complain about the distortion parameters if they
+    # weren't correctly deleted from the header
+    w = wcs.WCS(hdulist[1].header, hdulist, naxis=['celestial'])
+    assert w.naxis == 2
+    assert w.wcs.naxis == 2
+
+    path = get_pkg_data_filename("maps/1904-66_SIN.hdr")
+    with open(path, 'rb') as fd:
+        content = fd.read()
+    w = wcs.WCS(content, naxis=['celestial'])
+    assert w.naxis == 2
+    assert w.wcs.naxis == 2
+
+    w = wcs.WCS(content, naxis=['spectral'])
+    assert w.naxis == 0
+    assert w.wcs.naxis == 0
+
+
+def test_sip_broken():
+    # This header caused wcslib to segfault because it has a SIP
+    # specification in a non-default keyword
+    hdr = get_pkg_data_contents("data/sip-broken.hdr")
+
+    w = wcs.WCS(hdr)
