@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.19 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2013, Mark Calabretta
+  WCSLIB 5.10 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2015, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,24 +22,25 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: fitshdr.h,v 4.19 2013/09/29 14:17:51 mcalabre Exp $
+  $Id: fitshdr.h,v 5.10 2015/10/09 08:19:15 mcalabre Exp $
 *=============================================================================
 *
-* The Flexible Image Transport System (FITS), a data format widely used in
-* astronomy for data interchange and archive, is described in
-*
-*   "Definition of The Flexible Image Transport System (FITS)",
-*   Hanisch, R.J., Farris, A., Greisen, E.W., et al. 2001, A&A, 376, 359
-*
-* which formalizes NOST 100-2.0, a document produced by the NASA/Science
-* Office of Standards and Technology, see http://fits.gsfc.nasa.gov.
-*
-* Refer to the README file provided with WCSLIB for an overview of the
-* library.
+* WCSLIB 5.10 - C routines that implement the FITS World Coordinate System
+* (WCS) standard.  Refer to the README file provided with WCSLIB for an
+* overview of the library.
 *
 *
 * Summary of the fitshdr routines
 * -------------------------------
+* The Flexible Image Transport System (FITS), is a data format widely used in
+* astronomy for data interchange and archive.  It is described in
+*
+=   "Definition of the Flexible Image Transport System (FITS), version 3.0",
+=   Pence, W.D., Chiappetti, L., Page, C.G., Shaw, R.A., & Stobie, E. 2010,
+=   A&A, 524, A42 - http://dx.doi.org/10.1051/0004-6361/201015362
+*
+* See also http://fits.gsfc.nasa.gov
+*
 * fitshdr() is a generic FITS header parser provided to handle keyrecords that
 * are ignored by the WCS header parsers, wcspih() and wcsbth().  Typically the
 * latter may be set to remove WCS keyrecords from a header leaving fitshdr()
@@ -73,7 +74,7 @@
 *                       keyids[] provides a convienient way of indexing them.
 *                       The fitskeyid struct contains three members;
 *                       fitskeyid::name must be set by the user while
-*                       fitskeyid::count and fitskeyid::name are returned by
+*                       fitskeyid::count and fitskeyid::idx are returned by
 *                       fitshdr().  All matched keywords will have their
 *                       fitskey::keyno member negated.
 *
@@ -133,7 +134,7 @@
 *
 *      f: Sect. 5.2.3 offers no comment on the size of an integer keyvalue
 *         except indirectly in limiting it to 70 digits.  The parser will
-*         translates an integer keyvalue to a 32-bit signed integer if it
+*         translate an integer keyvalue to a 32-bit signed integer if it
 *         lies in the range -2147483648 to +2147483647, otherwise it
 *         interprets it as a 64-bit signed integer if possible, or else a
 *         "very long" integer (see fitskey::type).
@@ -249,7 +250,7 @@
 *
 *   int type
 *     (Returned) Keyvalue data type:
-*       - 0: No keyvalue.
+*       - 0: No keyvalue (both the value and type are undefined).
 *       - 1: Logical, represented as int.
 *       - 2: 32-bit signed integer.
 *       - 3: 64-bit signed integer (see below).
