@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.23 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2014, Mark Calabretta
+  WCSLIB 5.10 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2015, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,7 +22,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: spx.c,v 4.23 2014/05/11 04:09:38 mcalabre Exp $
+  $Id: spx.c,v 5.10 2015/10/09 08:19:15 mcalabre Exp $
 *===========================================================================*/
 
 #include <math.h>
@@ -319,6 +319,20 @@ struct spxprm *spx;
   return 0;
 }
 
+/*--------------------------------------------------------------------------*/
+
+int spxperr(const struct spxprm *spx, const char *prefix)
+
+{
+  if (spx == 0x0) return SPXERR_NULL_POINTER;
+
+  if (spx->err) {
+    wcserr_prt(spx->err, prefix);
+  }
+
+  return 0;
+}
+
 
 /*============================================================================
 *   Conversions between frequency and vacuum wavelength.
