@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.23 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2014, Mark Calabretta
+  WCSLIB 5.10 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2015, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,38 +22,48 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: wcs.h,v 4.23 2014/05/11 04:09:38 mcalabre Exp $
+  $Id: wcs.h,v 5.10 2015/10/09 08:19:15 mcalabre Exp $
 *=============================================================================
 *
-* WCSLIB 4.23 - C routines that implement the FITS World Coordinate System
-* (WCS) standard.  Refer to
-*
-*   "Representations of world coordinates in FITS",
-*   Greisen, E.W., & Calabretta, M.R. 2002, A&A, 395, 1061 (Paper I)
-*
-*   "Representations of celestial coordinates in FITS",
-*   Calabretta, M.R., & Greisen, E.W. 2002, A&A, 395, 1077 (Paper II)
-*
-*   "Representations of spectral coordinates in FITS",
-*   Greisen, E.W., Calabretta, M.R., Valdes, F.G., & Allen, S.L.
-*   2006, A&A, 446, 747 (Paper III)
-*
-* Refer to the README file provided with WCSLIB for an overview of the
-* library.
+* WCSLIB 5.10 - C routines that implement the FITS World Coordinate System
+* (WCS) standard.  Refer to the README file provided with WCSLIB for an
+* overview of the library.
 *
 *
 * Summary of the wcs routines
 * ---------------------------
-* These routines implement the FITS World Coordinate System (WCS) standard
-* which defines methods to be used for computing world coordinates from image
-* pixel coordinates, and vice versa.  They are based on the wcsprm struct
-* which contains all information needed for the computations.  The struct
-* contains some members that must be set by the user, and others that are
-* maintained by these routines, somewhat like a C++ class but with no
-* encapsulation.
+* Routines in this suite implement the FITS World Coordinate System (WCS)
+* standard which defines methods to be used for computing world coordinates
+* from image pixel coordinates, and vice versa.  The standard, and proposed
+* extensions for handling distortions, are described in
 *
-* Three routines, wcsini(), wcssub(), and wcsfree() are provided to manage the
-* wcsprm struct and another, wcsprt(), to prints its contents.  Refer to the
+=   "Representations of world coordinates in FITS",
+=   Greisen, E.W., & Calabretta, M.R. 2002, A&A, 395, 1061 (WCS Paper I)
+=
+=   "Representations of celestial coordinates in FITS",
+=   Calabretta, M.R., & Greisen, E.W. 2002, A&A, 395, 1077 (WCS Paper II)
+=
+=   "Representations of spectral coordinates in FITS",
+=   Greisen, E.W., Calabretta, M.R., Valdes, F.G., & Allen, S.L.
+=   2006, A&A, 446, 747 (WCS Paper III)
+=
+=   "Representations of distortions in FITS world coordinate systems",
+=   Calabretta, M.R. et al. (WCS Paper IV, draft dated 2004/04/22),
+=   available from http://www.atnf.csiro.au/people/Mark.Calabretta
+=
+=   "Mapping on the HEALPix grid",
+=   Calabretta, M.R., & Roukema, B.F. 2007, MNRAS, 381, 865 (WCS Paper V)
+=
+=   "Representing the 'Butterfly' Projection in FITS -- Projection Code XPH",
+=   Calabretta, M.R., & Lowe, S.R. 2013, PASA, 30, e050 (WCS Paper VI)
+*
+* These routines are based on the wcsprm struct which contains all information
+* needed for the computations.  The struct contains some members that must be
+* set by the user, and others that are maintained by these routines, somewhat
+* like a C++ class but with no encapsulation.
+*
+* wcsnpv(), wcsnps(), wcsini(), wcssub(), and wcsfree() are provided to manage
+* the wcsprm struct and another, wcsprt(), prints its contents.  Refer to the
 * description of the wcsprm struct for an explanation of the anticipated usage
 * of these routines.  wcscopy(), which does a deep copy of one wcsprm struct
 * to another, is defined as a preprocessor macro function that invokes
@@ -79,6 +89,8 @@
 * wcssptr() translates the spectral axis in a wcsprm struct.  For example, a
 * 'FREQ' axis may be translated into 'ZOPT-F2W' and vice versa.
 *
+* wcslib_version() returns the WCSLIB version number.
+*
 * Quadcube projections:
 * ---------------------
 *   The quadcube projections (TSC, CSC, QSC) may be represented in FITS in
@@ -107,6 +119,36 @@
 *   routines.
 *
 *
+* wcsnpv() - Memory allocation for PVi_ma
+* ---------------------------------------
+* wcsnpv() changes the value of NPVMAX (default 64).  This global variable
+* controls the number of pvcard structs, for holding PVi_ma keyvalues, that
+* wcsini() should allocate space for.
+*
+* PLEASE NOTE: This function is not thread-safe.
+*
+* Given:
+*   n         int       Value of NPVMAX; ignored if < 0.
+*
+* Function return value:
+*             int       Current value of NPVMAX.
+*
+*
+* wcsnps() - Memory allocation for PSi_ma
+* ---------------------------------------
+* wcsnps() changes the values of NPSMAX (default 8).  This global variable
+* controls the number of pscard structs, for holding PSi_ma keyvalues, that
+* wcsini() should allocate space for.
+*
+* PLEASE NOTE: This function is not thread-safe.
+*
+* Given:
+*   n         int       Value of NPSMAX; ignored if < 0.
+*
+* Function return value:
+*             int       Current value of NPSMAX.
+*
+*
 * wcsini() - Default constructor for the wcsprm struct
 * ----------------------------------------------------
 * wcsini() optionally allocates memory for arrays in a wcsprm struct and sets
@@ -153,49 +195,23 @@
 *                       wcsprm::err if enabled, see wcserr_enable().
 *
 *
-* wcsnpv() - Memory allocation for PVi_ma
-* ---------------------------------------
-* wcsnpv() changes the value of NPVMAX (default 64).  This global variable
-* controls the number of PVi_ma keywords that wcsini() should allocate space
-* for.
-*
-* PLEASE NOTE: This function is not thread-safe.
-*
-* Given:
-*   n         int       Value of NPVMAX; ignored if < 0.
-*
-* Function return value:
-*             int       Current value of NPVMAX.
-*
-*
-* wcsnps() - Memory allocation for PSi_ma
-* ---------------------------------------
-* wcsnps() changes the values of NPSMAX (default 8).  This global variable
-* controls the number of PSi_ma keywords that wcsini() should allocate space
-* for.
-*
-* PLEASE NOTE: This function is not thread-safe.
-*
-* Given:
-*   n         int       Value of NPSMAX; ignored if < 0.
-*
-* Function return value:
-*             int       Current value of NPSMAX.
-*
-*
 * wcssub() - Subimage extraction routine for the wcsprm struct
 * ------------------------------------------------------------
 * wcssub() extracts the coordinate description for a subimage from a wcsprm
 * struct.  It does a deep copy, using wcsini() to allocate memory for its
 * arrays if required.  Only the "information to be provided" part of the
-* struct is extracted; a call to wcsset() is required to set up the remainder.
+* struct is extracted.  Consequently, wcsset() need not have been, and won't
+* be invoked on the struct from which the subimage is extracted.  A call to
+* wcsset() is required to set up the subimage struct.
 *
 * The world coordinate system of the subimage must be separable in the sense
 * that the world coordinates at any point in the subimage must depend only on
 * the pixel coordinates of the axes extracted.  In practice, this means that
-* the PCi_ja matrix of the original image must not contain non-zero
-* off-diagonal terms that associate any of the subimage axes with any of the
-* non-subimage axes.
+* the linear transformation matrix of the original image must not contain
+* non-zero off-diagonal terms that associate any of the subimage axes with any
+* of the non-subimage axes.  Likewise, if any distortions are associated with
+* the subimage axes, they must not depend on any of the axes that are not
+* being extracted.
 *
 * Note that while the required elements of the tabprm array are extracted, the
 * wtbarr array is not.  (Thus it is not appropriate to call wcssub() after
@@ -334,6 +350,11 @@
 *                           of the same map projection but may not align on
 *                           the same grid map.  Overrides WCSCOMPARE_TILING.
 *
+*   tol       double    Tolerance for comparison of floating-point values.
+*                       For example, for tol == 1e-6, all floating-point
+*                       values in the structs must be equal to the first 6
+*                       decimal places.  A value of 0 implies exact equality.
+*
 *   wcs1      const struct wcsprm*
 *                       The first wcsprm struct to compare.
 *
@@ -761,16 +782,49 @@
 *                       wcsprm::err if enabled, see wcserr_enable().
 *
 *
+* wcslib_version() - WCSLIB version number
+* ----------------------------------------
+* wcslib_version() returns the WCSLIB version number.
+*
+* The major version number changes when the ABI changes or when the license
+* conditions change.  ABI changes typically result from a change to the
+* contents of one of the structs.  The major version number is used to
+* distinguish between incompatible versions of the sharable library.
+*
+* The minor version number changes with new functionality or bug fixes that do
+* not involve a change in the ABI.
+*
+* The auxiliary version number (which is often absent) signals changes to the
+* documentation, test suite, build procedures, or any other change that does
+* not affect the compiled library.
+*
+* Returned:
+*   vers[3]   int[3]    The broken-down version number:
+*                         0: Major version number.
+*                         1: Minor version number.
+*                         2: Auxiliary version number (zero if absent).
+*                       May be given as a null pointer if not required.
+*
+* Function return value:
+*             char*     A null-terminated, statically allocated string
+*                       containing the version number in the usual form, i.e.
+*                       "<major>.<minor>.<auxiliary>".
+*
+*
 * wcsprm struct - Coordinate transformation parameters
 * ----------------------------------------------------
 * The wcsprm struct contains information required to transform world
 * coordinates.  It consists of certain members that must be set by the user
 * ("given") and others that are set by the WCSLIB routines ("returned").
-* Some of the former are not actually required for transforming coordinates.
-* These are described as "auxiliary"; the struct simply provides a place to
-* store them, though they may be used by wcshdo() in constructing a FITS
-* header from a wcsprm struct.  Some of the returned values are supplied for
-* informational purposes and others are for internal use only as indicated.
+* While the addresses of the arrays themselves may be set by wcsini() if it
+* (optionally) allocates memory, their contents must be set by the user.
+*
+* Some parameters that are given are not actually required for transforming
+* coordinates.  These are described as "auxiliary"; the struct simply provides
+* a place to store them, though they may be used by wcshdo() in constructing a
+* FITS header from a wcsprm struct.  Some of the returned values are supplied
+* for informational purposes and others are for internal use only as
+* indicated.
 *
 * In practice, it is expected that a WCS parser would scan the FITS header to
 * determine the number of coordinate axes.  It would then use wcsini() to
@@ -803,7 +857,11 @@
 *       - wcsprm::ps,
 *       - wcsprm::cd,
 *       - wcsprm::crota,
-*       - wcsprm::altlin.
+*       - wcsprm::altlin,
+*       - wcsprm::ntab,
+*       - wcsprm::nwtb,
+*       - wcsprm::tab,
+*       - wcsprm::wtb.
 *
 *     This signals the initialization routine, wcsset(), to recompute the
 *     returned members of the celprm struct.  celset() will reset flag to
@@ -932,9 +990,8 @@
 *     otherwise it must be set by the user.  See also wcsnpv().
 *
 *   struct pvcard *pv
-*     (Given or returned) Address of the first element of an array of length
-*     npvmax of pvcard structs.  Set by wcsini() if it allocates memory for
-*     pv[], otherwise it must be set by the user.  See also wcsnpv().
+*     (Given) Address of the first element of an array of length npvmax of
+*     pvcard structs.
 *
 *     As a FITS header parser encounters each PVi_ma keyword it should load it
 *     into a pvcard struct in the array and increment npv.  wcsset()
@@ -955,9 +1012,8 @@
 *     otherwise it must be set by the user.  See also wcsnps().
 *
 *   struct pscard *ps
-*     (Given or returned) Address of the first element of an array of length
-*     npsmax of pscard structs.  Set by wcsini() if it allocates memory for
-*     ps[], otherwise it must be set by the user.  See also wcsnps().
+*     (Given) Address of the first element of an array of length npsmax of
+*     pscard structs.
 *
 *     As a FITS header parser encounters each PSi_ma keyword it should load it
 *     into a pscard struct in the array and increment nps.  wcsset()
@@ -1031,6 +1087,9 @@
 *   int velref
 *     (Given) AIPS velocity code VELREF, refer to spcaips().
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::velref is changed.
+*
 *   char alt[4]
 *     (Given, auxiliary) Character code for alternate coordinate descriptions
 *     (i.e. the 'a' in keyword names such as CTYPEia).  This is blank for the
@@ -1040,6 +1099,9 @@
 *     An array of four characters is provided for alignment purposes, only the
 *     first is used.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::alt is changed.
+*
 *   int colnum
 *     (Given, auxiliary) Where the coordinate representation is associated
 *     with an image-array column in a FITS binary table, this variable may be
@@ -1047,6 +1109,9 @@
 *
 *     It should be set to zero for an image header or pixel list.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::colnum is changed.
+*
 *   int *colax
 *     (Given, auxiliary) Address of the first element of an array of int
 *     recording the column numbers for each axis in a pixel list.
@@ -1054,6 +1119,9 @@
 *     The array elements should be set to zero for an image header or image
 *     array in a binary table.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::colax is changed.
+*
 *   char (*cname)[72]
 *     (Given, auxiliary) The address of the first element of an array of
 *     char[72] containing the coordinate axis names, CNAMEia.
@@ -1062,69 +1130,126 @@
 *     keyword, being limited to 68 characters, plus the null-terminating
 *     character.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::cname is changed.
+*
 *   double *crder
 *     (Given, auxiliary) Address of the first element of an array of double
 *     recording the random error in the coordinate value, CRDERia.
+*
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::crder is changed.
+*
 *   double *csyer
 *     (Given, auxiliary) Address of the first element of an array of double
 *     recording the systematic error in the coordinate value, CSYERia.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::csyer is changed.
+*
 *   char dateavg[72]
 *     (Given, auxiliary) The date of a representative mid-point of the
 *     observation in ISO format, yyyy-mm-ddThh:mm:ss.
+*
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::dateavg is changed.
+*
 *   char dateobs[72]
 *     (Given, auxiliary) The date of the start of the observation unless
 *     otherwise explained in the comment field of the DATE-OBS keyword, in
 *     ISO format, yyyy-mm-ddThh:mm:ss.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::dateobs is changed.
+*
 *   double equinox
 *     (Given, auxiliary) The equinox associated with dynamical equatorial or
 *     ecliptic coordinate systems, EQUINOXa (or EPOCH in older headers).  Not
 *     applicable to ICRS equatorial or ecliptic coordinates.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::equinox is changed.
+*
 *   double mjdavg
 *     (Given, auxiliary) Modified Julian Date (MJD = JD - 2400000.5), MJD-AVG,
 *     corresponding to DATE-AVG.
+*
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::mjdavg is changed.
+*
 *   double mjdobs
 *     (Given, auxiliary) Modified Julian Date (MJD = JD - 2400000.5), MJD-OBS,
 *     corresponding to DATE-OBS.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::mjdobs is changed.
+*
 *   double obsgeo[3]
 *     (Given, auxiliary) Location of the observer in a standard terrestrial
 *     reference frame, OBSGEO-X, OBSGEO-Y, OBSGEO-Z [m].
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::obsgeo is changed.
+*
 *   char radesys[72]
 *     (Given, auxiliary) The equatorial or ecliptic coordinate system type,
 *     RADESYSa.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::radesys is changed.
+*
 *   char specsys[72]
 *     (Given, auxiliary) Spectral reference frame (standard of rest),
-*     SPECSYSa, and ...
+*     SPECSYSa.
+*
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::specsys is changed.
+*
 *   char ssysobs[72]
-*     (Given, auxiliary) ... the actual frame in which there is no
+*     (Given, auxiliary) The spectral reference frame in which there is no
 *     differential variation in the spectral coordinate across the
 *     field-of-view, SSYSOBSa.
+*
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::ssysobs is changed.
+*
 *   double velosys
 *     (Given, auxiliary) The relative radial velocity [m/s] between the
 *     observer and the selected standard of rest in the direction of the
 *     celestial reference coordinate, VELOSYSa.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::velosys is changed.
+*
 *   double zsource
-*     (Given, auxiliary) The redshift, ZSOURCEa, of the source, and ...
+*     (Given, auxiliary) The redshift, ZSOURCEa, of the source.
+*
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::zsource is changed.
+*
 *   char ssyssrc[72]
-*     (Given, auxiliary) ... the spectral reference frame (standard of rest)
-*     in which this was measured, SSYSSRCa.
+*     (Given, auxiliary) The spectral reference frame (standard of rest),
+*     SSYSSRCa, in which wcsprm::zsource was measured.
+*
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::ssyssrc is changed.
 *
 *   double velangl
 *     (Given, auxiliary) The angle [deg] that should be used to decompose an
 *     observed velocity into radial and transverse components.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::velangl is changed.
+*
 *   char wcsname[72]
 *     (Given, auxiliary) The name given to the coordinate representation,
 *     WCSNAMEa.  This variable accomodates the longest allowed string-valued
 *     FITS keyword, being limited to 68 characters, plus the null-terminating
 *     character.
 *
+*     It is not necessary to reset the wcsprm struct (via wcsset()) when
+*     wcsprm::wcsname is changed.
+*
 *   int ntab
 *     (Given) See wcsprm::tab.
 *
@@ -1227,7 +1352,7 @@
 *     prologue to spc.h).
 *
 *   struct wcserr *err
-*     (Returned) If enabled, when an error status is returned this struct
+*     (Returned) If enabled, when an error status is returned, this struct
 *     contains detailed information about the error, see wcserr_enable().
 *
 *   void *m_padding
@@ -1270,23 +1395,6 @@
 *     (For internal use only.)
 *
 *
-* pscard struct - Store for PSi_ma keyrecords
-* -------------------------------------------
-* The pscard struct is used to pass the parsed contents of PSi_ma keyrecords
-* to wcsset() via the wcsprm struct.
-*
-* All members of this struct are to be set by the user.
-*
-*   int i
-*     (Given) Axis number (1-relative), as in the FITS PSi_ma keyword.
-*
-*   int m
-*     (Given) Parameter number (non-negative), as in the FITS PSi_ma keyword.
-*
-*   char value[72]
-*     (Given) Parameter value.
-*
-*
 * pvcard struct - Store for PVi_ma keyrecords
 * -------------------------------------------
 * The pvcard struct is used to pass the parsed contents of PVi_ma keyrecords
@@ -1305,6 +1413,23 @@
 *     (Given) Parameter value.
 *
 *
+* pscard struct - Store for PSi_ma keyrecords
+* -------------------------------------------
+* The pscard struct is used to pass the parsed contents of PSi_ma keyrecords
+* to wcsset() via the wcsprm struct.
+*
+* All members of this struct are to be set by the user.
+*
+*   int i
+*     (Given) Axis number (1-relative), as in the FITS PSi_ma keyword.
+*
+*   int m
+*     (Given) Parameter number (non-negative), as in the FITS PSi_ma keyword.
+*
+*   char value[72]
+*     (Given) Parameter value.
+*
+*
 * wtbarr struct - Extraction of coordinate lookup tables from BINTABLE
 * --------------------------------------------------------------------
 * Function wcstab(), which is invoked automatically by wcspih(), sets up an
@@ -1368,8 +1493,6 @@
 #include "lin.h"
 #include "cel.h"
 #include "spc.h"
-#include "tab.h"
-#include "wcserr.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -1397,12 +1520,12 @@ enum wcs_errmsg_enum {
   WCSERR_SINGULAR_MTX    =  3,	/* Linear transformation matrix is
 				   singular. */
   WCSERR_BAD_CTYPE       =  4,	/* Inconsistent or unrecognized coordinate
-				   axis types. */
+				   axis type. */
   WCSERR_BAD_PARAM       =  5,	/* Invalid parameter value. */
-  WCSERR_BAD_COORD_TRANS =  6,	/* Invalid coordinate transformation
-				   parameters. */
+  WCSERR_BAD_COORD_TRANS =  6,	/* Unrecognized coordinate transformation
+				   parameter. */
   WCSERR_ILL_COORD_TRANS =  7,	/* Ill-conditioned coordinate transformation
-				   parameters. */
+				   parameter. */
   WCSERR_BAD_PIX         =  8,	/* One or more of the pixel coordinates were
 				   invalid. */
   WCSERR_BAD_WORLD       =  9,	/* One or more of the world coordinates were
@@ -1423,6 +1546,9 @@ struct pvcard {
   double value;			/* Parameter value.                         */
 };
 
+/* Size of the pvcard struct in int units, used by the Fortran wrappers. */
+#define PVLEN (sizeof(struct pvcard)/sizeof(int))
+
 /* Struct used for storing PSi_ma keywords. */
 struct pscard {
   int i;			/* Axis number, as in PSi_ma (1-relative).  */
@@ -1430,6 +1556,9 @@ struct pscard {
   char value[72];		/* Parameter value.                         */
 };
 
+/* Size of the pscard struct in int units, used by the Fortran wrappers. */
+#define PSLEN (sizeof(struct pscard)/sizeof(int))
+
 				/* For extracting wcstab arrays.  Matches   */
 				/* the wtbarr typedef defined in CFITSIO    */
 				/* header fitsio.h.                         */
@@ -1535,7 +1664,7 @@ struct wcsprm {
   struct celprm cel;		/* Celestial transformation parameters.     */
   struct spcprm spc;		/* Spectral  transformation parameters.     */
 
-  /* Error handling                                                         */
+  /* Error handling, if enabled.                                            */
   /*------------------------------------------------------------------------*/
   struct wcserr *err;
 
@@ -1568,8 +1697,8 @@ int wcsini(int alloc, int naxis, struct wcsprm *wcs);
 int wcssub(int alloc, const struct wcsprm *wcssrc, int *nsub, int axes[],
            struct wcsprm *wcsdst);
 
-int wcscompare(int cmp, const struct wcsprm *wcs1, const struct wcsprm *wcs2,
-               int *equal);
+int wcscompare(int cmp, double tol, const struct wcsprm *wcs1,
+               const struct wcsprm *wcs2, int *equal);
 
 int wcsfree(struct wcsprm *wcs);
 
@@ -1595,6 +1724,8 @@ int wcsmix(struct wcsprm *wcs, int mixpix, int mixcel, const double vspan[],
 
 int wcssptr(struct wcsprm *wcs, int *i, char ctype[9]);
 
+const char* wcslib_version(int vers[3]);
+
 /* Defined mainly for backwards compatibility, use wcssub() instead. */
 #define wcscopy(alloc, wcssrc, wcsdst) wcssub(alloc, wcssrc, 0x0, 0x0, wcsdst)
 
